package cn.wise.sc.cement.business.controller;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;
import cn.wise.sc.cement.business.entity.NonStandardValue;
import cn.wise.sc.cement.business.model.BaseResponse;
import cn.wise.sc.cement.business.model.PageQuery;
import cn.wise.sc.cement.business.service.INonStandardValueService;
import cn.wise.sc.cement.business.util.PageUtil;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.util.Date;
import java.util.List;

/**
 * <p>
 * 前端控制器
 * </p>
 *
 * @author wlb
 * @since 2020-09-23
 */
@Api(tags = "非标产值-非标产值")
@RestController
@RequestMapping("/tcdri/nonestandard_value")
public class NonStandardValueController {
	private static final Logger log = LoggerFactory.getLogger("NoneStandardValueController");

	@Autowired
	private INonStandardValueService inonStandardValueService;

	@ApiOperation("产值统计-非标产值")
	@GetMapping("/user_id")
	public BaseResponse nonValue(PageQuery pageQuery, String start, String end, Integer groups,String name) {
		Date startParse = null;
		if (StrUtil.isNotBlank(start)) {
			startParse = DateUtil.parse(start);
		}
		Date endParse = null;
		if (StrUtil.isNotBlank(end)) {
			endParse = DateUtil.parse(end);
		}
		//将list拆分成分页
		BaseResponse<List<NonStandardValue>> baseResponse = inonStandardValueService.nonValue(startParse, endParse, groups,name);
		List<NonStandardValue> data = baseResponse.getData();
		if (data.size() != 0) {
			Page<NonStandardValue> rts = PageUtil.listConvertToPage(data, pageQuery);
			return BaseResponse.okData(rts);
		}
		return baseResponse;
	}

	@ApiOperation(value = "产值统计-非标产值-产看明细")
	@GetMapping("/getPage")
	public BaseResponse getPage(PageQuery pageQuery, Integer userId, String name,String start, String end) {
		try {
			Date startParse = null;
			if (StrUtil.isNotBlank(start)) {
				startParse = DateUtil.parse(start);
			}
			Date endParse = null;
			if (StrUtil.isNotBlank(end)) {
				endParse = DateUtil.parse(end);
			}
			return inonStandardValueService.getPage(pageQuery, userId, name,startParse,endParse);
		} catch (Exception e) {
			log.debug("非标产值分页列表{}", e);
		}
		return BaseResponse.errorMsg("失败!");
	}

	@ApiOperation(value = "获取所有已通过的非标产值")
	@GetMapping("/getList")
	public BaseResponse getList(String start, String end, Integer groups,String name) {
		Date startParse = null;
		if (StrUtil.isNotBlank(start)) {
			startParse = DateUtil.parse(start);
		}
		Date endParse = null;
		if (StrUtil.isNotBlank(end)) {
			endParse = DateUtil.parse(end);
		}
		try {
			return inonStandardValueService.getList(startParse, endParse, groups,name);
		} catch (Exception e) {
			log.debug("获取所有已通过的非标产值{}", e);
		}
		return BaseResponse.errorMsg("失败!");
	}

	@ApiOperation("导出-产值统计-非标产值信息列表")
	@PostMapping("/exportList")
	public void exportList(String start, String end, Integer groups, String name, String filename,  HttpServletResponse response) {
		try {
			Date startParse = null;
			if (StrUtil.isNotBlank(start)) {
				startParse = DateUtil.parse(start);
			}
			Date endParse = null;
			if (StrUtil.isNotBlank(end)) {
				endParse = DateUtil.parse(end);
			}
			inonStandardValueService.exportList(startParse, endParse, groups, name, filename, response);
		} catch (Exception e) {
			log.debug("导出-产值统计-非标产值信息列表{}", e);
		}
	}


	@ApiOperation("导出-产值统计-非标产值信息列表")
	@PostMapping("/exportDetailList")
	public void exportDetailList(Date start, Date end, Integer userId, String name, String filename,HttpServletResponse response) {
		try {
			inonStandardValueService.exportDetailList(start, end, userId, name, filename, response);
		} catch (Exception e) {
			log.debug("导出-产值统计-非标产值信息列表{}", e);
		}
	}


}
