package cn.wise.sc.cement.business.controller;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateField;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;
import cn.wise.sc.cement.business.entity.NormProduction;
import cn.wise.sc.cement.business.model.vo.*;
import cn.wise.sc.cement.business.model.BaseResponse;
import cn.wise.sc.cement.business.model.LoginUser;
import cn.wise.sc.cement.business.model.PageQuery;
import cn.wise.sc.cement.business.service.INonStandardValueService;
import cn.wise.sc.cement.business.service.INormProductionService;
import cn.wise.sc.cement.business.service.IPrecipriceService;
import cn.wise.sc.cement.business.service.ISysUserService;
import cn.wise.sc.cement.business.util.PageUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.util.Assert;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletResponse;
import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 前端控制器
 * </p>
 *
 * @author ztw
 * @since 2020-09-23
 */
@RestController
@Api(tags = "人员产值统计-标准产值")
@RequestMapping("/business/norm/production")
public class NormProductionController {

    private static final Logger log = LoggerFactory.getLogger("NormProductionController");

    final
    INormProductionService iNormProductionService;
    final
    IPrecipriceService iPrecipriceService;
    final
    INonStandardValueService inonStandardValueService;
    final
    ISysUserService iSysUserService;

    public NormProductionController(INormProductionService iNormProductionService, IPrecipriceService iPrecipriceService, INonStandardValueService inonStandardValueService, ISysUserService iSysUserService) {
        this.iNormProductionService = iNormProductionService;
        this.iPrecipriceService = iPrecipriceService;
        this.inonStandardValueService = inonStandardValueService;
        this.iSysUserService = iSysUserService;
    }

    //*************************************产值设置***********************************************

    @PostMapping("/create")
    @ApiOperation("新增标准产值配置")
    public BaseResponse<NormProduction> addObj(@RequestBody NormProduction normProduction) {
        return iNormProductionService.addObj(normProduction);
    }


    @PutMapping("/edit")
    @ApiOperation("编辑标准产值配置")
    public BaseResponse<Boolean> edit(@RequestBody NormProduction production) {
        return iNormProductionService.edit(production);
    }


    @GetMapping("/page")
    @ApiOperation("标准产值配置分页列表")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(name = "type", value = "类型  0:处理项 1:检测组 2:检测项", paramType = "query", dataType = "Integer")
    })
    public BaseResponse<IPage<NormProductionVo>> page(PageQuery pageQuery, Integer type) {
        return iNormProductionService.getPage(pageQuery, type);
    }


    @GetMapping("/{id}")
    @ApiOperation("根据id获取标准产值配置")
    public BaseResponse<NormProduction> getById(@PathVariable("id") Integer id) {
        if (id == null || id <= 0) {
            return BaseResponse.errorMsg("id不能为空!");
        }
        NormProduction rts = iNormProductionService.getById(id);

        if (rts != null) {
            return BaseResponse.okData(rts);
        } else {
            return BaseResponse.errorMsg("没找着符合要求的信息!");
        }
    }

    @PutMapping("/status/{id}")
    @ApiOperation("启用/禁用")
    public BaseResponse<Integer> activeOrForbidden(@PathVariable("id") Integer id) {

        if (id == null || id <= 0) {
            return BaseResponse.errorMsg("编号id不能为空!");
        }
        return iNormProductionService.activeOrForbidden(id);
    }

    //**************************************产值统计***********************************************************

    @ApiOperation("产值统计-标准产值-分页列表")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(name = "start", value = "开始日期", paramType = "query", dataType = "String"),
            @ApiImplicitParam(name = "end", value = "结束日期", paramType = "query", dataType = "String"),
            @ApiImplicitParam(name = "userName", value = "姓名", paramType = "query", dataType = "String")
    })
    @GetMapping("/statistics")
    public BaseResponse statistics(PageQuery pageQuery, String start, String end, String userName) {

        try {
            return iPrecipriceService.normProductionStatistics(pageQuery, start, end, userName);
        } catch (Exception e) {
            log.debug("产值统计-标准产值-分页列表{}", e);
        }
        return BaseResponse.errorMsg("失败!");
    }

    @ApiOperation("产值统计-标准产值-列表")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(name = "startDate", value = "开始日期", paramType = "query", dataType = "String"),
            @ApiImplicitParam(name = "endDate", value = "结束日期", paramType = "query", dataType = "String"),
            @ApiImplicitParam(name = "userName", value = "姓名", paramType = "query", dataType = "String")
    })
    @GetMapping("/statisticsList")
    public BaseResponse statisticsList(String startDate, String endDate, String userName) {

        try {
            return iPrecipriceService.statisticsList(startDate, endDate, userName);
        } catch (Exception e) {
            log.debug("产值统计-标准产值-分页列表{}", e);
        }
        return BaseResponse.errorMsg("失败!");
    }

    @GetMapping("/statistics/detail")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(name = "startDate", value = "开始日期", paramType = "query", dataType = "String"),
            @ApiImplicitParam(name = "endDate", value = "结束日期", paramType = "query", dataType = "String"),
            @ApiImplicitParam(name = "userId", value = "用户id（必传）", paramType = "query", dataType = "Integer")
    })
    @ApiOperation("产值统计-标准产值-个人详情")
    public BaseResponse normProductionDetails(PageQuery pageQuery, String startDate, String endDate, Integer userId) {
	    try {
		    return iPrecipriceService.normProductionDetails(pageQuery, startDate, endDate, userId);
	    } catch (Exception e) {
		    log.debug("产值统计-标准产值-个人详情{}", e);
	    }
	    return BaseResponse.errorMsg("失败!");
    }


    @GetMapping("/statistics/specificDetail")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(name = "startDate", value = "开始日期", paramType = "query", dataType = "String"),
            @ApiImplicitParam(name = "endDate", value = "结束日期", paramType = "query", dataType = "String"),
            @ApiImplicitParam(name = "userId", value = "用户id（必传）", paramType = "query", dataType = "Integer"),
            @ApiImplicitParam(name = "type", value = "类型  0:处理项 1:检测组 2:检测项", paramType = "query", dataType = "Integer"),
            @ApiImplicitParam(name = "objId", value = "对象id（非必传）", paramType = "query", dataType = "Integer")
    })
    @ApiOperation("产值统计-标准产值-个人详情-检测项目详情")
    public BaseResponse normProductionSpecificDetails(PageQuery pageQuery,
                                                      String startDate, String endDate, Integer userId, Integer type, Integer objId) {
        try {
            return iPrecipriceService.normProductionSpecificDetails(pageQuery,
                    startDate, endDate, userId, type, objId);
        } catch (Exception e) {
            log.debug("产值统计-标准产值-个人详情-检测项目详情{}", e);
        }
        return BaseResponse.errorMsg("失败!");
    }


    @GetMapping("/total/production")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(name = "start", value = "开始日期", paramType = "query", dataType = "String"),
            @ApiImplicitParam(name = "end", value = "结束日期", paramType = "query", dataType = "String"),
            @ApiImplicitParam(name = "name", value = "姓名", paramType = "query", dataType = "String"),
            @ApiImplicitParam(name = "groupId", value = "工作组id", paramType = "query", dataType = "Integer")
    })
    @ApiOperation("产值统计-总产值统计")
    public BaseResponse<Page<ProductionVo>> production(PageQuery pageQuery, String name, String start, String end, Integer groupId) {
        Long startTime = null;
        Long endTime = null;
        if (StrUtil.isNotBlank(start) && StrUtil.isNotBlank(end)) {
            startTime = DateUtil.parseDate(start).getTime();
            endTime = DateUtil.parseDate(end).offsetNew(DateField.DAY_OF_MONTH, 1).getTime();
        }
        List<ProductionVo> rts = iPrecipriceService.production(name, startTime, endTime, groupId);
        rts.stream().forEach(arg -> {
            arg.setId(Integer.valueOf(arg.getUserId()));
            arg.setProductionTotalValue(arg.getProductionValue() + arg.getNonProductionValue());
        });
        return BaseResponse.okData(PageUtil.listConvertToPage(rts.stream().sorted(Comparator.comparing(ProductionVo::getId)).collect(Collectors.toList()), pageQuery));
    }



//****************************导出*****************************************************
    @ApiOperation("导出-产值统计-标准产值-列表")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(name = "startDate", value = "开始日期", paramType = "query", dataType = "String"),
            @ApiImplicitParam(name = "endDate", value = "结束日期", paramType = "query", dataType = "String"),
            @ApiImplicitParam(name = "userName", value = "姓名", paramType = "query", dataType = "String")
    })
    @PostMapping("/export/statistics")
    public void exportStatisticsList(String startDate, String endDate, String userName, String fileName, HttpServletResponse response) {

        try {
            iPrecipriceService.exportStatisticsList(startDate, endDate, userName, fileName, response);
        } catch (Exception e) {
            log.debug("导出-产值统计-标准产值-列表{}", e);
        }
    }


    @ApiOperation("导出-产值统计-标准产值-个人详情")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(name = "startDate", value = "开始日期", paramType = "query", dataType = "String"),
            @ApiImplicitParam(name = "endDate", value = "结束日期", paramType = "query", dataType = "String"),
            @ApiImplicitParam(name = "userId", value = "用户id（必传）", paramType = "query", dataType = "Integer")
    })
    @PostMapping("/export/statisticsDetail")
    public void exportStatisticsDetail(String startDate, String endDate, Integer userId, String fileName, HttpServletResponse response) {

        try {
            iPrecipriceService.exportStatisticsDetail(startDate, endDate, userId, fileName, response);
        } catch (Exception e) {
            log.debug("导出-产值统计-标准产值-个人详情{}", e);
        }
    }

    @ApiOperation("导出-产值统计-产值统计-总产值统计")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(name = "start", value = "开始日期", paramType = "query", dataType = "String"),
            @ApiImplicitParam(name = "end", value = "结束日期", paramType = "query", dataType = "String"),
            @ApiImplicitParam(name = "name", value = "姓名", paramType = "query", dataType = "String"),
            @ApiImplicitParam(name = "groupId", value = "工作组id", paramType = "query", dataType = "Integer")
    })
    @PostMapping("/export/total/production")
    public void exportProduction(String name, String start, String end, Integer groupId, String fileName, HttpServletResponse response) {
        Long startTime = null;
        Long endTime = null;
        if (StrUtil.isNotBlank(start) && StrUtil.isNotBlank(end)) {
            startTime = DateUtil.parseDate(start).getTime();
            endTime = DateUtil.parseDate(end).offsetNew(DateField.DAY_OF_MONTH, 1).getTime();
        }
        try {
            iPrecipriceService.exportProduction(name, startTime, endTime, groupId, fileName, response);
        } catch (Exception e) {
            log.debug("导出-产值统计-产值统计-总产值统计{}", e);
        }
    }

}

