package cn.wise.sc.cement.business.controller;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;
import cn.wise.sc.cement.business.entity.EntityEnclosure;
import cn.wise.sc.cement.business.model.*;
import cn.wise.sc.cement.business.enumation.FileExt;
import cn.wise.sc.cement.business.model.query.SampleHandleEnclosureQuery;
import cn.wise.sc.cement.business.model.vo.*;
import cn.wise.sc.cement.business.service.IEntityEnclosureService;
import cn.wise.sc.cement.business.service.IEntrustService;
import cn.wise.sc.cement.business.util.WordUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.Assert;
import org.springframework.web.bind.annotation.CrossOrigin;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.time.LocalDateTime;
import java.util.*;

/**
 * @description: 报告管理
 * @author: qh
 * @create: 2020-09-25 14:37
 **/
@Api(tags = "报告管理")
@Slf4j
@CrossOrigin
@RestController
@RequestMapping("/business/report")
public class ReportController {

    final
    IEntrustService iEntrustService;
    @Autowired
    IEntityEnclosureService iEntityEnclosureService;

    public ReportController(IEntrustService iEntrustService) {
        this.iEntrustService = iEntrustService;

    }

    @ApiOperation(value = "报告分页列表")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(name = "startDate", value = "开始日期", paramType = "query", dataType = "String"),
            @ApiImplicitParam(name = "endDate", value = "结束日期", paramType = "query", dataType = "String"),
            @ApiImplicitParam(name = "clientId", value = "委托单位id", paramType = "query", dataType = "Integer"),
            @ApiImplicitParam(name = "projectName", value = "项目名称", paramType = "query", dataType = "String"),
            @ApiImplicitParam(name = "projectCode", value = "项目编号", paramType = "query", dataType = "String")
    })
    @GetMapping("/getPage")
    public BaseResponse<IPage<EntrustVo>> page(PageQuery pageQuery, String startDate, String endDate,
                                               Integer clientId, String projectName, String projectCode) {
        return iEntrustService.getReportPage(pageQuery, startDate, endDate, clientId, projectName, projectCode);
    }

    @PostMapping("/{entrustId}")
    @ApiOperation("导出报告封面和首页")
    public void getReportDetail(@PathVariable("entrustId") Integer entrustId,
                                HttpServletResponse response) throws IOException {

        //首页及封面导出
        ReportDetailVo rts = iEntrustService.getReportDetail(entrustId);
        if (rts == null) {
            response.getWriter().write("需要的数据不全!");
            return;
        }
        Map<String, Object> beanParams = new HashMap<>(22);
        beanParams.put("sendName", rts.getSendName());
        beanParams.put("sender", rts.getSender());
        beanParams.put("sendDate", rts.getSendDate());
        beanParams.put("sampleNum", rts.getSampleNum());
        beanParams.put("sampleNames", set2String(rts.getSampleNames()));
        beanParams.put("teamNames", set2String(rts.getTeamNames()));
        beanParams.put("projectName", rts.getProjectName());
        beanParams.put("printDate", rts.getPrintDate());

        beanParams.put("year", DateUtil.year(DateUtil.date()) + "");
        beanParams.put("month", DateUtil.month(DateUtil.date()) + 1);
        beanParams.put("day", DateUtil.dayOfMonth(DateUtil.date()));
        beanParams.put("reportNo", StrUtil.isEmpty(rts.getReportNo()) ? "" : rts.getReportNo());

        beanParams.put("firstMethodName", getFirstSet(rts.getMethodNames()));
        beanParams.put("firstEquipment", getFirstSet(rts.getEquipmentNames()));
        beanParams.put("methodNames", moveFirst(rts.getMethodNames()));
        beanParams.put("equipmentNames", moveFirst(rts.getEquipmentNames()));

        //十元素1导出
        List<TenElementReport> list1 = iEntrustService.getTen1List(entrustId);
        beanParams.put("list1", list1);
        //十元素2导出
        List<TenElementReport> list2 = iEntrustService.getTen2List(entrustId);
        beanParams.put("list2", list2);
        //煤的工业分析
        List<IndustrialReport> list3 = iEntrustService.getIndustrialList(entrustId);
        beanParams.put("list3", list3);
        //工艺性能
        List<CraftReport> list4 = iEntrustService.getCraftList(entrustId);
        beanParams.put("list4", list4);
        //物理性能
        List<PhysicsReport> list5 = iEntrustService.getPhysicsList(entrustId);
        beanParams.put("list5", list5);
        //颗粒分析
        List<GrainReport> list6 = iEntrustService.getGrainList(entrustId);
        beanParams.put("list6", list6);

        //事项-----不用处理
        //品质-----不用处理

        WordUtil.writeWordReport(rts.getProjectName() + "(报告)", "report_new.ftl",
                beanParams, response, FileExt.EXCL);

    }


    private String getFirstSet(Set<String> set) {
        if(set != null && set.size()>0){
            Iterator<String> iterator = set.iterator();
            if (iterator.hasNext()) {
                return iterator.next();
            }
        }

        return "";
    }

    private static Set<String> moveFirst(Set<String> set) {
        if(set != null && set.size()>0){
            Set<String> newSet = new HashSet<>(set.size());
            Iterator<String> iterator = set.iterator();
            if (iterator.hasNext()) {
                String next = iterator.next();
                newSet.addAll(set);
                newSet.remove(next);
                return newSet;
            }
        }
        return null;
    }

    private String set2String(Set<String> source) {
        if (source.size() == 0) {
            return "";
        }
        StringBuilder strBuilder = new StringBuilder();
        for (String target : source) {
            strBuilder.append("、").append(target);
        }
        return strBuilder.replace(0, 1, "").toString();
    }


    @PostMapping("/upload")
    @ApiOperation("上传报告")
    public BaseResponse<Boolean> uploadReport(@RequestBody SampleHandleEnclosureQuery query) {

        Assert.notNull(query.getId(), "委托id不能为空!");

        QueryWrapper<EntityEnclosure> qw = new QueryWrapper();
        qw.eq("entity_type", 5);
        qw.eq("entity_id", query.getId());

        if (FileExt.EXCL.getName().equals(query.getExtName()) ||
                FileExt.EXCLX.getName().equals(query.getExtName()) ||
                FileExt.PDF.getName().equals(query.getExtName())) {
            EntityEnclosure one = iEntityEnclosureService.getOne(qw);
            if (BeanUtil.isEmpty(one)) {
                EntityEnclosure entityEnclosure = new EntityEnclosure();
                entityEnclosure.setEntityId(query.getId());
                entityEnclosure.setExtName(query.getExtName());
                entityEnclosure.setCreateTime(LocalDateTime.now());
                entityEnclosure.setUpdateTime(LocalDateTime.now());
                entityEnclosure.setEnclosureUrl(query.getEnclosureUrl());
                entityEnclosure.setAlias(query.getAlias());
                entityEnclosure.setEntityType(5);
                entityEnclosure.setPdfUrl(query.getPdfUrl());
                iEntityEnclosureService.save(entityEnclosure);
                return BaseResponse.okData(true);
            } else {
                one.setAlias(query.getAlias());
                one.setEnclosureUrl(query.getEnclosureUrl());
                one.setExtName(query.getExtName());
                one.setUpdateTime(LocalDateTime.now());
                one.setPdfUrl(query.getPdfUrl());
                iEntityEnclosureService.updateById(one);
                return BaseResponse.okData(true);
            }
        } else {
            return BaseResponse.errorMsg("文件类型只能是excel!");
        }


    }


}
