package cn.wise.sc.cement.business.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;
import cn.wise.sc.cement.business.entity.*;
import cn.wise.sc.cement.business.mapper.NormProductionMapper;
import cn.wise.sc.cement.business.mapper.TeamGroupMapper;
import cn.wise.sc.cement.business.mapper.TeamMapper;
import cn.wise.sc.cement.business.model.BaseResponse;
import cn.wise.sc.cement.business.model.PageQuery;
import cn.wise.sc.cement.business.model.vo.*;
import cn.wise.sc.cement.business.service.*;
import cn.wise.sc.cement.business.util.ExcelUtil;
import cn.wise.sc.cement.business.util.RedisUtil;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author ztw
 * @since 2020-09-23
 */
@Service
public class NormProductionServiceImpl extends ServiceImpl<NormProductionMapper, NormProduction> implements INormProductionService {

    /**
     * 标准产值配置信息  缓存
     */
    final public Map<String, NormProduction> normProductionCache = new HashMap<>();

    final
    ISampleDistributionService iSampleDistributionService;
    final
    ISampleHandleService iSampleHandleService;
    final
    ISysUserService iSysUserService;
    final
    ISysGroupService iSysGroupService;
    final
    ISampleCheckTeamService iSampleCheckTeamService;
    final
    ITeamGroupService iTeamGroupService;
    final
    ITeamService iTeamService;
    final
    IHandleService iHandleService;
    final
    ISysPostService iSysPostService;
    final
    INonStandardValueService iNonStandardValueService;
    final
    INonStandardApplyService iNonStandardApplyService;
    final
    RedisUtil redisUtil;
    @Resource
    private TeamMapper teamMapper;
    @Resource
    private TeamGroupMapper teamGroupMapper;


    public NormProductionServiceImpl(ISampleDistributionService iSampleDistributionService,
                                     ISampleHandleService iSampleHandleService,
                                     ISysUserService iSysUserService,
                                     ISysGroupService iSysGroupService,
                                     ISampleCheckTeamService iSampleCheckTeamService,
                                     ITeamGroupService iTeamGroupService, IHandleService iHandleService,
                                     ISysPostService iSysPostService,
                                     INonStandardValueService iNonStandardValueService,
                                     INonStandardApplyService iNonStandardApplyService, RedisUtil redisUtil,
                                     ITeamService iTeamService
    ) {
        this.iSampleDistributionService = iSampleDistributionService;
        this.iSampleHandleService = iSampleHandleService;
        this.iSysUserService = iSysUserService;
        this.iSysGroupService = iSysGroupService;
        this.iSampleCheckTeamService = iSampleCheckTeamService;
        this.iTeamGroupService = iTeamGroupService;
        this.iHandleService = iHandleService;
        this.iSysPostService = iSysPostService;
        this.iNonStandardValueService = iNonStandardValueService;
        this.iNonStandardApplyService = iNonStandardApplyService;
        this.redisUtil = redisUtil;
        this.iTeamService = iTeamService;
    }

    /**
     * 新增产值配置
     * @param normProduction 标准产值
     * @return
     */
    @Transactional
    @Override
    public BaseResponse<NormProduction> addObj(NormProduction normProduction) {
        if(normProduction.getType() == null){
            return BaseResponse.errorMsg("类型不允许为空!");
        }

        //如果是处理项 只能添加一次
        if(normProduction.getType() == 0){
            normProduction.setAssessId(null);
            //判断配置项目是否已存在
            QueryWrapper<NormProduction> qw = new QueryWrapper<>();
            qw.eq("type", normProduction.getType());
            int count = this.count(qw);
            if (count > 0) {
                return BaseResponse.errorMsg("当前处理项标准产值已存在!");
            }
        }else{
            if(normProduction.getAssessId() == null){
                return BaseResponse.errorMsg("对象id不允许为空!");
            }
            //判断配置项目是否已存在
            QueryWrapper<NormProduction> qw = new QueryWrapper<>();
            qw.eq("type", normProduction.getType());
            qw.eq("assess_id", normProduction.getAssessId());
            int count = this.count(qw);
            if (count > 0) {
                return BaseResponse.errorMsg("当前标准产值已存在!");
            }
        }


        //如果是检测组，判断检测组下面的检测项是否添加，如果添加进行提示。
        if(normProduction.getType() == 1){
            TeamGroup teamGroup = teamGroupMapper.selectById(normProduction.getAssessId());
            if(teamGroup == null){
                return BaseResponse.errorMsg("检测组信息有误!");
            }
            List<Integer> idList = teamMapper.getIdForGroup(normProduction.getAssessId());
            if(idList!= null && idList.size()>0){
                for(Integer teamId : idList){
                    QueryWrapper<NormProduction> qw1 = new QueryWrapper<>();
                    qw1.eq("type", 2);
                    qw1.eq("assess_id", teamId);
                    int count1 = this.count(qw1);
                    if (count1 > 0) {
                        return BaseResponse.errorMsg("检测组中检测项的配置已存在!");
                    }
                }
            }
        }
        //如果是检测项  判断对应的检测组是否已经添加，如果添加进行提示。
        if(normProduction.getType() == 2){
            Team team = teamMapper.selectById(normProduction.getAssessId());
            if(team == null){
                return BaseResponse.errorMsg("检测项信息有误!");
            }
            if(team!= null && team.getGroupId() != null){
                QueryWrapper<NormProduction> qw1 = new QueryWrapper<>();
                qw1.eq("type", 1);
                qw1.eq("assess_id", team.getGroupId());
                int count1 = this.count(qw1);
                if (count1 > 0) {
                    return BaseResponse.errorMsg("检测项所在检测组的配置已存在!");
                }
            }
        }

        //判断比例是否为总值100%
        if (normProduction.getAnalyseRate()
                + normProduction.getAssessRate()
                + normProduction.getSeparateRate()
                + normProduction.getReportedResultRate() != 1) {
            return BaseResponse.errorMsg("百分比总和不等于1");
        }

        //状态为1
        normProduction.setStatus(1);
        boolean save = this.save(normProduction);
        if (save) {
            return BaseResponse.okData(normProduction);
        } else {
            return BaseResponse.errorMsg("失败!除检测组id之外其他字段必填!");
        }
    }

    /**
     * 修改产值配置
     * @param normProduction
     * @return
     */
    @Transactional
    @Override
    public BaseResponse<Boolean> edit(NormProduction normProduction) {
        if (normProduction.getId() == null ) {
            return BaseResponse.errorMsg("参数错误!");
        }
        if(normProduction.getType() == null){
            return BaseResponse.errorMsg("类型不允许为空!");
        }
        NormProduction normProductionOld = this.getById(normProduction.getId());
        if(normProductionOld == null){
            return BaseResponse.errorMsg("信息错误!");
        }

        //如果是处理项 只能添加一次
        if(normProduction.getType() == 0){
            normProduction.setAssessId(null);
            //判断配置项目是否已存在
            QueryWrapper<NormProduction> qw = new QueryWrapper<>();
            qw.eq("type", normProduction.getType());
            qw.ne("id", normProductionOld.getId());
            int count = this.count(qw);
            if (count > 0) {
                return BaseResponse.errorMsg("当前处理项标准产值已存在!");
            }
        }else{
            if(normProduction.getAssessId() == null){
                return BaseResponse.errorMsg("对象id不允许为空!");
            }
            //判断配置项目是否已存在
            QueryWrapper<NormProduction> qw = new QueryWrapper<>();
            qw.eq("assess_id", normProduction.getAssessId());
            qw.eq("type", normProduction.getType());
            qw.ne("id", normProductionOld.getId());
            int count = this.count(qw);
            if (count > 0) {
                return BaseResponse.errorMsg("当前标准产值已存在!");
            }
        }


        //如果是检测组，判断检测组下面的检测项是否添加，如果添加进行提示。
        if(normProduction.getType() == 1){
            TeamGroup teamGroup = teamGroupMapper.selectById(normProduction.getAssessId());
            if(teamGroup == null){
                return BaseResponse.errorMsg("检测组信息有误!");
            }
            List<Integer> idList = teamMapper.getIdForGroup(normProduction.getAssessId());
            if(idList!= null && idList.size()>0){
                for(Integer teamId : idList){
                    QueryWrapper<NormProduction> qw1 = new QueryWrapper<>();
                    qw1.eq("type", 2);
                    qw1.eq("assess_id", teamId);
                    int count1 = this.count(qw1);
                    if (count1 > 0) {
                        return BaseResponse.errorMsg("检测组中检测项的配置已存在!");
                    }
                }
            }
        }
        //如果是检测项  判断对应的检测组是否已经添加，如果添加进行提示。
        if(normProduction.getType() == 2){
            Team team = teamMapper.selectById(normProduction.getAssessId());
            if(team == null){
                return BaseResponse.errorMsg("检测项信息有误!");
            }
            if(team!= null && team.getGroupId() != null){
                QueryWrapper<NormProduction> qw1 = new QueryWrapper<>();
                qw1.eq("type", 1);
                qw1.eq("assess_id", team.getGroupId());
                int count1 = this.count(qw1);
                if (count1 > 0) {
                    return BaseResponse.errorMsg("检测项所在检测组的配置已存在!");
                }
            }
        }
        //判断比例是否为总值100%
        if (normProduction.getAnalyseRate()
                + normProduction.getAssessRate()
                + normProduction.getSeparateRate()
                + normProduction.getReportedResultRate() != 1) {
            return BaseResponse.errorMsg("百分比总和不等于1");
        }


        boolean b = this.updateById(normProduction);
        if (b) {
            return BaseResponse.okData(true);
        } else {
            return BaseResponse.errorMsg("修改失败!");
        }

    }

    /**
     * 产值配置分页列表
     * @param pageQuery 分页条件
     * @return
     */
    @Override
    public BaseResponse<IPage<NormProductionVo>> getPage(PageQuery pageQuery,Integer type) {
        IPage<NormProduction> page = new Page<>(pageQuery.getPageNo(), pageQuery.getPageSize());
        QueryWrapper<NormProduction> qw = new QueryWrapper<>();
        if(type != null){
            qw.eq("type", type);
        }
        IPage<NormProduction> productionIPage = this.page(page, qw);
        //关联名字
        List<NormProduction> normProductions = productionIPage.getRecords();

        List<Integer> npTeamGroupIds = new ArrayList<>(normProductions.size());
        List<Integer> npTeamIds = new ArrayList<>(normProductions.size());
        normProductions.forEach(arg -> {
            if(arg.getType() == 1) {
                npTeamGroupIds.add(arg.getAssessId());
            }else if(arg.getType() == 2) {
                npTeamIds.add(arg.getAssessId());
            }
        });
        List<TeamGroup> teamGroups = new ArrayList<>();
        if (npTeamGroupIds.size() != 0) {
            teamGroups = iTeamGroupService.listByIds(npTeamGroupIds);
        }
        List<Team> teams = new ArrayList<>();
        if (npTeamIds.size() != 0) {
            teams = iTeamService.listByIds(npTeamIds);
        }

        IPage<NormProductionVo> rts = new Page<>(pageQuery.getPageNo(), pageQuery.getPageSize());
        List<NormProductionVo> records = new ArrayList<>(pageQuery.getPageSize());
        rts.setTotal(productionIPage.getTotal());
        rts.setCurrent(productionIPage.getCurrent());
        rts.setPages(productionIPage.getPages());
        rts.setSize(productionIPage.getSize());

        List<TeamGroup> finalTeamGroups = teamGroups;
        List<Team> finalTeams = teams;

        normProductions.forEach(arg -> {
            NormProductionVo normProductionVo = new NormProductionVo();
            BeanUtil.copyProperties(arg, normProductionVo);
            normProductionVo.setAssessName("/");
            if (arg.getType() == 0) {
                normProductionVo.setAssessName("处理项");
            } else if(arg.getType() == 1) {
                finalTeamGroups.stream()
                        .filter(opt -> opt.getId().intValue() == arg.getAssessId())
                        .findFirst()
                        .ifPresent(opt -> normProductionVo.setAssessName(opt.getName()));
            } else if(arg.getType() == 2) {
                finalTeams.stream()
                        .filter(opt -> opt.getId().intValue() == arg.getAssessId())
                        .findFirst()
                        .ifPresent(opt -> normProductionVo.setAssessName(opt.getName()));
            }
            if (arg.getType() == 0) {
                normProductionVo.setType("处理项");
            }else if(arg.getType() == 1) {
                normProductionVo.setType("检测组");
            }else if(arg.getType() == 2) {
                normProductionVo.setType("检测项");
            }
            records.add(normProductionVo);
        });
        rts.setRecords(records);
        return BaseResponse.okData(rts);
    }

    /**
     * 产值配置列表
     * @return
     */
    @Override
    public List<NormProductionVo> getList() {
        QueryWrapper<NormProduction> qw = new QueryWrapper<>();
        //关联名字
        List<NormProduction> normProductions = this.list(qw);

        List<Integer> npTeamGroupIds = new ArrayList<>(normProductions.size());
        List<Integer> npTeamIds = new ArrayList<>(normProductions.size());
        normProductions.forEach(arg -> {
            if(arg.getType() == 1) {
                npTeamGroupIds.add(arg.getAssessId());
            }else if(arg.getType() == 2) {
                npTeamIds.add(arg.getAssessId());
            }
        });
        List<TeamGroup> teamGroups = new ArrayList<>();
        if (npTeamGroupIds.size() != 0) {
            teamGroups = iTeamGroupService.listByIds(npTeamGroupIds);
        }
        List<Team> teams = new ArrayList<>();
        if (npTeamIds.size() != 0) {
            teams = iTeamService.listByIds(npTeamIds);
        }

        List<NormProductionVo> records = new ArrayList<>();
        List<TeamGroup> finalTeamGroups = teamGroups;
        List<Team> finalTeams = teams;
        normProductions.forEach(arg -> {
            NormProductionVo normProductionVo = new NormProductionVo();
            BeanUtil.copyProperties(arg, normProductionVo);
            normProductionVo.setAssessName("/");
            if (arg.getType() == 0) {
                normProductionVo.setAssessName("处理项");
            } else if(arg.getType() == 1) {
                finalTeamGroups.stream()
                        .filter(opt -> opt.getId().intValue() == arg.getAssessId())
                        .findFirst()
                        .ifPresent(opt -> normProductionVo.setAssessName(opt.getName()));
            } else if(arg.getType() == 2) {
                finalTeams.stream()
                        .filter(opt -> opt.getId().intValue() == arg.getAssessId())
                        .findFirst()
                        .ifPresent(opt -> normProductionVo.setAssessName(opt.getName()));
            }
            if (arg.getType() == 0) {
                normProductionVo.setType("处理项");
            }else if(arg.getType() == 1) {
                normProductionVo.setType("检测组");
            }else if(arg.getType() == 2) {
                normProductionVo.setType("检测项");
            }
            records.add(normProductionVo);
        });

        return records;
    }



    /**
     * 启用禁用标准产值
     * @param id id
     * @return
     */
    @Override
    public BaseResponse<Integer> activeOrForbidden(Integer id) {

        NormProduction normProduction = this.getById(id);
        Integer status = normProduction.getStatus();
        if (status == 1) {
            normProduction.setStatus(0);
            this.updateById(normProduction);
            return BaseResponse.okData(0);
        } else {
            normProduction.setStatus(1);
            this.updateById(normProduction);
            return BaseResponse.okData(1);
        }

    }

}
