package cn.wise.sc.cement.business.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;
import cn.wise.sc.cement.business.entity.*;
import cn.wise.sc.cement.business.mapper.*;
import cn.wise.sc.cement.business.model.BaseResponse;
import cn.wise.sc.cement.business.model.LoginUser;
import cn.wise.sc.cement.business.model.PageQuery;
import cn.wise.sc.cement.business.model.vo.*;
import cn.wise.sc.cement.business.service.*;
import cn.wise.sc.cement.business.util.ExcelUtil;
import cn.wise.sc.cement.business.util.PageUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.apache.ibatis.annotations.Param;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;
import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.time.Instant;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author ztw
 * @since 2020-11-02
 */
@Service
public class PrecipriceServiceImpl extends ServiceImpl<PrecipriceMapper, Preciprice> implements IPrecipriceService {

    @Autowired
    private ISysUserService userService;
    @Resource
    PrecipriceMapper precipriceMapper;
    @Resource
    private SysUserMapper userMapper;
    @Resource
    private SysGroupMapper groupMapper;
    @Resource
    private EntrustMapper entrustMapper;
    @Resource
    private SampleMapper sampleMapper;
    @Resource
    private NormProductionMapper normProductionMapper;
    @Resource
    private TeamMapper teamMapper;
    @Resource
    private TeamGroupMapper teamGroupMapper;
    @Resource
    private SampleDistributionMapper distributionMapper;
    @Resource
    private NonStandardApplyMapper nonStandardApplyMapper;
    @Autowired
    private INonStandardValueService nonStandardValueService;
    @Autowired
    private INormProductionService normProductionService;


    /**
     * 计算产值
     * @param userId
     * @param entrustId
     * @param targetId
     * @param type
     * @param sampleId
     * @param isUrgent
     * @return
     */

    @Override
    @Transactional(rollbackFor = Exception.class, propagation = Propagation.NESTED)
    public boolean createPreciprice(final Integer userId, final Integer entrustId,
                                    final Integer targetId, final Integer type,
                                    final Integer sampleId, boolean isUrgent) {

        Assert.notNull(userId, "用户id不能为空！");
        Assert.notNull(entrustId, "项目id不能为空！");
        Assert.notNull(sampleId, "样品id不能为空！");
        Assert.notNull(type, "处理项类型不能为空！");
        //获取项目编号和名字
        Entrust entrust = entrustMapper.selectById(entrustId);
        if (BeanUtil.isEmpty(entrust)) {
            log.debug("产值记录新增失败,项目没找到!");
            throw new IllegalArgumentException("无效的项目id!");
        }
        Preciprice preciprice = new Preciprice();
        preciprice.setEntrustId(entrust.getId());
        preciprice.setProjectName(entrust.getProjectName());
        preciprice.setProjectCode(entrust.getProjectCode());
        preciprice.setEntrustCode(entrust.getEntrustCode());
        preciprice.setIsUrgent(isUrgent ? 1 : 0);
        //获取样品信息
        Sample sample = sampleMapper.selectById(sampleId);
        if (BeanUtil.isEmpty(sample)) {
            log.debug("产值记录新增失败,项目没找到!");
            throw new IllegalArgumentException("无效的项目id!");
        }
        preciprice.setSampleId(sample.getId());
        preciprice.setSampleName(sample.getName());

        //获取产值信息
        QueryWrapper<NormProduction> queryWrapper = new QueryWrapper<>();
        if(targetId != null){
            queryWrapper.eq("assess_id", targetId);
        }
        queryWrapper.eq("type", type);
        queryWrapper.eq("status", 1);
        NormProduction normProduction = normProductionMapper.selectOne(queryWrapper);

        if (BeanUtil.isEmpty(normProduction)) {
            log.debug("产值记录新增失败,产值信息没找到!");
            throw new IllegalArgumentException("联系管理员.配置产值信息!");
        }
        preciprice.setAnalyseRate(BigDecimal.valueOf(normProduction.getAnalyseRate()));
        preciprice.setAssessRate(BigDecimal.valueOf(normProduction.getAssessRate()));
        preciprice.setAssessValue(BigDecimal.valueOf(normProduction.getAssessValue()));
        preciprice.setReportedAnalyseResult(BigDecimal.valueOf(normProduction.getReportedAnalyseResult()));
        preciprice.setReportedResultRate(BigDecimal.valueOf(normProduction.getReportedResultRate()));
        preciprice.setCoefficient(BigDecimal.valueOf(normProduction.getCoefficient()));
        preciprice.setCreateTime(DateUtil.date().getTime());
        preciprice.setSeparateRate(BigDecimal.valueOf(normProduction.getSeparateRate()));
        preciprice.setQuotaDay(BigDecimal.valueOf(normProduction.getQuotaDay()));
        //产值计算算法
        if (type == 0) {
            if (isUrgent) {
                preciprice.setPreciprice(new BigDecimal(3).multiply(calculateWorkTimeCoefficient(normProduction, normProduction.getAnalyseRate())));
            } else {
                preciprice.setPreciprice(calculateWorkTimeCoefficient(normProduction, normProduction.getAnalyseRate()));
            }
        }else if(type == 1){
            if (isUrgent) {
                preciprice.setPreciprice(new BigDecimal(3).multiply(calculateWorkTimeCoefficient(normProduction, normProduction.getAnalyseRate())));
            } else {
                preciprice.setPreciprice(calculateWorkTimeCoefficient(normProduction, normProduction.getAnalyseRate()));
            }
        }else if(type == 2){
            if (isUrgent) {
                preciprice.setPreciprice(new BigDecimal(3).multiply(calculateWorkTimeCoefficient(normProduction, normProduction.getAnalyseRate())));
            } else {
                preciprice.setPreciprice(calculateWorkTimeCoefficient(normProduction, normProduction.getAnalyseRate()));
            }
        }
        //获取用户职位和账号
        SysUser sysUser = userMapper.selectById(userId);
        if (BeanUtil.isEmpty(sysUser)) {
            log.debug("产值记录新增失败,用户没找到!");
            throw new IllegalArgumentException("无效的用户id!");
        }
        preciprice.setUserId(sysUser.getId());
        preciprice.setUserName(sysUser.getName());
        preciprice.setSex(sysUser.getSex());
        SysGroup sysGroup = groupMapper.selectById(sysUser.getGroupId());
        if (BeanUtil.isEmpty(sysGroup)) {
            log.debug("产值记录新增失败,用户所在部门未找到!");
            throw new IllegalArgumentException("用户所在部门需要设置!");
        }
        preciprice.setPosition(sysGroup.getName());
        preciprice.setGroupId(sysGroup.getId());

        //获取检测组名
        if (type == 0) {
            preciprice.setGroupTeamName("处理项");
        } else if(type == 1) {
            TeamGroup teamGroup = teamGroupMapper.selectById(targetId);
            preciprice.setGroupTeamName(teamGroup.getName());
        }else if(type == 2) {
            Team team = teamMapper.selectById(targetId);
            preciprice.setGroupTeamName(team.getName());
        }
        preciprice.setTargetId(targetId);
        preciprice.setType(type);

        try {
            return this.save(preciprice);
        } catch (Exception e) {
            log.error("添加产值记录失败!");
            return false;
        }
    }

    /**
     * 产值统计-标准产值-分页列表
     * @param pageQuery
     * @param startDate
     * @param endDate
     * @param userName
     * @return
     */
    public BaseResponse<IPage<NormProductionStatisticsVo>> normProductionStatistics(PageQuery pageQuery, String startDate, String endDate,String userName) {
        LoginUser loginUser = userService.getLoginUser();
        if (loginUser == null) {
            return BaseResponse.noLogin("请登录账号");
        }

        Map<String, Object> params = new HashMap<>();
        params.put("startDate", StrUtil.isNotBlank(startDate)?DateUtil.parseDate(startDate).getTime():null);
        params.put("endDate", StrUtil.isNotBlank(endDate)?DateUtil.parseDate(endDate).getTime():null);
        params.put("userName", userName);
        //只有管理员角色的用户可以查看所有人的产值，其他人只能查看自己的产值
        if(loginUser.getRoleList().getId() != 1){
            params.put("userId", loginUser.getId());
        }

        Page<NormProductionStatisticsVo> page = new Page<>(pageQuery.getPageNo(), pageQuery.getPageSize());
        IPage<NormProductionStatisticsVo> pages = precipriceMapper.getStatisticsPage(page, params);

        return BaseResponse.okData(pages);
    }

    /**
     * 产值统计-标准产值-列表
     * @param startDate
     * @param endDate
     * @param userName
     * @return
     */
    public BaseResponse<List<NormProductionStatisticsVo>> statisticsList(String startDate, String endDate,String userName) {
        LoginUser loginUser = userService.getLoginUser();
        if (loginUser == null) {
            return BaseResponse.noLogin("请登录账号");
        }
        //标准产值
        Map<String, Object> params = new HashMap<>();
        params.put("userName", userName);
        params.put("startDate", StrUtil.isNotBlank(startDate)?DateUtil.parseDate(startDate).getTime():null);
        params.put("endDate", StrUtil.isNotBlank(endDate)?DateUtil.parseDate(endDate).getTime():null);
        //只有管理员角色的用户可以查看所有人的产值，其他人只能查看自己的产值
        if(loginUser.getRoleList().getId() != 1){
            params.put("userId", loginUser.getId());
        }
        List<NormProductionStatisticsVo> productionStatistics = precipriceMapper.getStatisticsList(params);
        return BaseResponse.okData(productionStatistics);
    }



    /**
     * 导出-产值统计-标准产值-列表
     * @param startDate
     * @param endDate
     * @param userName
     */
    @Override
    public void exportStatisticsList(String startDate, String endDate, String userName, String fileName, HttpServletResponse response) {
        BaseResponse<List<NormProductionStatisticsVo>> listBaseResponse = statisticsList(startDate, endDate, userName);
        if (listBaseResponse.getCode() == 200) {
            List<NormProductionStatisticsVo> data = listBaseResponse.getData();
            if (CollectionUtil.isNotEmpty(data)) {
                String[] headers = new String[6];
                headers[0] = "序号";
                headers[1] = "姓名";
                headers[2] = "账户";
                headers[3] = "性别";
                headers[4] = "检测项目数";
                headers[5] = "产值绩效";
                List<Object[]> exportData = new ArrayList<>(data.size());
                int i=0;
                for (NormProductionStatisticsVo vo : data) {
                    Object[] objs = new Object[headers.length];
                    objs[0] = i++;
                    objs[1] = vo.getUserName();
                    objs[2] = vo.getAccount();
                    objs[3] = vo.getSex();
                    objs[4] = vo.getCount();
                    objs[5] = vo.getCoefficient();
                    exportData.add(objs);
                }
                ExcelUtil.excelExport(
                        fileName == null || fileName.trim().length() <= 0 ? "标准产值统计": fileName, headers,
                        exportData, response);
            }
        }
    }





    /**
     * 产值统计-标准产值-个人详情
     * @param pageQuery
     * @param startDate
     * @param endDate
     * @param userId
     * @return
     */
    @Override
    public BaseResponse<IPage<NormProduction.NormProductionUserDetail>>normProductionDetails(PageQuery pageQuery, String startDate, String endDate, Integer userId) {
        if(userId == null ){
            return BaseResponse.errorMsg("参数错误");
        }
        SysUser sysUser = userService.getById(userId);
        if(sysUser == null){
            return BaseResponse.errorMsg("信息错误");
        }

        QueryWrapper<Preciprice> qw = new QueryWrapper<>();
        if (StrUtil.isNotBlank(startDate)) {
            qw.gt("create_time", DateUtil.parseDate(startDate).getTime());
        }
        if (StrUtil.isNotBlank(endDate)) {
            qw.le("create_time", DateUtil.parseDate(endDate).getTime());
        }
        if (!BeanUtil.isEmpty(userId)) {
            qw.eq("user_id", userId);
        }
//        qw.eq("status", 1);
        qw.orderByDesc("create_time");
        List<Preciprice> list = this.list(qw);
        List<NormProduction.NormProductionUserDetail> rts = new ArrayList<>(list.size());

        //处理项
        List<Preciprice> handlelist =list.stream().filter(arg ->(arg.getType()==0)).collect(Collectors.toList());
        if(handlelist != null && handlelist.size()>0){
            NormProduction.NormProductionUserDetail UserDetailRts = new NormProduction.NormProductionUserDetail();

            UserDetailRts.setType(0);
            UserDetailRts.setUserName(sysUser.getName());
            UserDetailRts.setObjId(null);
            UserDetailRts.setObjName("处理项");
            UserDetailRts.setCountNums(handlelist.size());
            UserDetailRts.setCountPricel(handlelist.stream().map(Preciprice::getPreciprice).reduce(BigDecimal.ZERO,BigDecimal::add));
            rts.add(UserDetailRts);
        }


        //检测组
        List<Preciprice> teamGrouplist = list.stream().filter(arg ->(arg.getType()==1)).collect(Collectors.toList());
        if(teamGrouplist != null && teamGrouplist.size()>0){

            Map<Integer, List<Preciprice>> mapByGroupId = teamGrouplist.stream().collect(Collectors.groupingBy(Preciprice::getTargetId, Collectors.toList()));
            for(Map.Entry<Integer, List<Preciprice>> entry : mapByGroupId.entrySet()){
                NormProduction.NormProductionUserDetail UserDetailRts = new NormProduction.NormProductionUserDetail();
                UserDetailRts.setType(1);
                UserDetailRts.setUserName(sysUser.getName());
                UserDetailRts.setObjId(entry.getKey());
                UserDetailRts.setObjName(entry.getValue().get(0).getGroupTeamName());
                UserDetailRts.setCountNums(entry.getValue().size());
                UserDetailRts.setCountPricel(entry.getValue().stream().map(Preciprice::getPreciprice).reduce(BigDecimal.ZERO,BigDecimal::add));
                rts.add(UserDetailRts);
            }

        }
        //检测项
        List<Preciprice> teamlist = list.stream().filter(arg ->(arg.getType()==2)).collect(Collectors.toList());
        if(teamlist != null && teamlist.size()>0){

            Map<Integer, List<Preciprice>> mapByTeamId = teamlist.stream().collect(Collectors.groupingBy(Preciprice::getTargetId, Collectors.toList()));
            for(Map.Entry<Integer, List<Preciprice>> entry : mapByTeamId.entrySet()){
                NormProduction.NormProductionUserDetail UserDetailRts = new NormProduction.NormProductionUserDetail();
                UserDetailRts.setType(2);
                UserDetailRts.setUserName(sysUser.getName());
                UserDetailRts.setObjId(entry.getKey());
                UserDetailRts.setObjName(entry.getValue().get(0).getGroupTeamName());
                UserDetailRts.setCountNums(entry.getValue().size());
                UserDetailRts.setCountPricel(entry.getValue().stream().map(Preciprice::getPreciprice).reduce(BigDecimal.ZERO,BigDecimal::add));
                rts.add(UserDetailRts);
            }

        }

        if (rts!=null && rts.size() > 0) {
            Page<NormProduction.NormProductionUserDetail> rts2 = PageUtil.listConvertToPage(rts, pageQuery);
            return BaseResponse.okData(rts2);
        }

        return BaseResponse.okData(new Page<>());
    }

    /**
     * 导出-产值统计-标准产值-个人详情
     * @param startDate
     * @param endDate
     * @param userId
     * @param fileName
     * @param response
     */
    @Override
    public void exportStatisticsDetail(String startDate, String endDate, Integer userId, String fileName, HttpServletResponse response) {
        Assert.notNull(userId, "用户id不能为空！");
        SysUser sysUser = userService.getById(userId);
        QueryWrapper<Preciprice> qw = new QueryWrapper<>();
        if (StrUtil.isNotBlank(startDate)) {
            qw.gt("create_time", DateUtil.parseDate(startDate).getTime());
        }
        if (StrUtil.isNotBlank(endDate)) {
            qw.le("create_time", DateUtil.parseDate(endDate).getTime());
        }
        if (!BeanUtil.isEmpty(userId)) {
            qw.eq("user_id", userId);
        }
//        qw.eq("status", 1);
        qw.orderByDesc("create_time");
        List<Preciprice> list = this.list(qw);
        List<NormProduction.NormProductionUserDetail> rts = new ArrayList<>(list.size());

        //处理项
        List<Preciprice> handlelist =list.stream().filter(arg ->(arg.getType()==0)).collect(Collectors.toList());
        if(handlelist != null && handlelist.size()>0){
            NormProduction.NormProductionUserDetail UserDetailRts = new NormProduction.NormProductionUserDetail();

            UserDetailRts.setType(0);
            UserDetailRts.setUserName(sysUser.getName());
            UserDetailRts.setObjId(null);
            UserDetailRts.setObjName("处理项");
            UserDetailRts.setCountNums(handlelist.size());
            UserDetailRts.setCountPricel(handlelist.stream().map(Preciprice::getPreciprice).reduce(BigDecimal.ZERO,BigDecimal::add));
            rts.add(UserDetailRts);
        }


        //检测组
        List<Preciprice> teamGrouplist = list.stream().filter(arg ->(arg.getType()==1)).collect(Collectors.toList());
        if(teamGrouplist != null && teamGrouplist.size()>0){

            Map<Integer, List<Preciprice>> mapByGroupId = teamGrouplist.stream().collect(Collectors.groupingBy(Preciprice::getTargetId, Collectors.toList()));
            for(Map.Entry<Integer, List<Preciprice>> entry : mapByGroupId.entrySet()){
                NormProduction.NormProductionUserDetail UserDetailRts = new NormProduction.NormProductionUserDetail();
                UserDetailRts.setType(1);
                UserDetailRts.setUserName(sysUser.getName());
                UserDetailRts.setObjId(entry.getKey());
                UserDetailRts.setObjName(entry.getValue().get(0).getGroupTeamName());
                UserDetailRts.setCountNums(entry.getValue().size());
                UserDetailRts.setCountPricel(entry.getValue().stream().map(Preciprice::getPreciprice).reduce(BigDecimal.ZERO,BigDecimal::add));
                rts.add(UserDetailRts);
            }

        }
        //检测项
        List<Preciprice> teamlist = list.stream().filter(arg ->(arg.getType()==2)).collect(Collectors.toList());
        if(teamlist != null && teamlist.size()>0){

            Map<Integer, List<Preciprice>> mapByTeamId = teamGrouplist.stream().collect(Collectors.groupingBy(Preciprice::getTargetId, Collectors.toList()));
            for(Map.Entry<Integer, List<Preciprice>> entry : mapByTeamId.entrySet()){
                NormProduction.NormProductionUserDetail UserDetailRts = new NormProduction.NormProductionUserDetail();
                UserDetailRts.setType(2);
                UserDetailRts.setUserName(sysUser.getName());
                UserDetailRts.setObjId(entry.getKey());
                UserDetailRts.setObjName(entry.getValue().get(0).getGroupTeamName());
                UserDetailRts.setCountNums(entry.getValue().size());
                UserDetailRts.setCountPricel(entry.getValue().stream().map(Preciprice::getPreciprice).reduce(BigDecimal.ZERO,BigDecimal::add));
                rts.add(UserDetailRts);
            }

        }
        List<NormProductionVo> normProductions = normProductionService.getList();

        if (CollectionUtil.isNotEmpty(rts)) {
            String[] headers = new String[14];
            headers[0] = "序号";
            headers[1] = "姓名";
            headers[2] = "检测项目";
            headers[3] = "数量";
            headers[4] = "合计";

            headers[5] = "产值设置参数:";

            headers[6] = "额定工日";
            headers[7] = "报出对应分析结果";
            headers[8] = "建议系数";
            headers[9] = "考核工值";
            headers[10] = "分析占比";
            headers[11] = "分样占比";
            headers[12] = "校核占比";
            headers[13] = "报结果占比";

            List<Object[]> exportData = new ArrayList<>(rts.size());
            int i=0;
            for (NormProduction.NormProductionUserDetail target : rts) {
                Object[] objs = new Object[14];
                objs[0] = i++;
                objs[1] = target.getUserName();
                objs[2] = target.getObjName();
                objs[3] = target.getCountNums();
                objs[4] = target.getCountPricel();

//                objs[5] = "";
                NormProductionVo vo = null;
                if(target.getType() == 0){
                    vo =normProductions.stream()
                                    .filter(arg -> (arg.getType().equals("处理项") && arg.getAssessId()==null)).findFirst().orElse(null);
                }else if(target.getType() == 1){
                    vo = normProductions.stream()
                                    .filter(arg -> (arg.getType().equals("检测组") && arg.getAssessId().equals(target.getObjId())))
                                    .findFirst().orElse(null);

                }else if(target.getType() == 2){
                    vo =normProductions.stream()
                            .filter(arg -> (arg.getType().equals("检测项") && arg.getAssessId().equals(target.getObjId()))).findFirst().orElse(null);
                }

                if(vo != null){
                    objs[6] = vo.getQuotaDay();
                    objs[7] = vo.getReportedAnalyseResult();
                    objs[8] = vo.getCoefficient();
                    objs[9] = vo.getAssessValue();
                    objs[10] = vo.getAnalyseRate();
                    objs[11] = vo.getSeparateRate();
                    objs[12] = vo.getAssessRate();
                    objs[13] = vo.getReportedResultRate();
                }


                exportData.add(objs);
            }
            ExcelUtil.excelExport(fileName == null || fileName.trim().length() <= 0 ? "标准产值详情-"+sysUser.getName(): fileName , headers, exportData, response);
        }
    }



    /**
     * 产值统计-标准产值-个人详情-检测项目详情
     * @param pageQuery
     * @param startDate
     * @param endDate
     * @param userId
     * @param type
     * @param objId
     * @return
     */
    @Override
    public BaseResponse<IPage<NormProduction.NormProductionDetail>>normProductionSpecificDetails(PageQuery pageQuery,
                         String startDate, String endDate, Integer userId, Integer type, Integer objId) {
        if(userId == null ){
            return BaseResponse.errorMsg("参数错误");
        }
        if(type == null ){
            return BaseResponse.errorMsg("请传入类型");
        }

        SysUser sysUser = userService.getById(userId);
        if(sysUser == null){
            return BaseResponse.errorMsg("信息错误");
        }
        QueryWrapper<Preciprice> qw = new QueryWrapper<>();
        if (StrUtil.isNotBlank(startDate)) {
            qw.gt("create_time", DateUtil.parseDate(startDate).getTime());
        }
        if (StrUtil.isNotBlank(endDate)) {
            qw.le("create_time", DateUtil.parseDate(endDate).getTime());
        }
        if (!BeanUtil.isEmpty(userId)) {
            qw.eq("user_id", userId);
        }
        if (!BeanUtil.isEmpty(type)) {
            qw.eq("type", type);
        }
        if(type == 0){
            qw.eq("group_team_name", "处理项");
        }else{
            qw.eq("target_id", objId);
        }
//        qw.eq("status", 1);
        qw.orderByDesc("create_time");
        List<Preciprice> list = this.list(qw);
        List<NormProduction.NormProductionDetail> rts = new ArrayList<>(list.size());
        for(Preciprice pp : list){
            NormProduction.NormProductionDetail UserDetailRts = new NormProduction.NormProductionDetail();
            UserDetailRts.setUserName(pp.getUserName());
            UserDetailRts.setObjName(pp.getGroupTeamName());
            UserDetailRts.setProjectName(pp.getProjectName());
            UserDetailRts.setProjectCode(pp.getProjectCode());
            UserDetailRts.setEntrustCode(pp.getEntrustCode());
            UserDetailRts.setCheckTime(LocalDateTime.ofInstant(Instant.ofEpochMilli(pp.getCreateTime()), ZoneId.systemDefault()));
            UserDetailRts.setCountPricel(pp.getPreciprice());
            rts.add(UserDetailRts);
        }

        if (rts!=null && rts.size() > 0) {
            Page<NormProduction.NormProductionDetail> rts2 = PageUtil.listConvertToPage(rts, pageQuery);
            return BaseResponse.okData(rts2);
        }

        return BaseResponse.okData(new Page<>());
    }



    /**
     * 产值统计-产值统计-总产值统计
     * @param name  用户
     * @param start 开始时间
     * @param end   结束时间
     * @param groupId
     * @return
     */
    @Override
    public List<ProductionVo> production(String name, Long start, Long end, Integer groupId) {
        Date startDate = null;
        Date endDate = new Date();
        if (start != null && start != 0) {
            startDate = DateUtil.date(start);
        }
        if (end != null && end != 0) {
            endDate = DateUtil.date(end);
        }

        //非标准产值
        List<NonStandardValue> nonStandardValues = nonStandardValueService.nonValue(startDate, endDate, groupId, name).getData();
        //标准产值
        Map<String, Object> params = new HashMap<>();
        params.put("userName", name);
        params.put("startDate", start);
        params.put("endDate", end);
        params.put("groupId", groupId);
        List<NormProductionStatisticsVo> productionStatistics = precipriceMapper.getStatisticsList(params);

        //以userId找关系
        Set<String> userIds = new HashSet<>();
        nonStandardValues.forEach(arg -> userIds.add(arg.getUserId().toString()));
        productionStatistics.forEach(arg -> userIds.add(arg.getUserId()));

        List<ProductionVo> rts = new ArrayList<>(userIds.size());
        userIds.forEach(arg -> {
            ProductionVo productionVo = new ProductionVo();
            productionVo.setUserId(arg);
            //找到标准准产值里面的数据
            productionStatistics.forEach(opt -> {
                if (opt.getUserId().equals(arg)) {
                    productionVo.setProductionValue(opt.getCoefficient());
                    productionVo.setUserName(opt.getUserName());
                    productionVo.setAccount(opt.getAccount());
                    productionVo.setSex(opt.getSex());
                }
            });
            //找到非标产值里面的数据
            nonStandardValues.forEach(opt -> {
                if (arg.equals(opt.getUserId().toString())) {
                    productionVo.setNonProductionValue(opt.getFinalValue());
                }
            });
            rts.add(productionVo);
        });


        return rts;
    }

    /**
     * 导出-产值统计-产值统计-总产值统计
     * @param name
     * @param start
     * @param end
     * @param groupId
     * @param fileName
     * @param response
     */
    @Override
    public void exportProduction(String name, Long start, Long end, Integer groupId, String fileName, HttpServletResponse response) {
        List<ProductionVo> list = production(name, start, end, groupId);
        list.stream()
                .forEach(arg -> {
            arg.setId(Integer.valueOf(arg.getUserId()));
            arg.setProductionTotalValue(arg.getProductionValue() + arg.getNonProductionValue());
        });
        List<ProductionVo> data = list.stream().sorted(Comparator.comparing(ProductionVo::getId)).collect(Collectors.toList());
        if (CollectionUtil.isNotEmpty(data)) {
            String[] headers = new String[6];
            headers[0] = "序号";
            headers[1] = "姓名";
            headers[2] = "账户";
            headers[3] = "标准产值";
            headers[4] = "非标准产值";
            headers[5] = "产值统计";
            List<Object[]> exportData = new ArrayList<>(data.size());
            int i = 0;
            for (ProductionVo target : data) {
                Object[] objs = new Object[6];
                objs[0] = i++;
                objs[1] = target.getUserName();
                objs[2] = target.getAccount();
                objs[3] = target.getProductionValue();
                objs[4] = target.getNonProductionValue();
                objs[5] = target.getProductionTotalValue();
                exportData.add(objs);
            }
            ExcelUtil.excelExport(fileName == null || fileName.trim().length() <= 0 ? "产值统计": fileName , headers, exportData, response);
        }
    }




    /**
     * 根据报告id更新
     * @param entrustId
     * @return
     */
    @Override
    public boolean updateStatusByEntrustId(Integer entrustId) {
        cn.hutool.core.lang.Assert.notNull(entrustId, "报告id不能为空!");

        try {
            precipriceMapper.updateStatusByEntrustId(entrustId);
            return true;
        } catch (Exception e) {
            log.error("更新产值状态失败!");
            return false;
        }

    }


    /**
     * 计算标准产值的工时标准系数 plan
     * 额定工日*建议系数/报出对应分析结果*考核工值*分析占比
     *
     * @param normProduction 标准产值配置对象  如:六元素
     * @param rate           具体的操作占比   如:统计分析  85%
     * @return BigDecimal
     */
    private BigDecimal calculateWorkTimeCoefficient(NormProduction normProduction, Double rate) {
        //计算出工时标准系数:
        //额定工日*建议系数/报出对应分析结果*考核工值*分析占比
        BigDecimal quotaDayBig = BigDecimal.valueOf(normProduction.getQuotaDay()); //定额工日
        BigDecimal coefficientBig = BigDecimal.valueOf(normProduction.getCoefficient()); //建议系数
        BigDecimal reportedAnalyseRtsBig = BigDecimal.valueOf(normProduction.getReportedAnalyseResult()); //报出分析结果
        BigDecimal assessValueBig = BigDecimal.valueOf(normProduction.getAssessValue()); //考核工值
        BigDecimal analyseRateBig = BigDecimal.valueOf(rate);  //分析占比

        return quotaDayBig.multiply(coefficientBig)
                .divide(reportedAnalyseRtsBig, 10,BigDecimal.ROUND_HALF_UP)
                .multiply(assessValueBig).multiply(analyseRateBig)
                .setScale(2, BigDecimal.ROUND_HALF_UP);
    }
}
