package cn.wise.sc.cement.business.service.impl;

import cn.hutool.core.util.StrUtil;
import cn.wise.sc.cement.business.entity.Handle;
import cn.wise.sc.cement.business.entity.Method;
import cn.wise.sc.cement.business.entity.Team;
import cn.wise.sc.cement.business.entity.TeamGroup;
import cn.wise.sc.cement.business.exception.BusinessExceptionEnum;
import cn.wise.sc.cement.business.mapper.HandleMapper;
import cn.wise.sc.cement.business.mapper.MethodMapper;
import cn.wise.sc.cement.business.mapper.TeamGroupMapper;
import cn.wise.sc.cement.business.mapper.TeamMapper;
import cn.wise.sc.cement.business.model.BaseResponse;
import cn.wise.sc.cement.business.model.PageQuery;
import cn.wise.sc.cement.business.model.query.TeamQuery;
import cn.wise.sc.cement.business.model.vo.*;
import cn.wise.sc.cement.business.service.IHandleService;
import cn.wise.sc.cement.business.service.IMethodService;
import cn.wise.sc.cement.business.service.ITeamGroupService;
import cn.wise.sc.cement.business.service.ITeamService;
import cn.wise.sc.cement.business.util.ExcelUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import io.swagger.annotations.ApiModelProperty;
import io.swagger.models.auth.In;
import org.apache.poi.ss.formula.functions.T;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author ztw
 * @since 2020-08-18
 */
@Service
public class TeamServiceImpl extends ServiceImpl<TeamMapper, Team> implements ITeamService {

	@Resource
	private TeamMapper teamMapper;
	@Resource
	private MethodMapper methodMapper;
	@Resource
	private HandleMapper handleMapper;
	@Autowired
	private ITeamGroupService teamGroupService;
	@Autowired
	private IMethodService methodService;
	@Autowired
	private IHandleService handleService;


	/**
	 * 分页查询
	 *
	 * @param pageQuery
	 * @param status
	 * @param groupName
	 * @param name
	 * @return
	 */
	@Override
	public BaseResponse<IPage<TeamVo>> getPage(PageQuery pageQuery, Integer status, String groupName, String name,
												Integer qualifications) {
		Map<String, Object> params = new HashMap<>();
		params.put("status", status);
		params.put("groupName", groupName);
		params.put("name", name);
		params.put("qualifications", qualifications);
		Page<TeamVo> page = new Page<>(pageQuery.getPageNo(), pageQuery.getPageSize());
		IPage<TeamVo> pages = teamMapper.getPage(page, params);
		return BaseResponse.okData(pages);
	}

	/**
	 * 检测依据id转为检测依据名称和检测依据编号
	 * @return
	 */
	private Map<String, String> methodId2Name(String methodId){
		Map<String, String> map = new HashMap<>();
		String[] methodIds = methodId.split("、");
		if(methodIds != null){
			String methodNames = "";
			String number = "";
			for(String methodid:methodIds){
				Method method = methodMapper.selectById(Integer.valueOf(methodid));
				if(method != null){
					methodNames = methodNames.equals("")?method.getName():methodNames+"、"+method.getName();
					number = number.equals("")?method.getNumber():number+"、"+method.getNumber();
				}
			}
			map.put("methodNames", methodNames);
			map.put("number", number);
			return map;
		}
		return null;
	}

	/**
	 * 处理项id转为处理项名称
	 * @return
	 */
	private Map<String, String> handleId2Name(String handleId){
		Map<String, String> map = new HashMap<>();
		String[] handleIds = handleId.split("、");
		if(handleIds != null){
			String handleNames = "";
			for(String handleid:handleIds){
				Handle handle = handleMapper.selectById(Integer.valueOf(handleid));
				if(handle != null){
					handleNames = handleNames.equals("")?handle.getName():handleNames+"、"+handle.getName();
				}
			}
			map.put("handleNames", handleNames);
			return map;
		}
		return null;
	}

	/**
	 * 新增检测项
	 *
	 * @param query
	 * @return
	 */
	@Override
	@Transactional
	public BaseResponse<Team> create(TeamQuery query) {
		Team create = new Team();
		if (StringUtils.isBlank(query.getName())) {
			return BaseResponse.errorMsg("检测项名称不能为空");
		}
		if (query.getGroupId() == null) {
			return BaseResponse.errorMsg("请选择检测组");
		}
		if (query.getMethodId() == null || query.getMethodId().equals("")) {
			return BaseResponse.errorMsg("请选择检依据");
		}
		if (query.getIsDisplay() == null) {
			return BaseResponse.errorMsg("请选择委托是否可见");
		}
		if (query.getIsDisplay() != 1 && query.getIsDisplay() != 0) {
			return BaseResponse.errorMsg("委托是否可见参数错误");
		}
		if (query.getHandleId() == null || query.getHandleId().equals("") ) {
			return BaseResponse.errorMsg("请选择处理项");
		}
		if (query.getIsParallel() == null) {
			return BaseResponse.errorMsg("请选择是否需要平行");
		}
		if (query.getIsParallel() != 1 && query.getIsParallel() != 0) {
			return BaseResponse.errorMsg("是否需要平行参数错误");
		}
		QueryWrapper<Team> qw = new QueryWrapper<>();
		qw.eq("name", query.getName());
		int count = teamMapper.selectCount(qw);
		if (count > 0) {
			return BaseResponse.errorMsg(query.getName() + "已存在");
		}
		BeanUtils.copyProperties(query, create);
		//检测依据id转为检测依据名称和检测依据编号
		Map<String, String> mapMethod = methodId2Name(create.getMethodId());

		//处理项id转成处理项名称
		Map<String, String> mapHandle = handleId2Name(create.getHandleId());
		QueryWrapper<Team> teamQueryWrapper = new QueryWrapper<>();
		teamQueryWrapper.eq("group_id", query.getGroupId());
		Integer numsForGroup = teamMapper.selectCount(teamQueryWrapper);
		//no_check 标识校核计算的时候不显示计算的结果，因为没有公式
		create.setMethodName(mapMethod.get("methodNames"))
				.setNumber(mapMethod.get("number"))
				.setHandleName(mapHandle.get("handleNames"))
				.setSortNo(numsForGroup+1)
				.setStatus(1).setCreateTime(LocalDateTime.now()).setRemark("no_check");
		teamMapper.insert(create);
		return BaseResponse.okData(create);
	}



	/**
	 * 修改检测项
	 *
	 * @param query
	 * @return
	 */
	@Override
	@Transactional
	public BaseResponse<Team> update(TeamQuery query) {
		if (StringUtils.isBlank(query.getName())) {
			return BaseResponse.errorMsg("检测项名称不能为空");
		}
		Team update = teamMapper.selectById(query.getId());
		if (update == null) {
			return BaseResponse.errorMsg("信息错误");
		}
		if (query.getGroupId() == null) {
			return BaseResponse.errorMsg("请选择检测组");
		}
		if (query.getMethodId() == null || query.getMethodId().equals("")) {
			return BaseResponse.errorMsg("请选择检依据");
		}
		if (query.getIsDisplay() == null) {
			return BaseResponse.errorMsg("请选择委托是否可见");
		}
		if (query.getIsDisplay() != 1 && query.getIsDisplay() != 0) {
			return BaseResponse.errorMsg("委托是否可见参数错误");
		}
		if (query.getHandleId() == null || query.getHandleId().equals("") ) {
			return BaseResponse.errorMsg("请选择处理项");
		}
		if (query.getIsParallel() == null) {
			return BaseResponse.errorMsg("请选择是否需要平行");
		}
		if (query.getIsParallel() != 1 && query.getIsParallel() != 0) {
			return BaseResponse.errorMsg("是否需要平行参数错误");
		}
		QueryWrapper<Team> qw = new QueryWrapper<>();
		qw.eq("name", query.getName());
		qw.ne("id", query.getId());
		int count = teamMapper.selectCount(qw);
		if (count > 0) {
			return BaseResponse.errorMsg(query.getName() + "已存在");
		}
		//检测依据id转为检测依据名称和检测依据编号
		Map<String, String> mapMethod = methodId2Name(query.getMethodId());

		//处理项id转成处理项名称
		Map<String, String> mapHandle = handleId2Name(query.getHandleId());
		if(query.getSortNo() != update.getSortNo()){
			QueryWrapper<Team> teamQueryWrapper = new QueryWrapper<>();
			teamQueryWrapper.eq("group_id", query.getGroupId());
			Integer numsForGroup = teamMapper.selectCount(teamQueryWrapper);
			if(update.getSortNo() != null ){
				if(update.getSortNo()>numsForGroup || update.getSortNo()<1){
					return BaseResponse.errorMsg("编辑的本组序号有误或超过最大值");
				}

			}
			//编辑时，下拉框展示，如果将1改为4，则原来的4 变为  邹继远 20210128
			QueryWrapper<Team> teamQueryWrapperOld = new QueryWrapper<>();
			teamQueryWrapperOld.eq("group_id", query.getGroupId());
			teamQueryWrapperOld.eq("sort_no", query.getSortNo());
			Team teamOld = teamMapper.selectOne(teamQueryWrapperOld);
			if(teamOld != null){
				teamOld.setSortNo(update.getSortNo());
				teamMapper.updateById(teamOld);
			}


			update.setSortNo(query.getSortNo());
		}
		update.setGroupId(query.getGroupId())
				.setMethodId(query.getMethodId())
				.setMethodName(mapMethod.get("methodNames"))
				.setNumber(mapMethod.get("number"))
				.setHandleName(mapHandle.get("handleNames"))
				.setCharge(query.getCharge())
				.setName(query.getName())
				.setIsDisplay(query.getIsDisplay())
				.setQualifications(query.getQualifications())
				.setHandleId(query.getHandleId())
				.setIsParallel(query.getIsParallel())
				.setIsPf(query.getIsPf())
			    .setUnits(query.getUnits());
		teamMapper.updateById(update);
		return BaseResponse.okData(update);
	}

	/**
	 * 启用禁用
	 *
	 * @param status
	 * @param id
	 * @return
	 */
	@Override
	@Transactional
	public BaseResponse<String> status(Integer status, Integer id) {
		if (status == null || id == null) {
			return BaseResponse.errorMsg("参数错误!");
		}
		Team team = teamMapper.selectById(id);
		if (team == null) {
			return BaseResponse.errorMsg("数据错误!");
		}
		team.setStatus(status);
		teamMapper.updateById(team);
		String result = "";
		if (team.getStatus() == 1) {
			result = "已启用";
		} else {
			result = "已禁用";
		}
		return BaseResponse.okData(result);
	}

	/**
	 * 检测项详情
	 *
	 * @param id
	 * @return
	 */
	public BaseResponse<TeamVo> getDetail(Integer id) {
		TeamVo teamVo = teamMapper.getDetail(id);
		if (teamVo == null) {
			return BaseResponse.errorMsg("信息错误!");
		}
		return BaseResponse.okData(teamVo);
	}

	/**
	 * 检测项列表
	 *
	 * @return
	 */
	@Override
	public BaseResponse<List<TeamListVo>> getList() {
		List<TeamListVo> list = teamMapper.getList();
		return BaseResponse.okData(list);
	}

	@Override
	public void export(Integer status, Integer gourpId, String name, String fileName, HttpServletResponse response) {
		Map<String, Object> params = new HashMap<>();
		params.put("status", status);
		params.put("gourpId", gourpId);
		params.put("name", name);
		List<Map<String, Object>> list = teamMapper.exportList(params);

		if (!CollectionUtils.isEmpty(list)) {
			Map<String, Object> map = list.get(0);
			String[] headers = new String[map.size()];
			headers[0] = "序号";
			headers[1] = "检测项目";
			headers[2] = "所在检测组";
			headers[3] = "检测依据";
			headers[4] = "收费标准";
			headers[5] = "资质范围";
			List<Object[]> datas = new ArrayList<>(list.size());
			for (Map<String, Object> m : list) {
				Object[] objects = new Object[headers.length];
				for (int j = 0; j < headers.length; j++) {
					String obj = m.get(headers[j]).toString();
					if (j == 0) {
						obj = obj.split("\\.")[0];
					}
					objects[j] = obj;
				}
 				datas.add(objects);
			}
			ExcelUtil.excelExport(
					fileName == null || fileName.trim().length() <= 0 ? "检测项" : fileName, headers,
					datas, response);
		}
	}

	/**
	 * 根据检测组id获取检测项信息
	 *
	 * @param id
	 * @return
	 */
	@Override
	public BaseResponse<List<String>> getByGroup(Integer id) {
		List<String> map = teamMapper.getByGroup(id);
		return BaseResponse.okData(map);
	}

	@Override
	public BaseResponse<List<Integer>> getSortByGroup(Integer groupId) {
		List<Integer> listResult = teamMapper.getSortByGroup(groupId);
		return BaseResponse.okData(listResult);
	}

	/**
	 * 获取检测项所需列表（检测组，检测依据，处理项）
	 * @return
	 */
	@Override
	public BaseResponse<Map<String, Object>> getSummaryList() {
		List<SummaryVo> list = teamMapper.getSummaryList();
		List<SummaryVo> teamGroupList = new ArrayList<>();
		List<SummaryVo> handleList = new ArrayList<>();
		List<SummaryVo> methodList = new ArrayList<>();
		if(list != null && list.size()>0){
			for(SummaryVo summaryVo : list){
				if(summaryVo.getFlag() == 1){
					teamGroupList.add(summaryVo);
				}else if(summaryVo.getFlag() == 2){
					handleList.add(summaryVo);
				}else if(summaryVo.getFlag() == 3){
					methodList.add(summaryVo);
				}else{
					break;
				}
			}
		}
		Map<String, Object> map = new HashMap<>();
		map.put("teamGroupList", teamGroupList);
		map.put("handleList", handleList);
		map.put("methodList", methodList);
		return BaseResponse.okData(map);
	}



	/**
	 *  优化获取检测项所需列表（检测组，检测依据，处理项）
	 * @return
	 */
	@Override
	public BaseResponse<SummaryListVo> getSummaryListNew() {
		SummaryListVo summaryListVo = teamMapper.getSummaryList2();
		return BaseResponse.okData(summaryListVo);
	}








}
