package cn.wise.sc.cement.business.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;
import cn.wise.sc.cement.business.entity.*;
import cn.wise.sc.cement.business.mapper.*;
import cn.wise.sc.cement.business.model.*;
import cn.wise.sc.cement.business.model.query.*;
import cn.wise.sc.cement.business.model.vo.*;
import cn.wise.sc.cement.business.service.*;
import cn.wise.sc.cement.business.util.CheckCountUtil;
import cn.wise.sc.cement.business.util.ExcelUtil;
import cn.wise.sc.cement.business.util.RedisUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.Data;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.ss.usermodel.HorizontalAlignment;
import org.apache.poi.ss.usermodel.VerticalAlignment;
import org.apache.poi.ss.util.CellRangeAddress;
import org.apache.poi.xssf.usermodel.XSSFCell;
import org.apache.poi.xssf.usermodel.XSSFCellStyle;
import org.apache.poi.xssf.usermodel.XSSFFont;
import org.apache.poi.xssf.usermodel.XSSFRow;
import org.apache.poi.xssf.usermodel.XSSFSheet;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.io.FileInputStream;
import java.io.InputStream;
import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.regex.Pattern;
import java.util.stream.Collectors;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author ztw
 * @since 2020-08-24
 */
@Service
public class EntrustServiceImpl extends ServiceImpl<EntrustMapper, Entrust> implements IEntrustService {

	@Resource
	private EntrustMapper entrustMapper;
	@Autowired
	private ISampleTmpService sampleTmpService;
	@Autowired
	private ISysUserService userService;
	@Resource
	private TeamMapper teamMapper;
	@Autowired
	private CommonServiceImpl commonService;
	@Autowired
	private ISampleService sampleService;
	@Resource
	private EntrustApprovalMapper approvalMapper;
	@Autowired
	private ISysLogsService logsService;
	@Resource
	private SampleHandleMapper sampleHandleMapper;
	@Autowired
	private ISampleDistributionService distributionService;
	@Resource
	private SampleDistributionMapper distributionMapper;
	@Resource
	private ProjectMapper projectMapper;
	@Autowired
	RedisUtil redisUtil;
	@Resource
	private SampleMapper sampleMapper;
	@Resource
	private SysApprovalMapper sysApprovalMapper;
	@Resource
	private MethodMapper methodMapper;
	@Resource
	private HandleMapper handleMapper;
	@Resource
	private SampleHandleEnclosureMapper sampleHandleEnclosureMapper;
	@Resource
	private TeamGroupMapper teamGroupMapper;
	@Resource
	private SampleCheckMapper sampleCheckMapper;
	@Resource
	private SampleCheckTeamMapper sampleCheckTeamMapper;
	@Resource
	private EquipmentUseMapper equipmentUseMapper;
	@Resource
	private ClientMapper clientMapper;
	@Autowired
	private ISysUserMessageService userMessageService;
	@Autowired
	private WeiXinService weiXinService;
	@Autowired
	private INormProductionService iNormProductionService;
	@Autowired
	private IPrecipriceService iPrecipriceService;
	@Resource
	private EntityEnclosureMapper entityEnclosureMapper;

	@Value("${weixin.systemUrl}")
	private String systemUrl;

	@Resource
	private SampleHandleReturnMapper sampleHandleReturnMapper;
	@Resource
	private EquipmentMapper equipmentMapper;
	@Resource
	private SampleDistributionEquipmentMapper sampleDistributionEquipmentMapper;
	@Autowired
	private ISampleDistributionEquipmentService sampleDistributionEquipmentService;
	@Resource
	private SampleDistributionReturnMapper sampleDistributionReturnMapper;
	@Resource
	private SampleDistributionEnclosureMapper sampleDistributionEnclosureMapper;
	@Resource
	private SampleDistributionCheckMapper sampleDistributionCheckMapper;
	@Resource
	private SampleDistributionCheckinputMapper sampleDistributionCheckinputMapper;
	@Resource
	private TitrationMapper titrationMapper;
	@Resource
	private ITeamService teamService;
	@Resource
	private ISampleDistributionService sampleDistributionService;
	@Resource
	private NormProductionMapper normProductionMapper;

	/**
	 * 委托分页
	 *
	 * @param pageQuery
	 * @param startDate
	 * @param endDate
	 * @param status
	 * @param clientName
	 * @param projectName
	 * @return
	 */
	@Override
	public BaseResponse<IPage<EntrustVo>> getPage(PageQuery pageQuery, String entrustCode, String startDate, String endDate, Integer status,
	                                              String clientName, String projectName, String projectCode, Integer isUrgent) {
		LoginUser loginUser = userService.getLoginUser();
		if (loginUser == null) {
			return BaseResponse.noLogin("请登录账号");
		}
		Map<String, Object> params = new HashMap<>();
		params.put("entrustCode", entrustCode);
		params.put("startDate", startDate);
		params.put("endDate", endDate);
		params.put("status", status);
		params.put("clientName", clientName);
		params.put("projectName", projectName);
		params.put("projectCode", projectCode);
		params.put("isUrgent", isUrgent);
		List<Integer> approvalIdList = sysApprovalMapper.getApprovalId("委托评审");
		if (approvalIdList == null) {
			return BaseResponse.errorMsg("委托评审信息错误");
		}
		if (!approvalIdList.contains(loginUser.getId())) {
			params.put("userId", loginUser.getId());
		}
		Page<EntrustVo> page = new Page<>(pageQuery.getPageNo(), pageQuery.getPageSize());
		IPage<EntrustVo> pages = entrustMapper.getPage(page, params);
		List<EntrustVo> list = pages.getRecords();
		if (list != null && list.size() > 0) {
			for (EntrustVo entrustVo : list) {
				QueryWrapper<SampleTmp> queryWrapper = new QueryWrapper<>();
				queryWrapper.eq("entrust_id", entrustVo.getId());
				List<SampleTmp> sampleTmpList = sampleTmpService.list(queryWrapper);
				//检测项目名称
				String sampleNames = "";
				if (sampleTmpList != null && sampleTmpList.size() > 0) {
					//样品名称 列表中的拼接
					for (SampleTmp sampleTmp : sampleTmpList) {
						sampleNames = sampleNames.equals("") ? sampleTmp.getName() : (sampleNames + "、" + sampleTmp.getName());
					}
					entrustVo.setSampleNames(sampleNames);
					//检测项 和检测依据 列表里只显示第一个样品的
					SampleTmp sampleTmp = sampleTmpList.get(0);
					String teamIds = sampleTmp.getTeamIds();
					String checkTeam = "";
					if (teamIds != null) {
						String[] teamIdS = teamIds.split("、");
						for (String teamId : teamIdS) {
							Team team = teamMapper.selectById(Integer.valueOf(teamId));
							if (team != null) {
								checkTeam = checkTeam.equals("") ? team.getName() : (checkTeam + "、" + team.getName());
							}
						}
					}
					entrustVo.setCheckTeam(checkTeam);
					entrustVo.setCheckMethodNumber(sampleTmp.getMethodNumbers());
				}

				//查询报告,如果有上传的委托报告，用户可已进行下载
				QueryWrapper<EntityEnclosure> wrapper = new QueryWrapper<>();
				wrapper.eq("entity_type", EntityEnclosure.EntityType.ENTRUST_REPORT);
				wrapper.eq("entity_id", entrustVo.getId());
				wrapper.eq("is_deleted", 1);
				List<EntityEnclosure> entityEnclosureList = entityEnclosureMapper.selectList(wrapper);
				if (entityEnclosureList != null && entityEnclosureList.size() > 0) {
					EntityEnclosure entityEnclosure = entityEnclosureList.get(0);
					entrustVo.setAlias(entityEnclosure.getAlias());
					entrustVo.setEnclosureUrl(entityEnclosure.getPdfUrl());
					entrustVo.setExtName(entityEnclosure.getExtName());
				}
			}
		}
		pages.setRecords(list);
		return BaseResponse.okData(pages);
	}

	/**
	 * 置顶取消置顶
	 *
	 * @param id 委托表id
	 * @return
	 */
	public BaseResponse<String> setTopping(Integer id) {
		if (id == null) {
			return BaseResponse.errorMsg("参数错误");
		}
		Entrust entrust = entrustMapper.selectById(id);
		if (entrust == null) {
			return BaseResponse.errorMsg("数据错误");
		}
		String msg = "";
		if (entrust.getCreateTime().isEqual(entrust.getUpdateTime())) {
			entrust.setUpdateTime(LocalDateTime.now())
					.setIsTop(1);
			msg = "置顶成功";
		} else {
			entrust.setUpdateTime(entrust.getCreateTime())
					.setIsTop(0);
			msg = "取消置顶成功";
		}
		entrustMapper.updateById(entrust);
		return BaseResponse.okMsg(msg);
	}

	/**
	 * 新增委托
	 *
	 * @param query
	 * @return
	 */
	@Transactional
	@Override
	public BaseResponse<Entrust> create(EntrustQuery query) {
		LoginUser loginUser = userService.getLoginUser();
		if (loginUser == null) {
			return BaseResponse.noLogin("请登录账号");
		}
		if (query.getClientId() == null) {
			return BaseResponse.errorMsg("请选择委托单位");
		}
		if (query.getProjectId() == null) {
			if (StringUtils.isEmpty(query.getProjectName())) {
				return BaseResponse.errorMsg("请填写项目名称");
			}
			if (StringUtils.isEmpty(query.getProjectCode())) {
				return BaseResponse.errorMsg("请填写项目编号");
			}
		}
		if (query.getProjectId() != null) {
			Project project = projectMapper.selectById(query.getProjectId());
			if (project == null) {
				return BaseResponse.errorMsg("项目信息错误");
			}
		}
		if (StringUtils.isEmpty(query.getSendName()) || StringUtils.isEmpty(query.getSendPhone())) {
			return BaseResponse.errorMsg("请填写送样人信息");
		}
		if (query.getSendPhone().length() > 15) {
			return BaseResponse.errorMsg("送样人电话过长");
		}
		if (query.getRemark().length() > 1000) {
			return BaseResponse.errorMsg("备注信息过长");
		}

		Entrust entrust = new Entrust();
		BeanUtils.copyProperties(query, entrust);

		//对样品进行过校验
		if (query.getSampleTmpList() != null && query.getSampleTmpList().size() > 0) {
			for (SampleTmpQuery sampleTmpQuery : query.getSampleTmpList()) {
				if (StringUtils.isEmpty(sampleTmpQuery.getName())) {
					return BaseResponse.errorMsg("请填写样品名");
				}
				if (StringUtils.isEmpty(sampleTmpQuery.getSampleCode())) {
					return BaseResponse.errorMsg("请填写来样编号");
				}
				if (StringUtils.isEmpty(sampleTmpQuery.getSampleForm())) {
					return BaseResponse.errorMsg("请选择样品状态");
				}
				if (StringUtils.isEmpty(sampleTmpQuery.getTeamIds())) {
					return BaseResponse.errorMsg("请选择检测项目");
				}
				if (sampleTmpQuery.getWeight() == null) {
					return BaseResponse.errorMsg("请填写样品重量");
				}
				if (sampleTmpQuery.getWeight().toString().length()>10) {
					return BaseResponse.errorMsg("样品重量输入过长");
				}


				if (sampleTmpQuery.getWeightType() == null) {
					return BaseResponse.errorMsg("请选择重量单位");
				}
				if (StringUtils.isEmpty(sampleTmpQuery.getOrigin())) {
					return BaseResponse.errorMsg("请选择样品产地");
				}
			}
		}
		//生成委托编号
//        String entrustCode = commonService.createNo("WT", entrustMapper.counts());
		entrust.setEntrustCode(null)
				.setUserId(loginUser.getId())
				.setStatus(0)
				.setIsDelete(1)
				.setIsHandle(0)
				.setIsDistribution(0)
				.setCreateTime(LocalDateTime.now())
				.setUpdateTime(entrust.getCreateTime())
				.setSampleNum(query.getSampleTmpList().size())
				.setIsTop(0);
		entrustMapper.insert(entrust);
		if (query.getSampleTmpList() != null && query.getSampleTmpList().size() > 0) {
			List<SampleTmp> sampleTmpList = new ArrayList<>();
			for (SampleTmpQuery sampleTmpQuery : query.getSampleTmpList()) {
				SampleTmp sampleTmp = new SampleTmp();
				BeanUtils.copyProperties(sampleTmpQuery, sampleTmp);
				String teamIds = sampleTmpQuery.getTeamIds(); //选择的检测项id集合
				BigDecimal charge = new BigDecimal("0.00");
				if (teamIds != null) {
					String[] teamIdS = teamIds.split("、");
					List<String> teamNameList = new ArrayList<>();
					for (String teamId : teamIdS) {
						Team team = teamMapper.selectById(Integer.valueOf(teamId));
						if (team != null) {
							if (!teamNameList.contains(teamId)) {
								charge = charge.compareTo(BigDecimal.ZERO) == 0 ? team.getCharge() : (charge.add(team.getCharge()));
								teamNameList.add(team.getName());
							}
						}
					}
					//选择Mg必须选择Ca；选择Al必须选择Fe
					if (teamNameList.contains("MgO")) {
						if (!teamNameList.contains("CaO")) {
							return BaseResponse.errorMsg("选择MgO必须要选择CaO");
						}
					}
					if (teamNameList.contains("Al<sub>2</sub>O<sub>3</sub>")) {
						if (!teamNameList.contains("Fe<sub>2</sub>O<sub>3</sub>")) {
							return BaseResponse.errorMsg("选择Al2O3必须要选择Fe2O3");
						}
					}
					//选择比表面积必须选择密度
					if (teamNameList.contains("比表面积")) {
						QueryWrapper<Team> teamQW = new QueryWrapper<>();
						teamQW.eq("name", "密度");
						Team teamMD = teamMapper.selectOne(teamQW);
						if (teamMD == null) {
							return BaseResponse.errorMsg("请联系管理员配置密度检测项");
						}
						if (!teamNameList.contains("密度")) {
							return BaseResponse.errorMsg("选择比表面积必须选择密度");
						}
					}
				}
				//如果为加急委托，产值是正常的3倍
				if (entrust.getIsUrgent() == 1) {
					charge = charge.multiply(new BigDecimal(3));
				}
				sampleTmp.setEntrustId(entrust.getId())
						.setCharge(charge)
						.setCreateTime(LocalDateTime.now());
				sampleTmpList.add(sampleTmp);
			}
			sampleTmpService.saveBatch(sampleTmpList);
		} else {
			return BaseResponse.errorMsg("样品及检测项不可为空");
		}


		logsService.saveLog(SysLogs.ObjType.ENTRUST_LOG, entrust.getId(), "提交了委托单", null);
		//消息推送
		List<Integer> approvalIdList = sysApprovalMapper.getApprovalId("委托评审");
		if (approvalIdList == null) {
			return BaseResponse.errorMsg("委托评审信息错误");
		}
		String userIds = "";
		for (Integer approvalId : approvalIdList) {
			SysUser approvalUser = userService.getById(approvalId);
			if (approvalUser == null) {
				return BaseResponse.errorMsg("委托评审信息错误");
			}
			BaseResponse wrapper = userMessageService.sendMessage(approvalId, "您有一条新的委托单申请等待评审", entrust.getId(), SysUserMessage.MessageType.ENTRUST);
			if (wrapper.getCode() != 200) {
				return wrapper;
			}
			if (StringUtils.isNotBlank(approvalUser.getWxId())) {
				userIds = userIds.equals("") ? approvalUser.getWxId() : userIds + "|" + approvalUser.getWxId();
			}
		}
		//发送企业微信消息内容
//        Integer notice = (int)(1+Math.random()*(10-1+1));
//        String  URL = "https://lab.wisenergy.cn/lab-bangye?id="+entrust.getId()+"&path=/blanche/entrustdetail&notice="+notice;
//        String systemUrL = systemUrl.replace("U_R_L", URLUtil.encodeAll(URL));

		String systemUrL = weiXinService.createMessage(entrust.getId(), "/blanche/entrustdetail");
		String content = "你好！" + loginUser.getName() + "向您申请了一条新委托，请前往\n" +
				systemUrL + "进行评审。";
		System.out.println("**************************content**********************");
		System.out.println(content);
		weiXinService.sendTextMessage(userIds, content);

		return BaseResponse.okData(entrust);
	}

	public static void main(String[] args) {

		/*String URL = "https://lab.wisenergy.cn/lab-bangye?id=" + 12 + "&path=/blanche/entrustdetail&notice=" + 21;
		String a = URLUtil.encodeAll(URL);
		String b = URLUtil.encode(URL).replaceAll("&", "%26");
//        String a = URLUtil.encode(URL);
		System.out.println(a);
		System.out.println(b);*/


		/*String aa = "Mad=1.87  \n" +
				" Aad=21.04  \n" +
				" Vad=24.24  \n" +
				" Qnet,ad MJ/kg=23.14  \n" +
				" 焦渣=2";
		String[] str = aa.split("\n");
		Map<String,String> map = new HashMap<>();
		for(String str2 : str){
			String[] ele = str2.trim().split("=");
			map.put(ele[0], ele[1]);
		}
		System.out.println(map);*/


		BigDecimal endResult = new BigDecimal("2").divide(new BigDecimal("1"),10,BigDecimal.ROUND_HALF_UP);
		System.out.println(endResult);

	}

	/**
	 * 修改委托
	 *
	 * @param query
	 * @return
	 */
	@Transactional
	@Override
	public BaseResponse<Entrust> update(EntrustUpdateQuery query) {
		LoginUser loginUser = userService.getLoginUser();
		if (loginUser == null) {
			return BaseResponse.noLogin("请登录账号");
		}
		if (query.getId() == null) {
			return BaseResponse.errorMsg("参数错误");
		}
		if (query.getClientId() == null) {
			return BaseResponse.errorMsg("请选择委托单位");
		}
       /* if (query.getProjectId() == null || StringUtils.isEmpty(query.getProjectCode())) {
            if (StringUtils.isEmpty(query.getProjectName())) {
                return BaseResponse.errorMsg("请填写项目名称或选择已有项目");
            }
        }*/
		if (StringUtils.isEmpty(query.getSendName()) || StringUtils.isEmpty(query.getSendPhone())) {
			return BaseResponse.errorMsg("请填写送样人信息");
		}
		if (query.getSendPhone().length() > 15) {
			return BaseResponse.errorMsg("送样人电话过长");
		}
		if (query.getRemark().length() > 1000) {
			return BaseResponse.errorMsg("备注信息过长");
		}
		Entrust entrust = entrustMapper.selectById(query.getId());
		if (entrust == null) {
			return BaseResponse.errorMsg("信息错误");
		}

		if (entrust.getProjectId() == null) {
			if (query.getProjectId() == null) {
				return BaseResponse.errorMsg("请添加项目信息");
			}
			Project project = projectMapper.selectById(query.getProjectId());
			if (project == null) {
				return BaseResponse.errorMsg("项目信息有误");
			}
			entrust.setProjectId(project.getId())
					.setProjectCode(project.getCode())
					.setProjectName(project.getName());
		}



		//消息推送  和 判断是否符合修改权限 会用到
		List<Integer> approvalIdList = sysApprovalMapper.getApprovalId("委托评审");
		if (approvalIdList == null) {
			return BaseResponse.errorMsg("委托评审信息错误");
		}
		if (!approvalIdList.contains(loginUser.getId()) && entrust.getUserId() != loginUser.getId()) {
			return BaseResponse.errorMsg("没有修改权限");
		}
		//如果时评审人员修改   存入委托编号
		if (approvalIdList.contains(loginUser.getId())) {
			String entrustCode = entrust.getProjectCode() + "-" + query.getSampleList().get(0).getCementCode();
			entrust.setEntrustCode(entrustCode);
		}

		BeanUtils.copyProperties(query, entrust);
		//生成委托编号
//        String entrustCode = commonService.createNo("WT", entrustMapper.counts());
		if (entrust.getUserId() == loginUser.getId() && entrust.getStatus() == 2) {
			entrust.setStatus(0); //未评审
		} else if (approvalIdList.contains(loginUser.getId()) && entrust.getStatus() == 0 ) {
			if (StringUtils.isEmpty(query.getProjectType())) {
				return BaseResponse.errorMsg("请选择项目类型");
			}
			if (entrust.getProjectId() == null) {
				return BaseResponse.errorMsg("请选择项目");
			}
			entrust.setStatus(9); //修改待确认
			entrust.setProjectType(query.getProjectType());
		}
		entrust.setEntrustCode(null)
				.setUserId(entrust.getUserId())
				.setIsDelete(1)
				.setIsHandle(0)
				.setIsDistribution(0)
				.setCreateTime(LocalDateTime.now())
				.setUpdateTime(entrust.getCreateTime())
				.setSampleNum(query.getSampleList().size())
				.setIsTop(0);

		if (query.getSampleList() == null || query.getSampleList().size() <= 0) {
			return BaseResponse.errorMsg("样品及检测项不可为空");
		}

		if (query.getSampleList() != null && query.getSampleList().size() > 0) {
			//对样品信息进行判断
			for (SampleQuery sampleQuery : query.getSampleList()) {
				if (StringUtils.isEmpty(sampleQuery.getName())) {
					return BaseResponse.errorMsg("请填写样品名");
				}
				if (StringUtils.isEmpty(sampleQuery.getSampleCode())) {
					return BaseResponse.errorMsg("请填写来样编号");
				}
				if (StringUtils.isEmpty(sampleQuery.getSampleForm())) {
					return BaseResponse.errorMsg("请选择样品状态");
				}
				if (StringUtils.isEmpty(sampleQuery.getTeamIds())) {
					return BaseResponse.errorMsg("请选择检测项目");
				}
				if (sampleQuery.getWeight() == null) {
					return BaseResponse.errorMsg("请填写样品重量");
				}
				if (sampleQuery.getWeight().toString().length()>10) {
					return BaseResponse.errorMsg("样品重量输入过长");
				}

				if (sampleQuery.getWeightType() == null) {
					return BaseResponse.errorMsg("请选择重量单位");
				}
				if (StringUtils.isEmpty(sampleQuery.getOrigin())) {
					return BaseResponse.errorMsg("请选择样品产地");
				}
			}
			//删除之前的样品
			QueryWrapper<SampleTmp> queryWrapper = new QueryWrapper<>();
			queryWrapper.eq("entrust_id", entrust.getId());
			sampleTmpService.remove(queryWrapper);
			List<SampleTmp> sampleTmpList = new ArrayList<>();
			for (SampleQuery sampleQuery : query.getSampleList()) {
				SampleTmp sampleTmp = new SampleTmp();
				BeanUtils.copyProperties(sampleQuery, sampleTmp);
				String teamIds = sampleQuery.getTeamIds(); //选择的检测项id集合
				BigDecimal charge = new BigDecimal("0.00");
				if (teamIds != null) {
					String[] teamIdS = teamIds.split("、");
					List<String> teamNameList = new ArrayList<>();
					for (String teamId : teamIdS) {
						Team team = teamMapper.selectById(Integer.valueOf(teamId));
						if (team != null) {
							if (!teamNameList.contains(teamId)) {
								charge = charge.compareTo(BigDecimal.ZERO) == 0 ? team.getCharge() : (charge.add(team.getCharge()));
								teamNameList.add(team.getName());
							}
						}
					}
					//选择Mg必须选择Ca；选择Al必须选择Fe
					if (teamNameList.contains("MgO")) {
						if (!teamNameList.contains("CaO")) {
							return BaseResponse.errorMsg("选择MgO必须要选择CaO");
						}
					}
					if (teamNameList.contains("Al<sub>2</sub>O<sub>3</sub>")) {
						if (!teamNameList.contains("Fe<sub>2</sub>O<sub>3</sub>")) {
							return BaseResponse.errorMsg("选择Al2O3必须要选择Fe2O3");
						}
					}
					//选择比表面积必须选择密度
					if (teamNameList.contains("比表面积")) {
						QueryWrapper<Team> teamQW = new QueryWrapper<>();
						teamQW.eq("name", "密度");
						Team teamMD = teamMapper.selectOne(teamQW);
						if (teamMD == null) {
							return BaseResponse.errorMsg("请联系管理员配置密度检测项");
						}
						if (!teamNameList.contains("密度")) {
							return BaseResponse.errorMsg("选择比表面积必须选择密度");
						}
					}
				}
				//如果为加急委托，产值是正常的3倍
				if (entrust.getIsUrgent() == 1) {
					charge = charge.multiply(new BigDecimal(3));
				}
				sampleTmp.setEntrustId(entrust.getId())
						.setCharge(charge)
						.setCreateTime(LocalDateTime.now());
				sampleTmpList.add(sampleTmp);
				sampleTmpService.save(sampleTmp);
				sampleQuery.setId(sampleTmp.getId());

			}

			if (approvalIdList.contains(loginUser.getId()) && entrust.getStatus() == 9) {
				//当前平行样编号最大值
				String redisMaxCementCode = getMaxCementCode().getData();
				//保存样品信息
				BaseResponse results = saveSampleList(query.getSampleList(), redisMaxCementCode);
				if (results.getCode() != 200) {
					return results;
				}
			}

		} else {
			return BaseResponse.errorMsg("样品及检测项不可为空");
		}
		entrustMapper.updateById(entrust);

		logsService.saveLog(SysLogs.ObjType.ENTRUST_LOG, entrust.getId(), "修改委托单", null);

		//如果是委托人修改的
		if (entrust.getUserId() == loginUser.getId() && entrust.getStatus() == 0) {
			//委托人员修改委托，提示评审人员查看
			String userIds = "";
			for (Integer approvalId : approvalIdList) {
				SysUser approvalUser = userService.getById(approvalId);
				if (approvalUser == null) {
					return BaseResponse.errorMsg("委托评审信息错误");
				}
				BaseResponse wrapper = userMessageService.sendMessage(approvalId, "您有一条新的委托单申请等待评审", entrust.getId(), SysUserMessage.MessageType.ENTRUST);
				if (wrapper.getCode() != 200) {
					return wrapper;
				}
				if (StringUtils.isNotBlank(approvalUser.getWxId())) {
					userIds = userIds.equals("") ? approvalUser.getWxId() : userIds + "|" + approvalUser.getWxId();
				}
			}
			//发送企业微信消息内容
			String systemUrL = weiXinService.createMessage(entrust.getId(), "/blanche/entrustdetail");
			String content = "你好！" + loginUser.getName() + "向您修改了一条委托，" + "委托单号" + entrust.getEntrustCode() + "请前往\n" +
					systemUrL + "进行评审。";
			System.out.println("**************************content**********************");
			System.out.println(content);
			weiXinService.sendTextMessage(userIds, content);
		} else if (approvalIdList.contains(loginUser.getId()) && entrust.getStatus() == 9) {
			//如果是评审人员修改
			//评审人员修改委托，提示委托人查看
			SysUser createUser = userService.getById(entrust.getUserId());
			if (createUser == null) {
				return BaseResponse.errorMsg("委托人信息错误");
			}
			String userId = createUser.getWxId();
			BaseResponse wrapper = userMessageService.sendMessage(createUser.getId(), "您有一条新的委托信息被修改，请确认", entrust.getId(), SysUserMessage.MessageType.ENTRUST);
			if (wrapper.getCode() != 200) {
				return wrapper;
			}
			//发送企业微信消息内容
			String systemUrL = weiXinService.createMessage(entrust.getId(), "/blanche/entrustdetail");
			String content = "你好！" + loginUser.getName() + "修改了您的一条委托，" + "委托单号" + entrust.getEntrustCode() + "请前往\n" +
					systemUrL + "进行查看。";
			System.out.println("**************************content**********************");
			System.out.println(content);
			weiXinService.sendTextMessage(userId, content);

			//记录审批信息
			EntrustApproval entrustApproval = new EntrustApproval();
			entrustApproval.setEntrustId(entrust.getId())
					.setUserId(loginUser.getId())
					.setOpinion(query.getAuditReason())
					.setStatus(1)
					.setCreateTime(LocalDateTime.now())
					.setRemark(null);
			approvalMapper.insert(entrustApproval);
			//处理消息
			userMessageService.dealMessage(loginUser.getId(), entrust.getId(), SysUserMessage.MessageType.ENTRUST);
		}
		return BaseResponse.okData(entrust);
	}

	/**
	 * 删除委托样品
	 *
	 * @param sampleId
	 * @return
	 */
	@Override
	@Transactional
	public BaseResponse<String> deleteSample(Integer sampleId) {
		LoginUser loginUser = userService.getLoginUser();
		if (loginUser == null) {
			return BaseResponse.noLogin("请登录账号");
		}
		List<Integer> approvalIdList = sysApprovalMapper.getApprovalId("委托评审");
		if (approvalIdList == null) {
			return BaseResponse.errorMsg("委托评审信息错误");
		}
		if (!approvalIdList.contains(loginUser.getId())) {
			return BaseResponse.errorMsg("权限不足");
		}
		if (sampleId == null) {
			return BaseResponse.errorMsg("参数错误");
		}
		SampleTmp sampleTmp = sampleTmpService.getById(sampleId);
		if (sampleTmp == null || sampleTmp.getEntrustId() == null) {
			return BaseResponse.errorMsg("信息错误");
		}
		Entrust entrust = entrustMapper.selectById(sampleTmp.getEntrustId());
		if (entrust == null) {
			return BaseResponse.errorMsg("委托信息错误");
		}
		if (entrust.getStatus() != 0) {
			return BaseResponse.errorMsg("状态不允许删除");
		}
		QueryWrapper<SampleTmp> queryWrapper = new QueryWrapper<>();
		queryWrapper.eq("entrust_id", sampleTmp.getEntrustId());
		Integer conunts = sampleTmpService.count(queryWrapper);
		if (conunts < 2) {
			return BaseResponse.errorMsg("至少保留一个样品信息");
		}
		sampleTmpService.removeById(sampleId);
		return BaseResponse.okData("删除成功");
	}

	/**
	 * 委托详情
	 *
	 * @param id
	 * @return
	 */
	@Override
	public BaseResponse<EntrustVo> getDtail(Integer id) {
		if (id == null) {
			return BaseResponse.errorMsg("参数错误");
		}
		EntrustVo entrustVo = entrustMapper.getDetail(id);
		if (entrustVo.getStatus() != 0) {
			QueryWrapper<Sample> sampleWrapper = new QueryWrapper<>();
			sampleWrapper.eq("entrust_id", entrustVo.getId());
			sampleWrapper.orderByAsc("id");
			List<Sample> sampleList = sampleService.list(sampleWrapper);
			List<SampleVo> sampleVoList = new ArrayList<>();
			List<SampleVo> sampleHandleVoList = new ArrayList<>();
			if (sampleList != null && sampleList.size() > 0) {
				for (Sample sample : sampleList) {
					SampleVo sampleVo = new SampleVo();
					BeanUtils.copyProperties(sample, sampleVo);
					String teamIds = sample.getTeamIds();
					String teamName = "";
					//没有派发的列表
					List<SampleTeamGroupVo> sampleTeamGroupVoList = new ArrayList<>();
					if (teamIds != null) {
						String[] teamIdS = teamIds.split("、");
						List<Integer> teamGroupIdList = new ArrayList<>();
						for (String teamId : teamIdS) {
							TeamVo teamVo = teamMapper.getDetail(Integer.valueOf(teamId));
							//如果本所编号不相等 则为副样，副样需要判断检测项是否需要平行样，如果不需要不用进行派发检测
							if (!sample.getCementCode().equals(sample.getParallelCode()) && teamVo.getIsParallel() == 0) {
								continue;
							}
							if (teamVo != null && teamVo.getIsPf() != 0) {
								if (!teamGroupIdList.contains(teamVo.getGroupId())) {
									teamGroupIdList.add(teamVo.getGroupId());
									SampleTeamGroupVo sampleTeamGroupVo = new SampleTeamGroupVo();
									sampleTeamGroupVo.setTeamGroupId(teamVo.getGroupId());
									sampleTeamGroupVo.setTeamGroupName(teamVo.getGroupName());

									SampleDistributionTeamVo distributionTeamVo = new SampleDistributionTeamVo();
									distributionTeamVo.setTeamId(teamVo.getId());
									distributionTeamVo.setTeamName(teamVo.getName());

									List<SampleDistributionTeamVo> sampleDistributionTeamVoList = new ArrayList<>();
									sampleDistributionTeamVoList.add(distributionTeamVo);
									sampleTeamGroupVo.setSampleDistributionTeamVoList(sampleDistributionTeamVoList);

									sampleTeamGroupVoList.add(sampleTeamGroupVo);
								} else {
									SampleTeamGroupVo sampleTeamGroupVo = null;
									for (SampleTeamGroupVo vo : sampleTeamGroupVoList) {
										if (vo.getTeamGroupId() == teamVo.getGroupId()) {
											sampleTeamGroupVo = vo;
										}
									}
									SampleDistributionTeamVo distributionTeamVo = new SampleDistributionTeamVo();
									distributionTeamVo.setTeamId(teamVo.getId());
									distributionTeamVo.setTeamName(teamVo.getName());
									List<SampleDistributionTeamVo> sampleDistributionTeamVoList = sampleTeamGroupVo.getSampleDistributionTeamVoList();
									sampleDistributionTeamVoList.add(distributionTeamVo);
									sampleTeamGroupVo.setSampleDistributionTeamVoList(sampleDistributionTeamVoList);
								}
							}
							teamName = teamName.equals("") ? teamVo.getName() : (teamName + "、" + teamVo.getName());
						}
					}
					sampleVo.setTeamName(teamName);
					//评审人员可以查看所有的检测项内容
					List<SampleDistributionTeamVo> sampleDistributionTeamVoList =
							distributionMapper.getDistributionTeamList(sample.getId(), null);
					if (sampleDistributionTeamVoList != null && sampleDistributionTeamVoList.size() > 0) {
						//已经派发过的列表
						List<SampleTeamGroupVo> sampleTeamGroupVoListed = new ArrayList<>();
						List<Integer> teamGroupIdList = new ArrayList<>();
						//todo 分组可以在优化一下
//						Map<Integer,List<SampleDistributionTeamVo>> DistributionTeamVoMap = sampleDistributionTeamVoList.stream().map(Co)
						for (SampleDistributionTeamVo sampleDistributionTeamVo : sampleDistributionTeamVoList) {
							SampleDistribution sampleDistribution = distributionMapper.selectById(sampleDistributionTeamVo.getDistributionId());
							TeamGroup teamGroup = teamGroupMapper.selectById(sampleDistribution.getTeamGroupId());
							if (!teamGroupIdList.contains(sampleDistribution.getTeamGroupId())) {
								teamGroupIdList.add(sampleDistribution.getTeamGroupId());
								SampleTeamGroupVo sampleTeamGroupVo = new SampleTeamGroupVo();
								sampleTeamGroupVo.setTeamGroupId(sampleDistribution.getTeamGroupId());
								sampleTeamGroupVo.setTeamGroupName(teamGroup.getName());
								List<SampleDistributionTeamVo> sdtvList = new ArrayList<>();
								sdtvList.add(sampleDistributionTeamVo);
								sampleTeamGroupVo.setSampleDistributionTeamVoList(sdtvList);

								sampleTeamGroupVoListed.add(sampleTeamGroupVo);
							} else {
								SampleTeamGroupVo sampleTeamGroupVo2 = null;
								for (SampleTeamGroupVo vo : sampleTeamGroupVoListed) {
									if (vo.getTeamGroupId() == sampleDistribution.getTeamGroupId()) {
										sampleTeamGroupVo2 = vo;
									}
								}
								List<SampleDistributionTeamVo> sdtvList = sampleTeamGroupVo2.getSampleDistributionTeamVoList();
								sdtvList.add(sampleDistributionTeamVo);
								sampleTeamGroupVo2.setSampleDistributionTeamVoList(sdtvList);
								//sampleTeamGroupVoListed.add(sampleTeamGroupVo2);
							}
						}
						sampleVo.setSampleTeamGroupVoList(sampleTeamGroupVoListed);
					} else {
						sampleVo.setSampleTeamGroupVoList(sampleTeamGroupVoList);
					}
					sampleVoList.add(sampleVo);
					//样品处理列表只展示主样信息
					if (sample.getIsParallel() != 0) {
						if (sample.getCementCode().equals(sample.getParallelCode())) {
							sampleHandleVoList.add(sampleVo);
						}
					} else {
						sampleHandleVoList.add(sampleVo);
					}
				}
			}
			entrustVo.setSampleList(sampleVoList);//样品列表（展示平行样样品）
			entrustVo.setSampleHandleList(sampleHandleVoList);//处理样品列表（不展示平行样样品）
			//委托评审意见
			QueryWrapper<EntrustApproval> approvalWrapper = new QueryWrapper<>();
			approvalWrapper.eq("entrust_id", entrustVo.getId());
			approvalWrapper.orderByDesc("id");
			List<EntrustApproval> approvalList = approvalMapper.selectList(approvalWrapper);
			EntrustApproval approval = approvalList.get(0);
			entrustVo.setOpinion(approval.getOpinion());
		} else {
			QueryWrapper<SampleTmp> queryWrapper = new QueryWrapper<>();
			queryWrapper.eq("entrust_id", entrustVo.getId());
			List<SampleTmp> sampleTmpList = sampleTmpService.list(queryWrapper);
			List<SampleTmpVo> sampleTmpVoList = new ArrayList<>();
			if (sampleTmpList != null && sampleTmpList.size() > 0) {
				for (SampleTmp sampleTmp : sampleTmpList) {
					SampleTmpVo sampleTmpVo = new SampleTmpVo();
					BeanUtils.copyProperties(sampleTmp, sampleTmpVo);
					String teamIds = sampleTmp.getTeamIds();
					String teamName = "";
					if (teamIds != null) {
						String[] teamIdS = teamIds.split("、");
						for (String teamId : teamIdS) {
							Team team = teamMapper.selectById(Integer.valueOf(teamId));
							if (team != null) {
								team.getName();
								teamName = teamName.equals("") ? team.getName() : (teamName + "、" + team.getName());
							}
						}
					}
					sampleTmpVo.setTeamName(teamName);
					sampleTmpVoList.add(sampleTmpVo);
				}
			}
			entrustVo.setSampleTmpList(sampleTmpVoList);
		}

		LoginUser loginUser = userService.getLoginUser();
		if (loginUser == null) {
			return BaseResponse.noLogin("请登录账号");
		}
		userMessageService.checkMessage(loginUser.getId(), id, SysUserMessage.MessageType.ENTRUST);
		return BaseResponse.okData(entrustVo);
	}

	/**
	 * 详情-基本信息
	 *
	 * @param id
	 * @return
	 */
	public BaseResponse<EntrustVo> getBaseDtail(Integer id) {
		LoginUser loginUser = userService.getLoginUser();
		if (loginUser == null) {
			return BaseResponse.noLogin("请登录账号");
		}
		if (id == null) {
			return BaseResponse.errorMsg("参数错误");
		}
		EntrustVo entrustVo = entrustMapper.getDetail(id);
		if (entrustVo.getStatus() != 0) {
			QueryWrapper<Sample> sampleWrapper = new QueryWrapper<>();
			sampleWrapper.eq("entrust_id", entrustVo.getId());
			sampleWrapper.orderByAsc("id");
			List<Sample> sampleList = sampleService.list(sampleWrapper);
			List<SampleVo> sampleVoList = new ArrayList<>();
			if (sampleList != null && sampleList.size() > 0) {
				for (Sample sample : sampleList) {
					SampleVo sampleVo = new SampleVo();
					BeanUtils.copyProperties(sample, sampleVo);
					String teamIds = sample.getTeamIds();
					String teamName = "";
					//没有派发的列表
					List<SampleTeamGroupVo> sampleTeamGroupVoList = new ArrayList<>();
					if (teamIds != null) {
						String[] teamIdS = teamIds.split("、");
						List<Integer> teamGroupIdList = new ArrayList<>();
						for (String teamId : teamIdS) {
							TeamVo teamVo = teamMapper.getDetail(Integer.valueOf(teamId));
							if (teamVo != null) {
								if (!teamGroupIdList.contains(teamVo.getGroupId())) {
									teamGroupIdList.add(teamVo.getGroupId());
									SampleTeamGroupVo sampleTeamGroupVo = new SampleTeamGroupVo();
									sampleTeamGroupVo.setTeamGroupId(teamVo.getGroupId());
									sampleTeamGroupVo.setTeamGroupName(teamVo.getGroupName());

									SampleDistributionTeamVo distributionTeamVo = new SampleDistributionTeamVo();
									distributionTeamVo.setTeamId(teamVo.getId());
									distributionTeamVo.setTeamName(teamVo.getName());

									List<SampleDistributionTeamVo> sampleDistributionTeamVoList = new ArrayList<>();
									sampleDistributionTeamVoList.add(distributionTeamVo);
									sampleTeamGroupVo.setSampleDistributionTeamVoList(sampleDistributionTeamVoList);

									sampleTeamGroupVoList.add(sampleTeamGroupVo);
								} else {
									SampleTeamGroupVo sampleTeamGroupVo = null;
									for (SampleTeamGroupVo vo : sampleTeamGroupVoList) {
										if (vo.getTeamGroupId() == teamVo.getGroupId()) {
											sampleTeamGroupVo = vo;
										}
									}
									SampleDistributionTeamVo distributionTeamVo = new SampleDistributionTeamVo();
									distributionTeamVo.setTeamId(teamVo.getId());
									distributionTeamVo.setTeamName(teamVo.getName());
									List<SampleDistributionTeamVo> sampleDistributionTeamVoList = sampleTeamGroupVo.getSampleDistributionTeamVoList();
									sampleDistributionTeamVoList.add(distributionTeamVo);
									sampleTeamGroupVo.setSampleDistributionTeamVoList(sampleDistributionTeamVoList);
								}

								teamName = teamName.equals("") ? teamVo.getName() : (teamName + "、" + teamVo.getName());
							}
						}

					}
					sampleVo.setTeamName(teamName);
					sampleVo.setSampleTeamGroupVoList(sampleTeamGroupVoList);
					sampleVoList.add(sampleVo);
				}
			}
			entrustVo.setSampleList(sampleVoList);//样品列表（展示平行样样品）

			//查询报告,如果有上传的委托报告，用户可已进行下载
			QueryWrapper<EntityEnclosure> wrapper = new QueryWrapper<>();
			wrapper.eq("entity_type", EntityEnclosure.EntityType.ENTRUST_REPORT);
			wrapper.eq("entity_id", entrustVo.getId());
			wrapper.eq("is_deleted", 1);
			List<EntityEnclosure> entityEnclosureList = entityEnclosureMapper.selectList(wrapper);
			if (entityEnclosureList != null && entityEnclosureList.size() > 0) {
				EntityEnclosure entityEnclosure = entityEnclosureList.get(0);
				entrustVo.setAlias(entityEnclosure.getAlias());
				entrustVo.setEnclosureUrl(entityEnclosure.getPdfUrl());
				entrustVo.setExtName(entityEnclosure.getExtName());
			}
			//委托评审意见
			QueryWrapper<EntrustApproval> approvalWrapper = new QueryWrapper<>();
			approvalWrapper.eq("entrust_id", entrustVo.getId());
			approvalWrapper.orderByDesc("id");
			List<EntrustApproval> approvalList = approvalMapper.selectList(approvalWrapper);
			EntrustApproval approval = approvalList.get(0);
			entrustVo.setOpinion(approval.getOpinion());
		} else {
			QueryWrapper<SampleTmp> queryWrapper = new QueryWrapper<>();
			queryWrapper.eq("entrust_id", entrustVo.getId());
			List<SampleTmp> sampleTmpList = sampleTmpService.list(queryWrapper);
			List<SampleTmpVo> sampleTmpVoList = new ArrayList<>();
			if (sampleTmpList != null && sampleTmpList.size() > 0) {
				for (SampleTmp sampleTmp : sampleTmpList) {
					SampleTmpVo sampleTmpVo = new SampleTmpVo();
					BeanUtils.copyProperties(sampleTmp, sampleTmpVo);
					String teamIds = sampleTmp.getTeamIds();
					String teamName = "";
					if (teamIds != null) {
						String[] teamIdS = teamIds.split("、");
						for (String teamId : teamIdS) {
							Team team = teamMapper.selectById(Integer.valueOf(teamId));
							if (team != null) {
								team.getName();
								teamName = teamName.equals("") ? team.getName() : (teamName + "、" + team.getName());
							}
						}
					}
					sampleTmpVo.setTeamName(teamName);
					sampleTmpVoList.add(sampleTmpVo);
				}
			}
			entrustVo.setSampleTmpList(sampleTmpVoList);
		}
		return BaseResponse.okData(entrustVo);
	}

	/**
	 * 委托客户查看基本信息
	 *
	 * @param id
	 * @return
	 */
	public BaseResponse<EntrustVo> getBaseDtailClient(Integer id) {
		LoginUser loginUser = userService.getLoginUser();
		if (loginUser == null) {
			return BaseResponse.noLogin("请登录账号");
		}
		if (id == null) {
			return BaseResponse.errorMsg("参数错误");
		}
		EntrustVo entrustVo = entrustMapper.getDetail(id);

		QueryWrapper<SampleTmp> sampleTmpQueryWrapper = new QueryWrapper<>();
		sampleTmpQueryWrapper.eq("entrust_id", entrustVo.getId());
		List<SampleTmp> sampleList = sampleTmpService.list(sampleTmpQueryWrapper);
		List<SampleTmpVo> sampleVoList = new ArrayList<>();
		if (sampleList != null && sampleList.size() > 0) {
			for (SampleTmp sample : sampleList) {
				SampleTmpVo sampleVo = new SampleTmpVo();
				BeanUtils.copyProperties(sample, sampleVo);

				QueryWrapper<Sample> sampleQueryWrapper = new QueryWrapper<>();
				sampleQueryWrapper.eq("entrust_id", entrustVo.getId());
				sampleQueryWrapper.eq("sample_id", sample.getId());
				List<Sample> sampleList1 = sampleService.list(sampleQueryWrapper);
				if (sampleList1 != null && sampleList1.size() > 0) {
					Sample sample1 = sampleList1.get(0);
					sampleVo.setParallelCode(sample1.getCementCode());
				} else {
					sampleVo.setParallelCode(null);
				}
				String teamIds = sample.getTeamIds();
				String teamName = "";
				if (teamIds != null) {
					String[] teamIdS = teamIds.split("、");
					for (String teamId : teamIdS) {
						Team team = teamMapper.selectById(Integer.valueOf(teamId));
						if (team != null) {
							team.getName();
							teamName = teamName.equals("") ? team.getName() : (teamName + "、" + team.getName());
						}
					}
				}
				sampleVo.setTeamName(teamName);
				sampleVoList.add(sampleVo);
			}
		}
		entrustVo.setSampleTmpList(sampleVoList);
		//查询报告,如果有上传的委托报告，用户可已进行下载
		QueryWrapper<EntityEnclosure> wrapper = new QueryWrapper<>();
		wrapper.eq("entity_type", EntityEnclosure.EntityType.ENTRUST_REPORT);
		wrapper.eq("entity_id", entrustVo.getId());
		wrapper.eq("is_deleted", 1);
		List<EntityEnclosure> entityEnclosureList = entityEnclosureMapper.selectList(wrapper);
		if (entityEnclosureList != null && entityEnclosureList.size() > 0) {
			EntityEnclosure entityEnclosure = entityEnclosureList.get(0);
			entrustVo.setAlias(entityEnclosure.getAlias());
			entrustVo.setEnclosureUrl(entityEnclosure.getPdfUrl());
			entrustVo.setExtName(entityEnclosure.getExtName());
		}
		//委托评审意见
		QueryWrapper<EntrustApproval> approvalWrapper = new QueryWrapper<>();
		approvalWrapper.eq("entrust_id", entrustVo.getId());
		approvalWrapper.orderByDesc("id");
		List<EntrustApproval> approvalList = approvalMapper.selectList(approvalWrapper);
		if (approvalList != null && approvalList.size() > 0) {
			EntrustApproval approval = approvalList.get(0);
			entrustVo.setOpinion(approval.getOpinion());
		}

		return BaseResponse.okData(entrustVo);
	}

	/**
	 * 详情-样品处理信息
	 *
	 * @param id
	 * @return
	 */
	public BaseResponse<List<SampleHandleVo>> getSampleHandleDtail(Integer id) {
		LoginUser loginUser = userService.getLoginUser();
		if (loginUser == null) {
			return BaseResponse.noLogin("请登录账号");
		}

		if (id == null) {
			return BaseResponse.errorMsg("参数错误");
		}
		Map<String, Object> params = new HashMap<>();
		params.put("id", id);
		List<SampleHandleVo> sampleHandleList = sampleHandleMapper.getSampleHandleList(params);
		if (sampleHandleList != null && sampleHandleList.size() > 0) {
			for (SampleHandleVo shVo : sampleHandleList) {
				Handle handle = handleMapper.selectById(shVo.getHandleId());
				if (handle == null) {
					return BaseResponse.errorMsg("选择的处理项信息有误");
				}
				String contentName = handle.getName();
				shVo.setContentName(contentName);
			}
		}
		return BaseResponse.okData(sampleHandleList);
	}

	/**
	 * 详情-检测任务信息
	 *
	 * @param id
	 * @return
	 */
	public BaseResponse<List<SampleVo>> getSampleCheckDetail(Integer id) {
		LoginUser loginUser = userService.getLoginUser();
		if (loginUser == null) {
			return BaseResponse.noLogin("请登录账号");
		}

		if (id == null) {
			return BaseResponse.errorMsg("参数错误");
		}

		Entrust entrust = entrustMapper.selectById(id);
		if (entrust == null) {
			return BaseResponse.errorMsg("信息错误");
		}
		List<Sample> sampleList = sampleMapper.getSampleCheckList(id);
		List<SampleVo> sampleVoList = new ArrayList<>();

		if (sampleList != null && sampleList.size() > 0) {
			for (Sample sample : sampleList) {
				SampleVo sampleVo = new SampleVo();
				BeanUtils.copyProperties(sample, sampleVo);
				List<SampleDistributionTeamVo> sampleDistributionTeamVoList =
						distributionMapper.getDistributionTeamResultList(sample.getId(), null);
				if (sampleDistributionTeamVoList != null && sampleDistributionTeamVoList.size() > 0) {
					//已经派发过的列表
                   /* List<SampleTeamGroupVo> sampleTeamGroupVoListed = new ArrayList<>();
                    List<Integer> teamGroupIdList = new ArrayList<>();
                    for(SampleDistributionTeamVo sampleDistributionTeamVo : sampleDistributionTeamVoList){
                        SampleDistribution sampleDistribution = distributionMapper.selectById(sampleDistributionTeamVo.getDistributionId());
                        TeamGroup teamGroup = teamGroupMapper.selectById(sampleDistribution.getTeamGroupId());
                        if(!teamGroupIdList.contains(sampleDistribution.getTeamGroupId())){
                            teamGroupIdList.add(sampleDistribution.getTeamGroupId());
                            SampleTeamGroupVo sampleTeamGroupVo = new SampleTeamGroupVo();
                            sampleTeamGroupVo.setTeamGroupId(sampleDistribution.getTeamGroupId());
                            sampleTeamGroupVo.setTeamGroupName(teamGroup.getName());
                            List<SampleDistributionTeamVo> sdtvList = new ArrayList<>();
                            sdtvList.add(sampleDistributionTeamVo);
                            sampleTeamGroupVo.setSampleDistributionTeamVoList(sdtvList);

                            sampleTeamGroupVoListed.add(sampleTeamGroupVo);
                        }else{
                            SampleTeamGroupVo sampleTeamGroupVo2 = null;
                            for(SampleTeamGroupVo vo : sampleTeamGroupVoListed){
                                if(vo.getTeamGroupId() == sampleDistribution.getTeamGroupId() ){
                                    sampleTeamGroupVo2 = vo;
                                }
                            }
                            List<SampleDistributionTeamVo> sdtvList = sampleTeamGroupVo2.getSampleDistributionTeamVoList();
                            sdtvList.add(sampleDistributionTeamVo);
                            sampleTeamGroupVo2.setSampleDistributionTeamVoList(sdtvList);
                            sampleTeamGroupVoListed.add(sampleTeamGroupVo2);
                        }
                    }*/
					sampleVo.setSampleDistributionTeamVoList(sampleDistributionTeamVoList);
//                    sampleVo.setSampleDistributionTeamVoList(sampleDistributionTeamVoList);
				}

				sampleVoList.add(sampleVo);
			}
		}
		return BaseResponse.okData(sampleVoList);
	}

	/**
	 * 详情-检测结果信息
	 *
	 * @param id
	 * @return
	 */
	public BaseResponse<List<SampleVo>> getSampleCheckDtail(Integer id) {
		LoginUser loginUser = userService.getLoginUser();
		if (loginUser == null) {
			return BaseResponse.noLogin("请登录账号");
		}
		if (id == null) {
			return BaseResponse.errorMsg("参数错误");
		}
		Entrust entrust = entrustMapper.selectById(id);
		if (entrust == null) {
			return BaseResponse.errorMsg("信息错误");
		}
		//只看主样  对应的最终结果值
		List<Sample> sampleList = sampleMapper.getCheckSampleList(id);
		if (sampleList == null || sampleList.size() == 0) {
			return BaseResponse.errorMsg("样品等待检测");
		}
		List<SampleVo> sampleVoList = new ArrayList<>();

		if (sampleList != null && sampleList.size() > 0) {
			for (Sample sample : sampleList) {
				SampleVo sampleVo = new SampleVo();
				BeanUtils.copyProperties(sample, sampleVo);
				List<SampleDistributionTeamVo> sampleDistributionTeamVoList =
						distributionMapper.getDistributionTeamResultList(sample.getId(), null);

				if (sampleDistributionTeamVoList != null && sampleDistributionTeamVoList.size() > 0) {
					//已经派发过的列表
					List<SampleTeamGroupVo> sampleTeamGroupVoListed = new ArrayList<>();
					List<Integer> teamGroupIdList = new ArrayList<>();
					for (SampleDistributionTeamVo sampleDistributionTeamVo : sampleDistributionTeamVoList) {
						SampleDistribution sampleDistribution = distributionMapper.selectById(sampleDistributionTeamVo.getDistributionId());
						//该检测项的误差结果值和最终结果值

						//获取其他平行样的结果值  如果是平行样 必须都校核通过才可以显示结果。有一个点击保存 也不能显示结果
						int noCheckOver = 0;
						List<Sample> otherSampleList = sampleMapper.getOtherSampleList(entrust.getId(), sample.getCementCode(), sample.getId());
						if (otherSampleList != null && otherSampleList.size() > 0) {
							for (Sample s : otherSampleList) {
								QueryWrapper<SampleDistribution> ortherWrapper = new QueryWrapper<>();
								ortherWrapper.eq("entrust_id", entrust.getId());
								ortherWrapper.eq("sample_id", s.getId());
								ortherWrapper.eq("team_group_id", sampleDistribution.getTeamGroupId());
								ortherWrapper.eq("team_id", sampleDistribution.getTeamId());
								List<SampleDistribution> ortherDdistributionList = distributionMapper.selectList(ortherWrapper);
								if (ortherDdistributionList.size() > 1) {
									return BaseResponse.errorMsg("检测派发表检测项信息重复");
								}
								if (ortherDdistributionList != null && ortherDdistributionList.size() > 0) {
									SampleDistribution other = ortherDdistributionList.get(0);
									if(other.getCheckId() == null || other.getStatus() != 4){
										noCheckOver = noCheckOver + 1;
									}
								}
							}
						}

						if (sampleDistribution.getCheckId() != null && noCheckOver == 0) {
							SampleDistributionCheck disCheck = sampleDistributionCheckMapper.selectById(sampleDistribution.getCheckId());
							sampleDistributionTeamVo.setLastResult(disCheck.getLastResult());
						} else {
							sampleDistributionTeamVo.setLastResult(null);
						}

						TeamGroup teamGroup = teamGroupMapper.selectById(sampleDistribution.getTeamGroupId());
						if (!teamGroupIdList.contains(sampleDistribution.getTeamGroupId())) {
							teamGroupIdList.add(sampleDistribution.getTeamGroupId());
							SampleTeamGroupVo sampleTeamGroupVo = new SampleTeamGroupVo();
							sampleTeamGroupVo.setTeamGroupId(sampleDistribution.getTeamGroupId());
							sampleTeamGroupVo.setTeamGroupName(teamGroup.getName());
							List<SampleDistributionTeamVo> sdtvList = new ArrayList<>();
							sdtvList.add(sampleDistributionTeamVo);
							sampleTeamGroupVo.setSampleDistributionTeamVoList(sdtvList);

							sampleTeamGroupVoListed.add(sampleTeamGroupVo);
						} else {
							SampleTeamGroupVo sampleTeamGroupVo2 = null;
							for (SampleTeamGroupVo vo : sampleTeamGroupVoListed) {
								if (vo.getTeamGroupId() == sampleDistribution.getTeamGroupId()) {
									sampleTeamGroupVo2 = vo;
								}
							}
							List<SampleDistributionTeamVo> sdtvList = sampleTeamGroupVo2.getSampleDistributionTeamVoList();
							sdtvList.add(sampleDistributionTeamVo);
							sampleTeamGroupVo2.setSampleDistributionTeamVoList(sdtvList);
//                            sampleTeamGroupVoListed.add(sampleTeamGroupVo2);
						}
					}
					sampleVo.setSampleTeamGroupVoList(sampleTeamGroupVoListed);
				}
				sampleVoList.add(sampleVo);
			}
		}
		return BaseResponse.okData(sampleVoList);
	}


	/**
	 * 获取样品表里最大的平行样编号
	 *
	 * @return
	 */
	@Override
	public BaseResponse<String> getMaxCementCode() {
		Object obj = redisUtil.getString("maxCementCode");
		String maxCementCode = "";
		String regex = "[0-9]{2}\\-[0-9]{4}";
		if (obj == null || !obj.toString().matches(regex)) {
			maxCementCode = sampleMapper.getMaxCementCode();
		} else {
			maxCementCode = obj.toString();
		}
		//校验获取的最大本所编号
		SimpleDateFormat sdf = new SimpleDateFormat("yy");
		Date date = new Date();
		String yesrString = sdf.format(date);
		if (maxCementCode == null) {  //如果数据库没有本所编号，初始化本所编号
			maxCementCode = yesrString + "-0000";
		} else {
			String yearStr = commonService.getIntegerYear(maxCementCode);
			if (!yearStr.equals(yesrString)) { //如果数据库本所编号年份和当前不一样，初始化本所编号
				maxCementCode = yesrString + "-0000";
			}
		}
		System.out.println("redis获取最大值时的最大本所编号*******************************************" + maxCementCode);
		redisUtil.setString("maxCementCode", maxCementCode);
		return BaseResponse.okData(maxCementCode);
	}

	/**
	 * 委托评审
	 *
	 * @param query
	 * @return
	 */
	@Transactional
	@Override
	public BaseResponse<String> approval(ApprovalQuery query) {
		if (query.getId() == null || query.getIsAgree() == null) {
			return BaseResponse.errorMsg("参数错误");
		}
		LoginUser loginUser = userService.getLoginUser();
		if (loginUser == null) {
			return BaseResponse.noLogin("请登录账号");
		}
		List<Integer> approvalIdList = sysApprovalMapper.getApprovalId("委托评审");
		if (approvalIdList == null) {
			return BaseResponse.errorMsg("委托评审信息错误");
		}
		if (!approvalIdList.contains(loginUser.getId())) {
			return BaseResponse.errorMsg("没有评审权限");
		}
		EntrustVo entrustVo = entrustMapper.getDetail(query.getId());
		if (entrustVo == null) {
			return BaseResponse.errorMsg("信息错误");
		}
		if (entrustVo.getStatus() != 0) {
			return BaseResponse.errorMsg("当前状态不允许评审");
		}
		Entrust entrust = entrustMapper.selectById(entrustVo.getId());
		if (entrust.getProjectId() == null) {
			if (query.getProjectId() == null) {
				return BaseResponse.errorMsg("请添加项目信息");
			}
			Project project = projectMapper.selectById(query.getProjectId());
			if (project == null) {
				return BaseResponse.errorMsg("项目信息有误");
			}
			entrust.setProjectId(project.getId())
					.setProjectCode(project.getCode())
					.setProjectName(project.getName());
		}
		//获取产值信息
		QueryWrapper<NormProduction> queryWrapper = new QueryWrapper<>();
		queryWrapper.eq("type", 0);
		queryWrapper.eq("status", 1);
		List<NormProduction> normProduction = iNormProductionService.list(queryWrapper);
		if (normProduction == null || normProduction.size() != 1) {
			//还原本所编号最大值
			return BaseResponse.errorMsg("请配置处理项的产值信息!");
		}
      /*  //如果项目id或者项目编号为空，根据项目名称获取项目信息
        if (StringUtils.isEmpty(entrust.getProjectCode())) {
            ProjectVo projectVo = projectMapper.getByName(entrust.getProjectName());
            if (projectVo == null) {
                return BaseResponse.errorMsg("请添加项目信息");
            }
            entrust.setProjectId(projectVo.getId())
                    .setProjectCode(projectVo.getCode())
                    .setProjectName(projectVo.getName());
        }*/

		//审批后生成委托编号
		String entrustCode = entrust.getProjectCode() + "-" + query.getSampleList().get(0).getCementCode();
		entrust.setEntrustCode(entrustCode);
		if (query.getIsAgree() == 1) { //通过
			if (StringUtils.isEmpty(query.getProjectType())) {
				return BaseResponse.errorMsg("请选择项目类型");
			}
			//当前平行样编号最大值
			String redisMaxCementCode = getMaxCementCode().getData();
			//保存样品信息
			BaseResponse<String> results = saveSampleList(query.getSampleList(), redisMaxCementCode);
			if (results.getCode() != 200) {
				return results;
			}
			entrust.setStatus(1).setProjectType(query.getProjectType());
			logsService.saveLog(SysLogs.ObjType.ENTRUST_LOG, entrust.getId(), "修改评审状态为“通过”", null);
			//*****************************以上为评审接口******************
			//*****************************样品处理派发合并到评审接口中******************
			List<Sample> sampleList = sampleMapper.getSampleList(entrustVo.getId());
			String userIds = "";
			List<Integer> userIdList = new ArrayList<>();
			for (Sample sample : sampleList) {
				//样品的处理项
//                List<Integer> handleUserIds = new ArrayList<>();
				String teamIds = sample.getTeamIds();
				if (teamIds != null) {
					String[] teamIdS = teamIds.split("、");
					List<Integer> handleIds = new ArrayList<>();
					for (String teamId : teamIdS) {
						TeamVo teamVo = teamMapper.getDetail(Integer.valueOf(teamId));
						if (teamVo != null) {
							if (StringUtils.isNotBlank(teamVo.getHandleId())) {
								String[] handles = teamVo.getHandleId().split("、");
								for (String handleId : handles) {
									HandleVo handleVo = handleMapper.getDetail(Integer.valueOf(handleId));
									if (handleVo != null && !userIdList.contains(handleVo.getUserId())) {
										userIdList.add(handleVo.getUserId());
									}
									//样品处理表 同一个样品多个相同处理项时 去重。
									if (handleVo != null && !handleIds.contains(handleVo.getId())) {
										//20201229修改 ，去重处理项id
										handleIds.add(handleVo.getId());
										SampleHandle sampleHandle = new SampleHandle();
										sampleHandle.setSampleId(sample.getId())
												.setUserId(handleVo.getUserId())
												.setHandleId(handleVo.getId())
												.setPlanFinishDate(null)    //样品处理要求完成放到了委托表，评审的时候直接设定
//				                        .setStatus(0)
												.setCreateTime(LocalDateTime.now())
												.setStatus(1)  //直接默认接收
												.setAcceptTime(LocalDateTime.now());
										sampleHandleMapper.insert(sampleHandle);
										if (sampleHandle.getHandleId() != null) {
											Handle handle = handleMapper.selectById(sampleHandle.getHandleId());
											if (handle == null) {
												return BaseResponse.errorMsg("选择的处理项信息有误");
											}
                                            /*SampleHandleEnclosure sampleHandleEnclosure = new SampleHandleEnclosure();
                                            sampleHandleEnclosure.setSampleHandleId(sampleHandle.getId())
                                                    .setHandleId(handle.getId())
                                                    .setUserId(sampleHandle.getUserId())
                                                    .setCreateTime(LocalDateTime.now())
                                                    .setStatus(0)
                                                    .setAlias(handle.getName());
                                            sampleHandleEnclosureMapper.insert(sampleHandleEnclosure);*/

										}
										//消息推送
										BaseResponse wrapper = userMessageService.sendMessage(sampleHandle.getUserId(), "您有一条样品处理信息等待处理", entrust.getId(), SysUserMessage.MessageType.ENTRUST);
										if (wrapper.getCode() != 200) {
											return wrapper;
										}

										SysUser handleUser = userService.getById(sampleHandle.getUserId());
										if (handleUser == null) {
											return BaseResponse.errorMsg("样品处理人信息错误");
										}
										if (StringUtils.isNotBlank(handleUser.getWxId()) && !userIdList.contains(handleUser.getId())) {
											userIds = userIds.equals("") ? handleUser.getWxId() : userIds + "|" + handleUser.getWxId();
										}
									}
								}

							}

						}
					}
				}
			}
			//委托表改成已发送样品处理，状态改为样品处理状态中
			entrust.setIsHandle(1)
					.setStatus(3)
					.setFinishDate(query.getFinishDate());
			logsService.saveLog(SysLogs.ObjType.ENTRUST_LOG, entrust.getId(), "发送样品处理任务", null);

			//评审人员通过委托，提醒处理人员查看
			//发送企业微信消息内容
			String systemUrL = weiXinService.createMessage(entrust.getId(), "/blanche/wtdetail");
			String content = "你好！您有新的样品处理任务，" + "委托单号" + entrust.getEntrustCode() + "请前往\n" + systemUrL + "进行接查看。";
			weiXinService.sendTextMessage(userIds, content);

		} else if (query.getIsAgree() == 2) {  //驳回
			entrust.setEntrustCode(null);
			entrust.setStatus(2);
			logsService.saveLog(SysLogs.ObjType.ENTRUST_LOG, entrust.getId(), "修改评审状态为“驳回”", null);

			//评审人员驳回委托，提醒委托人员查看
			//发送企业微信消息内容
			String systemUrL = weiXinService.createMessage(entrust.getId(), "/blanche/wtdetail");
			String userIds = "";
			SysUser createUser = userService.getById(entrust.getUserId());
			if (createUser != null && createUser.getWxId() != null) {
				userIds = createUser.getWxId();
			}
			String content = "你好！您有委托被驳回，" + "委托单号" + entrust.getEntrustCode() + "请前往\n" + systemUrL + "进行接查看。";
			weiXinService.sendTextMessage(userIds, content);

		} else {
			return BaseResponse.errorMsg("请选择正确按钮");
		}
		//记录审批信息
		EntrustApproval entrustApproval = new EntrustApproval();
		entrustApproval.setEntrustId(entrust.getId())
				.setUserId(loginUser.getId())
				.setOpinion(query.getAuditReason())
				.setStatus(query.getIsAgree())
				.setCreateTime(LocalDateTime.now())
				.setRemark(null);
		approvalMapper.insert(entrustApproval);
		entrustMapper.updateById(entrust);
		//处理消息
		userMessageService.dealMessage(loginUser.getId(), entrust.getId(), SysUserMessage.MessageType.ENTRUST);
		return BaseResponse.okMsg("审批完成");
	}

	/**
	 * 委托人确认修改信息
	 *
	 * @param query
	 * @return
	 */
	@Transactional
//    @Override
	public BaseResponse<String> sureUpdate(sureUpdateQuery query) {
		if (query.getId() == null || query.getIsAgree() == null) {
			return BaseResponse.errorMsg("参数错误");
		}
		LoginUser loginUser = userService.getLoginUser();
		if (loginUser == null) {
			return BaseResponse.noLogin("请登录账号");
		}
		EntrustVo entrustVo = entrustMapper.getDetail(query.getId());
		if (entrustVo == null) {
			return BaseResponse.errorMsg("信息错误");
		}
		if (entrustVo.getStatus() != 9) {
			return BaseResponse.errorMsg("当前状态有误");
		}
		Entrust entrust = entrustMapper.selectById(entrustVo.getId());
		if (entrust.getUserId() != loginUser.getId()) {
			return BaseResponse.errorMsg("当前操作必须为委托人");
		}
		if (query.getIsAgree() == 1) { //同意
			//*****************************同意后 发送样品处理任务 同时添加产值*****************
			List<Sample> sampleList = sampleMapper.getSampleList(entrustVo.getId());
			String userIds = "";
			List<Integer> userIdList = new ArrayList<>();
			for (Sample sample : sampleList) {
				//样品的处理项
//                List<Integer> handleUserIds = new ArrayList<>();
				String teamIds = sample.getTeamIds();
				if (teamIds != null) {
					String[] teamIdS = teamIds.split("、");
					List<Integer> handleIds = new ArrayList<>();
					for (String teamId : teamIdS) {
						TeamVo teamVo = teamMapper.getDetail(Integer.valueOf(teamId));
						if (teamVo != null) {
							if (StringUtils.isNotBlank(teamVo.getHandleId())) {
								String[] handles = teamVo.getHandleId().split("、");
								for (String handleId : handles) {
									HandleVo handleVo = handleMapper.getDetail(Integer.valueOf(handleId));
									if (handleVo != null && !userIdList.contains(handleVo.getUserId())) {
										userIdList.add(handleVo.getUserId());
									}
									//样品处理表 同一个样品多个相同处理项时 去重。
									if (handleVo != null && !handleIds.contains(handleVo.getId())) {
										//20201229修改 ，去重处理项id
										handleIds.add(handleVo.getId());
										SampleHandle sampleHandle = new SampleHandle();
										sampleHandle.setSampleId(sample.getId())
												.setUserId(handleVo.getUserId())
												.setHandleId(handleVo.getId())
												.setPlanFinishDate(null)    //样品处理要求完成放到了委托表，评审的时候直接设定
//				                        .setStatus(0)
												.setCreateTime(LocalDateTime.now())
												.setStatus(1)  //直接默认接收
												.setAcceptTime(LocalDateTime.now());
										sampleHandleMapper.insert(sampleHandle);
										if (sampleHandle.getHandleId() != null) {
											Handle handle = handleMapper.selectById(sampleHandle.getHandleId());
											if (handle == null) {
												return BaseResponse.errorMsg("选择的处理项信息有误");
											}
											//获取产值信息
											QueryWrapper<NormProduction> queryWrapper = new QueryWrapper<>();
											queryWrapper.eq("type", 0);
											queryWrapper.eq("status", 1);
											List<NormProduction> normProduction = iNormProductionService.list(queryWrapper);
											if (normProduction == null || normProduction.size() != 1) {
												//评审未通过，删除评审后的样品
												sampleMapper.deleteBatchIds(sampleList);
												//还原本所编号最大值
//                                                redisUtil.setString("maxCementCode", redisMaxCementCode);
												return BaseResponse.errorMsg("请配置处理项的产值信息!");
											}
										}
										//消息推送
										BaseResponse wrapper = userMessageService.sendMessage(sampleHandle.getUserId(), "您有一条样品处理信息等待处理", entrust.getId(), SysUserMessage.MessageType.ENTRUST);
										if (wrapper.getCode() != 200) {
											return wrapper;
										}

										SysUser handleUser = userService.getById(sampleHandle.getUserId());
										if (handleUser == null) {
											return BaseResponse.errorMsg("样品处理人信息错误");
										}
										if (StringUtils.isNotBlank(handleUser.getWxId()) && !userIdList.contains(handleUser.getId())) {
											userIds = userIds.equals("") ? handleUser.getWxId() : userIds + "|" + handleUser.getWxId();
										}
									}
								}
							}
						}
					}
				}
			}

			//生成委托单号
			String entrustCode = entrust.getProjectCode() + "-" + sampleList.get(0).getCementCode();
			entrust.setEntrustCode(entrustCode);
			entrust.setIsHandle(1)
					.setStatus(3);
			logsService.saveLog(SysLogs.ObjType.ENTRUST_LOG, entrust.getId(), "同意委托修改", null);

			//委托人确认修改信息，提醒处理人员查看
			//发送企业微信消息内容
			String systemUrL = weiXinService.createMessage(entrust.getId(), "/blanche/wtdetail");
			String content = "你好！您有新的样品处理任务，" + "委托单号" + entrust.getEntrustCode() + "请前往\n" + systemUrL + "进行接查看。";
			weiXinService.sendTextMessage(userIds, content);

		} else if (query.getIsAgree() == 2) {  //拒绝
			entrust.setStatus(10); //作废状态
			logsService.saveLog(SysLogs.ObjType.ENTRUST_LOG, entrust.getId(), "不同意委托修改", null);
		} else {
			return BaseResponse.errorMsg("请选择正确按钮");
		}
		entrustMapper.updateById(entrust);
		return BaseResponse.okMsg("修改信息已确认");
	}

	/**
	 * 保存样品列表信息
	 *
	 * @param sampleQueryList
	 * @param redisMaxCementCode
	 * @return
	 */
	private BaseResponse<String> saveSampleList(List<SampleQuery> sampleQueryList, String redisMaxCementCode) {
		if (sampleQueryList != null && sampleQueryList.size() > 0) {
			//对样品信息进行判断
			for (SampleQuery sampleQuery : sampleQueryList) {
				if (StringUtils.isEmpty(sampleQuery.getName())) {
					return BaseResponse.errorMsg("请填写样品名");
				}
				if (StringUtils.isEmpty(sampleQuery.getSampleCode())) {
					return BaseResponse.errorMsg("请填写来样编号");
				}
				if (StringUtils.isEmpty(sampleQuery.getSampleForm())) {
					return BaseResponse.errorMsg("请选择样品状态");
				}
				if (StringUtils.isEmpty(sampleQuery.getTeamIds())) {
					return BaseResponse.errorMsg("请选择检测项目");
				}
				if (sampleQuery.getWeight() == null) {
					return BaseResponse.errorMsg("请填写样品重量");
				}
				if (sampleQuery.getWeight().toString().length()>10) {
					return BaseResponse.errorMsg("样品重量输入过长");
				}

				if (sampleQuery.getWeightType() == null) {
					return BaseResponse.errorMsg("请选择重量单位");
				}
				if (StringUtils.isEmpty(sampleQuery.getOrigin())) {
					return BaseResponse.errorMsg("请选择样品产地");
				}
			}

			List<Sample> sampleList = new ArrayList<>();
			String maxCementCode = ""; //最大本所编号对应的平行样编号
			Integer integerMaxCode = commonService.getIntegerCode(redisMaxCementCode);

			//除了主样还需要的编号数量
			//前端传入的最后一个样品的本所编号  后续的本所编号在这个基础上进行追加
			String lastCementCode = sampleQueryList.get(sampleQueryList.size() - 1).getCementCode();
			Integer integerLastCode = commonService.getIntegerCode(lastCementCode);
			if (integerLastCode <= integerMaxCode) {
				return BaseResponse.errorMsg("本所编号必须大于最大值");
			}

			//当前本所编号 截取的年份
			String yearStr = commonService.getIntegerYear(lastCementCode);
			//平行样的集合（乱序）
			Set<String> parallelSet = new HashSet<>();
			for (SampleQuery sq : sampleQueryList) {
				if (sq.getIsParallel() >= 2) {
					for (int i = 1; i < sq.getIsParallel(); i++) {
						integerLastCode = integerLastCode + 1;
						lastCementCode = yearStr + "-" + commonService.getCementCode(integerLastCode);
						parallelSet.add(lastCementCode);
					}
				}
			}
			for (SampleQuery sampleQuery : sampleQueryList) {



				if (StringUtils.isEmpty(sampleQuery.getCementCode())) {
					return BaseResponse.errorMsg("本所编号不能为空");
				}
				String regex = "[0-9]{2}\\-[0-9]{4}";
				if (!sampleQuery.getCementCode().matches(regex) || sampleQuery.getCementCode().length() != getMaxCementCode().getData().length()) {
					return BaseResponse.errorMsg("本所编号格式不符合规范");
				}
				//前端传入的本所编号
				Integer cementCodeInteger = commonService.getIntegerCode(sampleQuery.getCementCode());
				if (cementCodeInteger <= integerMaxCode) {
					return BaseResponse.errorMsg("本所编号必须大于最大值");
				}
				if (sampleQuery.getIsParallel() == null) {
					return BaseResponse.errorMsg("请选择是否需要平行样");
				}

				//先添加主样
				maxCementCode = sampleQuery.getCementCode();
				Sample sample = new Sample();
				BeanUtils.copyProperties(sampleQuery, sample);
				sample.setId(null)
						.setSampleId(sampleQuery.getId())
						.setStatus(0)  //样品状态为未领用状态
						.setCreateTime(LocalDateTime.now())
						.setIsHandle(0)//样品处理未完成状态
						.setIsDistribution(0)//任务派发未完成状态
						.setIsCheck(0)//样品校核未完成状态
						.setOutputValue(sample.getCharge()) //产值不用评审人员修改
						.setParallelCode(sampleQuery.getCementCode());
				sampleList.add(sample);

				//添加平行样integerMaxCode + sampleQueryLIst.size()    integerMaxCode + sampleQueryLIst.size()+num
				if (sampleQuery.getIsParallel() >= 2) { //非平行样
					if (sampleQuery.getIsParallel() < 2) {
						return BaseResponse.errorMsg("平行样数量至少2条");
					}
					if (sampleQuery.getIsParallel() > 4) {
						return BaseResponse.errorMsg("平行样数量最多4条");
					}
					for (int i = 1; i < sampleQuery.getIsParallel(); i++) {
//                        integerMaxCode = cementCodeInteger > integerMaxCode ? cementCodeInteger : integerMaxCode+1;
//                        String yearStr = commonService.getIntegerYear(sampleQuery.getCementCode());
//                        maxCementCode = yearStr + "-" + commonService.getCementCode(integerMaxCode);
						Sample sampleP = new Sample();
						BeanUtils.copyProperties(sampleQuery, sampleP);
						sampleP.setId(null)
								.setSampleId(sampleQuery.getId())
								.setStatus(0)  //样品状态为未领用状态
								.setCreateTime(LocalDateTime.now())
								.setIsHandle(0)//样品处理未完成状态
								.setIsDistribution(0)//任务派发未完成状态
								.setIsCheck(0)//样品校核未完成状态
								.setOutputValue(sampleP.getCharge()) //产值不用评审人员修改
								.setParallelCode(parallelSet.iterator().next());
						sampleList.add(sampleP);
						parallelSet.remove(sampleP.getParallelCode());
					}
				}
			}
			sampleService.saveBatch(sampleList);
			//最大的平行样编号放入redis
			System.out.println("审批时的最大本所编号*******************************************" + lastCementCode);
			redisUtil.setString("maxCementCode", lastCementCode);
		}else{
			return BaseResponse.errorMsg("样品及检测项不可为空");
		}
		return BaseResponse.okMsg("成功");
	}

	/**
	 * 发送样品处理任务
	 *
	 * @param query
	 * @return
	 */
    /*@Transactional
    @Override
    public BaseResponse<String> handle(HandleQuery query) {
        if (query.getId() == null) {
            return BaseResponse.errorMsg("参数错误");
        }
        Entrust entrust = entrustMapper.selectById(query.getId());
        if (entrust == null) {
            return BaseResponse.errorMsg("信息错误");
        }
        if (entrust.getStatus() != 1) {
            return BaseResponse.errorMsg("当前状态不允许样品处理");
        }
        String userIds = "";
        List<Integer> userIdList = new ArrayList<>();
        if (query.getSampleHandleQueryList() != null && query.getSampleHandleQueryList().size() > 0) {
            List<SampleHandleQuery> sampleHandleQueryList = query.getSampleHandleQueryList();
            for (SampleHandleQuery sampleHandleQuery : sampleHandleQueryList) {
                //样品处理表
                SampleHandle sampleHandle = new SampleHandle();
                if (sampleHandleQuery.getSampleId() == null) {
                    return BaseResponse.errorMsg("样品信息错误");
                }
                if (sampleHandleQuery.getUserId() == null) {
                    return BaseResponse.errorMsg("请选择处理人");
                }
                if (StringUtils.isEmpty(sampleHandleQuery.getContent())) {
                    return BaseResponse.errorMsg("请选择处理项");
                }
                if (sampleHandleQuery.getPlanFinishDate() == null) {
                    return BaseResponse.errorMsg("请选择要求完成日期");
                }
                BeanUtils.copyProperties(sampleHandleQuery, sampleHandle);
                sampleHandle.setStatus(0).setCreateTime(LocalDateTime.now());
                sampleHandleMapper.insert(sampleHandle);
                //样品处理项附件表
                if (sampleHandle.getContent() != null) {
                    String[] handIdS = sampleHandle.getContent().split("、");
                    List<SampleHandleEnclosure> sampleHandleEnclosureList = new ArrayList<>();
                    for (String handleId : handIdS) {
                        Handle handle = handleMapper.selectById(Integer.valueOf(handleId));
                        if (handle == null) {
                            return BaseResponse.errorMsg("选择的处理项信息有误");
                        }
                        SampleHandleEnclosure sampleHandleEnclosure = new SampleHandleEnclosure();
                        sampleHandleEnclosure.setSampleHandleId(sampleHandle.getId())
                                .setHandleId(handle.getId())
                                .setUserId(sampleHandle.getUserId())
                                .setCreateTime(LocalDateTime.now())
                                .setStatus(0)
                                .setAlias(handle.getName());
                        sampleHandleEnclosureList.add(sampleHandleEnclosure);
                        //获取产值信息
                        QueryWrapper<NormProduction> queryWrapper = new QueryWrapper<>();
                        queryWrapper.eq("assess_id", handle.getId());
                        queryWrapper.eq("type", 0);
                        queryWrapper.eq("status", 1);
                        NormProduction normProduction = iNormProductionService.getOne(queryWrapper);
                        if(normProduction == null){
                            return BaseResponse.errorMsg("联系管理员.配置产值信息!");
                        }
                    }
                    sampleHandleEnclosureService.saveBatch(sampleHandleEnclosureList);
                }
                //消息推送
                BaseResponse wrapper = userMessageService.sendMessage(sampleHandle.getUserId(), "您有一条样品处理信息等待处理",entrust.getId(), SysUserMessage.MessageType.ENTRUST);
                if(wrapper.getCode() != 200){
                    return wrapper;
                }

                SysUser handleUser = userService.getById(sampleHandle.getUserId());
                if(handleUser == null){
                    return BaseResponse.errorMsg("样品处理人信息错误");
                }
                if(StringUtils.isNotBlank(handleUser.getWxId()) && !userIdList.contains(handleUser.getId())){
                    userIdList.add(handleUser.getId());
                    userIds = userIds.equals("")?handleUser.getWxId():userIds+"|"+handleUser.getWxId();
                }
            }
        }
        //委托表改成已发送样品处理，状态改为样品处理状态中
        entrust.setIsHandle(1)
                .setStatus(3);
        entrustMapper.updateById(entrust);
        logsService.saveLog(SysLogs.ObjType.ENTRUST_LOG, entrust.getId(), "发送样品处理任务", null);

        //发送企业微信消息内容
        String content="你好！您有一条新的样品处理任务，请前往\n"+Global.systemUrl +"进行接受。";
        weiXinService.sendTextMessage(userIds, content);
        return BaseResponse.okMsg("已发送样品处理任务");
    }
*/

	/**
	 * 样品处理任务分页
	 *
	 * @param pageQuery
	 * @param projectCode
	 * @return
	 */
	@Override
	public BaseResponse<IPage<EntrustVo>> getSampleHandlePage(PageQuery pageQuery, String entrustCode, Integer status,
	                                                          String clientName, String projectName, String projectCode, Integer isUrgent,
	                                                          Integer handleStatus) {
		LoginUser loginUser = userService.getLoginUser();
		if (loginUser == null) {
			return BaseResponse.noLogin("请登录账号");
		}
		Map<String, Object> params = new HashMap<>();
		params.put("entrustCode", entrustCode);
		params.put("status", status);
		params.put("clientName", clientName);
		params.put("projectName", projectName);
		params.put("projectCode", projectCode);
		params.put("isUrgent", isUrgent);
		params.put("userId", loginUser.getId());
		params.put("handleStatus", handleStatus);

		Page<EntrustVo> page = new Page<>(pageQuery.getPageNo(), pageQuery.getPageSize());
		IPage<EntrustVo> pages = entrustMapper.getSampleHandlePage(page, params);
		List<EntrustVo> list = pages.getRecords();
		if (list != null && list.size() > 0) {
			for (EntrustVo entrustVo : list) {
				List<Sample> sampleList = sampleMapper.getSampleList(entrustVo.getId());
				//样品名称名称
				String sampleNames = "";
				//处理项名称
				String handleNames = "";
				if (sampleList != null && sampleList.size() > 0) {
					//样品名称 列表中的拼接
					for (Sample sample : sampleList) {
						sampleNames = sampleNames.equals("") ? sample.getName() : (sampleNames + "、" + sample.getName());
					}
					//处理项表里只显示第一个样品的
					Sample sample = sampleList.get(0);
					QueryWrapper<SampleHandle> shWrapper = new QueryWrapper<>();
					shWrapper.eq("sample_id", sample.getId());
					List<SampleHandle> shList = sampleHandleMapper.selectList(shWrapper);
					if (shList != null && shList.size() > 0) {
						for (SampleHandle sampleHandle : shList) {
							if (sampleHandle.getHandleId() != null) {
								Handle handle = handleMapper.selectById(sampleHandle.getHandleId());
								if (handle != null) {
									handleNames = handleNames.equals("") ? handle.getName() : (handleNames + "、" + handle.getName());
								}

							}
						}
					}
					//第一个样品的最早接受时间
					LocalDateTime acceptTime = sampleHandleMapper.getNo1AcceptTime(sample.getId());
					entrustVo.setAcceptTime(acceptTime);
				}
				//列表中的样品数量只展示关联到处理人的样品数量
				Map<String, Object> paramHander = new HashMap<>();
				paramHander.put("id", entrustVo.getId());
				paramHander.put("userId", loginUser.getId());
				List<SampleHandleVo> sampleHandleList = sampleHandleMapper.getSampleHandleList(paramHander);
				if (sampleHandleList != null) {
					entrustVo.setSampleNum(sampleHandleList.size());
				} else {
					entrustVo.setSampleNum(0);
				}
				entrustVo.setHandleNames(handleNames);
				entrustVo.setSampleNames(sampleNames);
			}
		}
		pages.setRecords(list);
		return BaseResponse.okData(pages);
	}

	/**
	 * 样品处理任务列表
	 *
	 * @param id 委托表id
	 * @return
	 */
	@Override
	public BaseResponse<Map<String, Object>> getSampleHandleList(Integer id) {
		Map<String, Object> map = new HashMap<>();
		Entrust entrust = entrustMapper.selectById(id);
		if (entrust == null) {
			return BaseResponse.errorMsg("信息错误");
		}
		LoginUser loginUser = userService.getLoginUser();
		Map<String, Object> params = new HashMap<>();
		params.put("id", id);
		params.put("userId", loginUser.getId());
		List<SampleHandleVo> sampleHandleList = sampleHandleMapper.getSampleHandleList(params);
		Integer handleStatus = null;
		LocalDateTime pfTime = null;
		if (sampleHandleList != null && sampleHandleList.size() > 0) {
			for (SampleHandleVo shVo : sampleHandleList) {
				pfTime = shVo.getCreateTime();
              /*  if (handleStatus != null && handleStatus != shVo.getStatus()) {
                    return BaseResponse.errorMsg("接受或完成的状态信息错误");
                }*/
				handleStatus = shVo.getStatus() == 1 ? 1 : (shVo.getStatus() == 2 ? 2 : 0);

				Handle handle = handleMapper.selectById(shVo.getHandleId());
				if (handle == null) {
					return BaseResponse.errorMsg("选择的处理项信息有误");
				}
				String contentName = handle.getName();
				shVo.setContentName(contentName);
			}
		}
		entrust.setIsHandle(handleStatus); //0显示接受按钮，1显示完成按钮（可以上传附件），2和null不显示按钮
		map.put("entrust", entrust);
		map.put("sampleHandleList", sampleHandleList);
		DateTimeFormatter df = DateTimeFormatter.ofPattern("yyyy-MM-dd");
		map.put("pfTime", df.format(pfTime));
		return BaseResponse.okData(map);
	}

	/**
	 * 接受样品处理任务
	 *
	 * @param ids 委托单
	 * @return
	 */
    /*@Transactional
    @Override
    public BaseResponse<String> acceptHandle(Integer[] ids) {
        LoginUser loginUser = userService.getLoginUser();
        if (loginUser == null) {
            return BaseResponse.noLogin("请登录账号");
        }
        if (ids == null || ids.length <= 0) {
            return BaseResponse.errorMsg("参数错误");
        }
        List<SampleHandle> sampleHandleList = new ArrayList<>();
        for (Integer id : ids) {
            SampleHandle sampleHandle = sampleHandleMapper.selectById(id);
            if (sampleHandle == null) {
                return BaseResponse.errorMsg("信息错误");
            }
            if (sampleHandle.getStatus() != 0) {
                return BaseResponse.errorMsg("状态错误");
            }
            if (loginUser.getId() != sampleHandle.getUserId()) {
                return BaseResponse.errorMsg("请登录样品处理人账号操作");
            }
            sampleHandle.setStatus(1).setAcceptTime(LocalDateTime.now());
            sampleHandleList.add(sampleHandle);
        }
        sampleHandleService.updateBatchById(sampleHandleList);
        //记录委托单日志
        Sample sample = sampleService.getById(sampleHandleList.get(0).getSampleId());
        if (sample == null) {
            return BaseResponse.errorMsg("样品信息错误");
        }
        logsService.saveLog(SysLogs.ObjType.ENTRUST_LOG, sample.getEntrustId(), "接受样品处理任务", null);
        //处理消息
        userMessageService.dealMessage(loginUser.getId(), sample.getEntrustId(), SysUserMessage.MessageType.ENTRUST);
        return BaseResponse.okMsg("已接受样品处理任务");
    }*/

	/**
	 * 样品处理详情
	 *
	 * @param id 样品处理表id
	 * @return
	 */
	@Override
	public BaseResponse<SampleHandleVo> getHandleDtail(Integer id) {
		SampleHandleVo sampleHandleVo = sampleHandleMapper.getDetail(id);
		return BaseResponse.okData(sampleHandleVo);
	}

	/**
	 * 完成样品处理任务
	 *
	 * @param id 样品处理表id
	 * @return
	 */
	@Transactional(rollbackFor = Exception.class, propagation = Propagation.NESTED)
	@Override
	public BaseResponse<String> finishHandle(Integer id) {
		LoginUser loginUser = userService.getLoginUser();
		if (loginUser == null) {
			return BaseResponse.noLogin("请登录账号");
		}
		if (id == null) {
			return BaseResponse.errorMsg("参数错误");
		}
		SampleHandle sampleHandle = sampleHandleMapper.selectById(id);
		if (sampleHandle == null) {
			return BaseResponse.errorMsg("信息错误");
		}
		if (sampleHandle.getStatus() != 1 && sampleHandle.getStatus() != 2) {
			return BaseResponse.errorMsg("状态错误");
		}
		if (loginUser.getId() != sampleHandle.getUserId()) {
			return BaseResponse.errorMsg("请登录样品处理人账号操作");
		}
		sampleHandle.setStatus(2).setFinishTime(LocalDateTime.now());
		sampleHandleMapper.updateById(sampleHandle);

		//判断该委托单下的所有样品的处理 及 处理项是否全部完成
		Sample sample = sampleMapper.selectById(sampleHandle.getSampleId());
		if (sample == null) {
			return BaseResponse.errorMsg("样品信息错误");
		}
		Entrust entrust = entrustMapper.selectById(sample.getEntrustId());
		if (entrust == null) {
			return BaseResponse.errorMsg("委托信息错误");
		}
		//计算产值
		if (sampleHandle.getHandleId() != null) {
			try {
				//判断该样品是否已经计算产值，如果参与过计算不进行再次的产值计算
				QueryWrapper<Preciprice> ppWrapper = new QueryWrapper<>();
				ppWrapper.eq("type", 0);
				ppWrapper.eq("entrust_id", entrust.getId());
				ppWrapper.eq("sample_id", sample.getId());
				ppWrapper.eq("user_id", sampleHandle.getUserId());
				List<Preciprice> ppList = iPrecipriceService.list(ppWrapper);
				if(ppList == null || ppList.size()==0){
					iPrecipriceService.createPreciprice(sampleHandle.getUserId(), entrust.getId(), null, 0, sample.getId(), false);
				}
			} catch (Exception e) {
				System.out.println(e.getMessage());
				return BaseResponse.errorMsg(e.getMessage());
			}
		}

		QueryWrapper<SampleHandle> sampleHandleWrapper = new QueryWrapper<>();
		sampleHandleWrapper.eq("sample_id", sample.getId());
		sampleHandleWrapper.ne("status", 2);
		List<SampleHandle> sampleHandleIngList = sampleHandleMapper.selectList(sampleHandleWrapper);
		if (sampleHandleIngList == null || sampleHandleIngList.size() == 0) {
			if (sample.getIsParallel() == 0) {
				sample.setIsHandle(1); //样品处理全部完成
				sampleMapper.updateById(sample);
			} else {
				List<Sample> sampleList1 = sampleMapper.getByCementCode(entrust.getId(), sample.getCementCode());
				if (sampleList1.size() < 2) {
					return BaseResponse.errorMsg("平行样样品数量信息错误");
				}
				// 如果是平行样，平行样的状态也改成已处理完成
				for (Sample sam : sampleList1) {
					sam.setIsHandle(1);
					sampleMapper.updateById(sam);
				}
			}
		}
		if (entrust != null) {
          /*  if (entrust.getStatus() != 3) {
                return BaseResponse.errorMsg("委托单状态错误");
            }*/
			List<Sample> sampleList = sampleMapper.getNoHandleList(sample.getEntrustId());
			if (sampleList == null || sampleList.size() == 0) {
				entrust.setStatus(4);
			}
			entrustMapper.updateById(entrust);
		}

		if (entrust == null) {
			return BaseResponse.errorMsg("委托信息错误");
		}
		if (entrust.getStatus() == 4) {
			handle2Distribution(entrust.getId());
		}

		//记录委托单日志
		logsService.saveLog(SysLogs.ObjType.ENTRUST_LOG, entrust.getId(), "完成样品处理任务", null);
		return BaseResponse.okMsg("已完成样品处理任务");
	}

	/**
	 * 样品处理任务退回
	 *
	 * @param query 回退请求
	 * @return
	 */
	@Transactional
	@Override
	public BaseResponse<String> returnHandle(ReturnQuery query) {
		LoginUser loginUser = userService.getLoginUser();
		if (loginUser == null) {
			return BaseResponse.noLogin("请登录账号");
		}
		if (query == null || query.getId() == null) {
			return BaseResponse.errorMsg("参数错误");
		}
		if (query.getReturnReason().length() > 100) {
			return BaseResponse.errorMsg("退回原因内容过长");
		}
		SampleHandle sampleHandle = sampleHandleMapper.selectById(query.getId());
		if (sampleHandle == null) {
			return BaseResponse.errorMsg("信息错误");
		}

		if (sampleHandle.getStatus() != 1 && sampleHandle.getStatus() != 2) {
			return BaseResponse.errorMsg("状态错误");
		}
		if (loginUser.getId() != sampleHandle.getUserId()) {
			return BaseResponse.errorMsg("请登录样品处理人账号操作");
		}
		//判断该委托单下的所有样品的处理 及 处理项是否全部完成
		Sample sample = sampleMapper.selectById(sampleHandle.getSampleId());
		if (sample == null) {
			return BaseResponse.errorMsg("样品信息错误");
		}
		Entrust entrust = entrustMapper.selectById(sample.getEntrustId());
		if (entrust == null) {
			return BaseResponse.errorMsg("委托信息错误");
		}

		sampleHandle.setStatus(3).setRemark(query.getReturnReason());
		sampleHandleMapper.updateById(sampleHandle);
		//样品处理退回记录
		SampleHandleReturn handleReturn = new SampleHandleReturn();
		handleReturn.setSampleHandleId(sampleHandle.getId());
		handleReturn.setUserId(sampleHandle.getUserId());
		handleReturn.setCreateTime(LocalDateTime.now());
		handleReturn.setReturnReason(sampleHandle.getRemark());
		handleReturn.setRemark(null);
		sampleHandleReturnMapper.insert(handleReturn);

		//记录委托单日志
		logsService.saveLog(SysLogs.ObjType.ENTRUST_LOG, entrust.getId(), "退回样品处理任务", null);

		//样品处理人退回样品处理任务，提醒评审人员查看
		List<Integer> approvalIdList = sysApprovalMapper.getApprovalId("委托评审");
		if (approvalIdList == null) {
			return BaseResponse.errorMsg("委托评审信息错误");
		}
		String userIds = "";
		for (Integer approvalId : approvalIdList) {
			SysUser approvalUser = userService.getById(approvalId);
			if (approvalUser == null) {
				return BaseResponse.errorMsg("委托评审信息错误");
			}
			BaseResponse wrapper = userMessageService.sendMessage(approvalId, "您有一条被退回的检测任务", entrust.getId(), SysUserMessage.MessageType.ENTRUST);
			if (wrapper.getCode() != 200) {
				return wrapper;
			}
			if (StringUtils.isNotBlank(approvalUser.getWxId())) {
				userIds = userIds.equals("") ? approvalUser.getWxId() : userIds + "|" + approvalUser.getWxId();
			}
		}
		//发送企业微信消息内容
		String systemUrL = weiXinService.createMessage(entrust.getId(), "/blanche/blancherecive");
		String content = "你好！您有样品处理任务被退回，" + "委托单号" + entrust.getEntrustCode() + "请前往\n" + systemUrL + "进行接查看。";
		weiXinService.sendTextMessage(userIds, content);

		return BaseResponse.okMsg("已退回样品处理任务");
	}

	/**
	 * 样品处理退回任务分页
	 *
	 * @param pageQuery
	 * @param projectCode
	 * @return
	 */
	@Override
	public BaseResponse<IPage<SampleReturnHandleVo>> getSampleReturnHandlePage(PageQuery pageQuery, String entrustCode, Integer userId,
	                                                                           String clientName, String projectName, String projectCode, Integer isUrgent) {
		LoginUser loginUser = userService.getLoginUser();
		if (loginUser == null) {
			return BaseResponse.noLogin("请登录账号");
		}
		Map<String, Object> params = new HashMap<>();
		params.put("entrustCode", entrustCode);
		params.put("clientName", clientName);
		params.put("projectName", projectName);
		params.put("projectCode", projectCode);
		params.put("isUrgent", isUrgent);
		params.put("userId", userId);

		Page<SampleReturnHandleVo> page = new Page<>(pageQuery.getPageNo(), pageQuery.getPageSize());
		IPage<SampleReturnHandleVo> pages = entrustMapper.getSampleReturnHandlePage(page, params);
		return BaseResponse.okData(pages);
	}

	/**
	 * 样品处理重新派发
	 *
	 * @param query
	 * @return
	 */
	public BaseResponse<String> handleAgain(HandleAgainQuery query) {
		LoginUser loginUser = userService.getLoginUser();
		if (loginUser == null) {
			return BaseResponse.noLogin("请登录账号");
		}
		List<Integer> approvalIdList = sysApprovalMapper.getApprovalId("委托评审");
		if (approvalIdList == null) {
			return BaseResponse.errorMsg("委托评审信息错误");
		}
		if (!approvalIdList.contains(loginUser.getId())) {
			return BaseResponse.errorMsg("没有重新派发权限");
		}

		if (query == null || query.getId() == null) {
			return BaseResponse.errorMsg("参数错误");
		}
		if (query.getUserId() == null) {
			return BaseResponse.errorMsg("请选择接收人");
		}
		SampleHandle sampleHandle = sampleHandleMapper.selectById(query.getId());
		if (sampleHandle == null) {
			return BaseResponse.errorMsg("信息错误");
		}
		if (sampleHandle.getStatus() != 3) {
			return BaseResponse.errorMsg("状态错误");
		}

		sampleHandle.setStatus(1)
				.setUserId(query.getUserId())
				.setRemark(null);
		sampleHandleMapper.updateById(sampleHandle);

		//判断该委托单下的所有样品的处理 及 处理项是否全部完成
		Sample sample = sampleMapper.selectById(sampleHandle.getSampleId());
		if (sample == null) {
			return BaseResponse.errorMsg("样品信息错误");
		}
		Entrust entrust = entrustMapper.selectById(sample.getEntrustId());
		if (entrust == null) {
			return BaseResponse.errorMsg("委托信息错误");
		}
		//记录委托单日志
		logsService.saveLog(SysLogs.ObjType.ENTRUST_LOG, entrust.getId(), "重新派发样品处理任务", null);

		//消息推送
		BaseResponse wrapper = userMessageService.sendMessage(sampleHandle.getUserId(), "您有一条样品处理信息等待处理", entrust.getId(), SysUserMessage.MessageType.ENTRUST);
		if (wrapper.getCode() != 200) {
			return wrapper;
		}

		//重新派发样品处理任务，提醒样品处理人查看
		//发送企业微信消息内容
		String systemUrL = weiXinService.createMessage(entrust.getId(), "/blanche/sampledetail");
		String content = "你好！您有一条新的样品处理任务，请前往\n" + systemUrL + "进行查看。";
		SysUser handleUser = userService.getById(sampleHandle.getUserId());
		if (handleUser == null || StringUtils.isEmpty(handleUser.getWxId())) {
			return BaseResponse.errorMsg("样品处理人信息错误");
		}
		weiXinService.sendTextMessage(handleUser.getWxId(), content);

		return BaseResponse.okMsg("已重新派发处理任务");
	}

	/**
	 * 样品处理进入到检测项表中
	 * <p>
	 * //判断改委托单的所有样品检测项对应的处理项对应的处理人，然后按照检测项存入检测项表，以便校核的时候可以校核结果
	 * //只显示主样，因为样品处理只显示主样
	 *
	 * @param entrustId
	 */
	private void handle2Distribution(Integer entrustId) {
		List<Sample> sampleHandleList = sampleMapper.getSampleList(entrustId);
		if (sampleHandleList != null && sampleHandleList.size() > 0) {
			for (Sample sample : sampleHandleList) {
				String teamIds = sample.getTeamIds();
				if (teamIds != null) {
					String[] teamIdS = teamIds.split("、");
					for (String teamId : teamIdS) {
						TeamVo teamVo = teamMapper.getDetail(Integer.valueOf(teamId));
//                            List<Integer> userIdsList = new ArrayList<>();
						if (teamVo != null && teamVo.getIsPf() == 0) {
							String handleIds = teamVo.getHandleId();
							if (handleIds != null) {
								String[] handleIdS = handleIds.split("、");
								for (String handleId : handleIdS) {
									//样品id 处理人id  关联的处理项表信息进行展示
									QueryWrapper<SampleHandle> handleWrapper = new QueryWrapper<>();
									handleWrapper.eq("sample_id", sample.getId());
									handleWrapper.eq("handle_id", handleId);
									List<SampleHandle> handleList = sampleHandleMapper.selectList(handleWrapper);
									if (handleList != null && handleList.size() > 0) {
										for (SampleHandle sampleHandle : handleList) {
											SampleDistribution sampleDistribution = new SampleDistribution();
											sampleDistribution.setEntrustId(entrustId)
													.setSampleId(sample.getId())
													.setTeamGroupId(teamVo.getGroupId())
													.setTeamId(teamVo.getId())
													.setUserId(sampleHandle.getUserId())
													.setCreateTime(sampleHandle.getCreateTime())
													.setStatus(2)  //直接进入待校核状态
													.setAcceptTime(sampleHandle.getAcceptTime())
													.setFinishTime(sampleHandle.getFinishTime());
											distributionService.save(sampleDistribution);
//                                                sampleDistributionList.add(sampleDistribution);

											//将样品处理的附件复制到样品检测的附件表中  如果校核附件有重复，可以去掉这个逻辑
											Map<String, Object> params = new HashMap<>();
											params.put("sampleHandleId", sampleHandle.getId());
											List<SampleHandleEnclosureVo> list = sampleHandleEnclosureMapper.getList(params);
											if (list != null && list.size() > 0) {
												for (SampleHandleEnclosureVo shev : list) {
													SampleDistributionEnclosure sampleDistributionEnclosure = new SampleDistributionEnclosure();
													sampleDistributionEnclosure.setSampleDistributionId(sampleDistribution.getId())
															.setTeamId(sampleDistribution.getTeamId())
															.setUserId(sampleDistribution.getUserId())
															.setCreateTime(shev.getCreateTime())
															.setStatus(shev.getStatus())
															.setAlias(shev.getAlias())
															.setEnclosureUrl(shev.getEnclosureUrl())
															.setExtName(shev.getExtName())
															.setUpdateTime(LocalDateTime.now())
															.setPdfUrl(shev.getPdfUrl())
															.setIsPublic(0); //默认不允许公开
													if (StringUtils.isEmpty(shev.getEnclosureUrl())) {
														sampleDistributionEnclosure.setStatus(2); //上传失败
													}
													sampleDistributionEnclosureMapper.insert(sampleDistributionEnclosure);
												}
											}
										}
									}
								}
							}
						}
					}
				}
			}
		}

		//根据检测项是否需要派发操作，判断委托单是否派发按钮，如果有一个检测项需要派发，则显示派发按钮，否则显示已经派发
		//样品处理如果有一个检测项需要派发，则不改变已派发字段状态，否则改变为已派发状态

		//未派发的样品
		List<Sample> sampleList = sampleMapper.getNoDistributionList(entrustId);
		if (sampleList != null && sampleList.size() > 0) {
			Integer sampleNum = 0;
			for (Sample sample : sampleList) {
				String teamIds = sample.getTeamIds();
				Integer disNum = 0;
				if (teamIds != null) {
					String[] teamIdS = teamIds.split("、");
					for (String teamId : teamIdS) {
						TeamVo teamVo = teamMapper.getDetail(Integer.valueOf(teamId));
						//如果本所编号不相等 则为副样，副样需要判断检测项是否需要平行样，如果不需要不用进行派发检测
						if (teamVo.getIsPf() == 1) {
							disNum = disNum + 1;
						}
					}
				}
				//如果disNum =0 说明该样品中没有需要派发的检测项，直接改样品的状态为已派发状态,样品状态改成已领用状态
				if (disNum == 0) {
					sample.setStatus(1).setIsDistribution(1);
					sampleMapper.updateById(sample);
				}
				sampleNum = sampleNum + disNum;
			}
			//sampleNum =0 说明该委托中没有需要派发的检测项，直接改样品的委托为已派发状态,委托单状态直接改成样品检测完成状态
			if (sampleNum == 0) {
				Entrust entrust = entrustMapper.selectById(entrustId);
				if (entrust != null) {
					entrust.setStatus(6).setIsDistribution(1);
					entrustMapper.updateById(entrust);
				}
			}

		}
	}

	/**
	 * 派发检测项目任务
	 *
	 * @param query
	 * @return
	 */
	@Transactional
	@Override
	public BaseResponse<String> distribution(DistributionQuery query) {
		LoginUser loginUser = userService.getLoginUser();
		if (loginUser == null) {
			return BaseResponse.noLogin("请登录账号");
		}
		if (query.getId() == null) {
			return BaseResponse.errorMsg("参数错误");
		}
		Entrust entrust = entrustMapper.selectById(query.getId());
		if (entrust == null) {
			return BaseResponse.errorMsg("信息错误");
		}
		if (entrust.getStatus() != 4) {
			return BaseResponse.errorMsg("当前状态不允许派发");
		}
		String userIds = "";
		List<Integer> userIdList = new ArrayList<>();
		if (query.getSampleDistributionQueryList() != null && query.getSampleDistributionQueryList().size() > 0) {
			List<SampleDistributionQuery> sampleDistributionQueryList = query.getSampleDistributionQueryList();
//            List<SampleDistribution> sampleDistributionList = new ArrayList<>();

			//判断产值是否已经配置
			for (SampleDistributionQuery distributionQuery : sampleDistributionQueryList) {
				if (distributionQuery == null || distributionQuery.getSampleId() == null) {
					return BaseResponse.errorMsg("样品信息错误");
				}
				if (distributionQuery.getSampleDistributionTeamQueryList() != null && distributionQuery.getSampleDistributionTeamQueryList().size() > 0) {
					List<SampleDistributionTeamQuery> sampleDistributionTeamQueryList = distributionQuery.getSampleDistributionTeamQueryList();
					for (SampleDistributionTeamQuery distributionTeamQuery : sampleDistributionTeamQueryList) {
						if (distributionTeamQuery == null || distributionTeamQuery.getTeamId() == null ||
								distributionTeamQuery.getTeamGroupId() == null) {
							return BaseResponse.errorMsg("检测项信息错误");
						}
						if (distributionTeamQuery.getUserId() == null) {
							return BaseResponse.errorMsg("请选择检测人员");
						}
						//获取产值信息
						QueryWrapper<NormProduction> queryWrapper = new QueryWrapper<>();
						queryWrapper.eq("assess_id", distributionTeamQuery.getTeamGroupId());
						queryWrapper.eq("type", 1);
						queryWrapper.eq("status", 1);
						List<NormProduction> normProduction = iNormProductionService.list(queryWrapper);
						if (normProduction == null || normProduction.size() == 0 ) {
							//获取产值信息
							QueryWrapper<NormProduction> queryWrapper2 = new QueryWrapper<>();
							queryWrapper2.eq("assess_id", distributionTeamQuery.getTeamId());
							queryWrapper2.eq("type", 2);
							queryWrapper2.eq("status", 1);
							List<NormProduction> normProduction2 = iNormProductionService.list(queryWrapper2);
							TeamGroup teamGroup = teamGroupMapper.selectById(distributionTeamQuery.getTeamGroupId());
							Team team = teamMapper.selectById(distributionTeamQuery.getTeamId());
							if (normProduction2 == null || normProduction2.size() == 0 ) {
								return BaseResponse.errorMsg("联系管理员.配置检测组名称为" + teamGroup.getName() + "或者检测项名称为"+ team.getName() +"产值信息!");
							}

						}
					}
				}
			}

			/*************上面是样品处理 (样品处理进入检测表提前到样品处理完成的时候了)  下面是样品检测的***************/

			List<Sample> sampleList = new ArrayList<>();
			for (SampleDistributionQuery distributionQuery : sampleDistributionQueryList) {
				if (distributionQuery.getSampleDistributionTeamQueryList() != null && distributionQuery.getSampleDistributionTeamQueryList().size() > 0) {
					List<SampleDistributionTeamQuery> sampleDistributionTeamQueryList = distributionQuery.getSampleDistributionTeamQueryList();
					for (SampleDistributionTeamQuery distributionTeamQuery : sampleDistributionTeamQueryList) {
						SampleDistribution sampleDistribution = new SampleDistribution();
						sampleDistribution.setEntrustId(entrust.getId())
								.setSampleId(distributionQuery.getSampleId())
								.setTeamGroupId(distributionTeamQuery.getTeamGroupId())
								.setTeamId(distributionTeamQuery.getTeamId())
								.setUserId(distributionTeamQuery.getUserId())
//                                .setStatus(0)
								.setCreateTime(LocalDateTime.now())
								.setStatus(1)  //直接接收检测任务
								.setAcceptTime(LocalDateTime.now());
						distributionService.save(sampleDistribution);
//                        sampleDistributionList.add(sampleDistribution);
						//消息推送
						BaseResponse wrapper = userMessageService.sendMessage(distributionTeamQuery.getUserId(), "您有一条样品检测信息等待检测", entrust.getId(), SysUserMessage.MessageType.ENTRUST);
						if (wrapper.getCode() != 200) {
							return wrapper;
						}
						SysUser distributionUser = userService.getById(distributionTeamQuery.getUserId());
						if (distributionUser == null) {
							return BaseResponse.errorMsg("样品检测人信息错误");
						}
						if (StringUtils.isNotBlank(distributionUser.getWxId()) && !userIdList.contains(distributionUser.getId())) {
							userIdList.add(distributionUser.getId());
							userIds = userIds.equals("") ? distributionUser.getWxId() : userIds + "|" + distributionUser.getWxId();
						}
					}
				}
				Sample sample = sampleService.getById(distributionQuery.getSampleId());
				if (sample == null) {
					return BaseResponse.errorMsg("样品信息错误");
				}
				sample.setStatus(1); //样品状态改成已领用状态
				sampleList.add(sample);
			}
//            distributionService.saveBatch(sampleDistributionList);

			sampleService.updateBatchById(sampleList);
		}
		entrust.setStatus(5).setIsDistribution(1);
		entrustMapper.updateById(entrust);
		logsService.saveLog(SysLogs.ObjType.ENTRUST_LOG, entrust.getId(), "派发检测项目任务", null);

		//派发样品样品检测任务，提醒检测人员查看
		//发送企业微信消息内容
		String systemUrL = weiXinService.createMessage(entrust.getId(), "/blanche/recivedetail");
		String content = "你好！您有新的样品检测任务，" + "委托单号" + entrust.getEntrustCode() + "请前往\n" + systemUrL + "进行接查看。";
		weiXinService.sendTextMessage(userIds, content);
		return BaseResponse.okMsg("已派发检测项目任务");
	}

	/**
	 * 派发任务分页
	 *
	 * @param pageQuery
	 * @param projectName
	 * @param projectCode
	 * @return
	 */
	@Override
	public BaseResponse<IPage<EntrustVo>> getSampleDistributionPage(PageQuery pageQuery, String entrustCode, Integer status,
	                                                                String clientName, String projectName, String projectCode,
	                                                                Integer isUrgent, Integer distributionStatus) {
		LoginUser loginUser = userService.getLoginUser();
		if (loginUser == null) {
			return BaseResponse.noLogin("请登录账号");
		}
		Map<String, Object> params = new HashMap<>();
		params.put("entrustCode", entrustCode);
		params.put("status", status);
		params.put("clientName", clientName);
		params.put("projectName", projectName);
		params.put("projectCode", projectCode);
		params.put("isUrgent", isUrgent);
		params.put("userId", loginUser.getId());
		params.put("distributionStatus", distributionStatus);
		Page<EntrustVo> page = new Page<>(pageQuery.getPageNo(), pageQuery.getPageSize());
		IPage<EntrustVo> pages = entrustMapper.getSampleDistributionPage(page, params);
		List<EntrustVo> list = pages.getRecords();
		if (list != null && list.size() > 0) {
			for (EntrustVo entrustVo : list) {
				List<Sample> sampleList = sampleMapper.getSampleList(entrustVo.getId());
				//检测项目名称
				String sampleNames = "";
				if (sampleList != null && sampleList.size() > 0) {
					//样品名称 列表中的拼接
					for (Sample sample : sampleList) {
						sampleNames = sampleNames.equals("") ? sample.getName() : (sampleNames + "、" + sample.getName());
					}
					//检测项 和检测依据 列表里只显示第一个样品的
					Sample sample = sampleList.get(0);
					String teamIds = sample.getTeamIds();
					String checkTeam = "";
					if (teamIds != null) {
						String[] teamIdS = teamIds.split("、");
						for (String teamId : teamIdS) {
							Team team = teamMapper.selectById(Integer.valueOf(teamId));
							if (team != null) {
								checkTeam = checkTeam.equals("") ? team.getName() : (checkTeam + "、" + team.getName());
							}
						}
					}
					entrustVo.setCheckTeam(checkTeam);
					entrustVo.setCheckMethodNumber(sample.getMethodNumbers());
				}

				//列表中的样品数量只展示关联到检测人的样品数量
				List<Sample> sampleCheckList = sampleMapper.getSampleCheckList(entrustVo.getId());
				List<SampleVo> sampleVoList = new ArrayList<>();
				if (sampleCheckList != null && sampleCheckList.size() > 0) {
					for (Sample sample : sampleCheckList) {
						SampleVo sampleVo = new SampleVo();
						BeanUtils.copyProperties(sample, sampleVo);
						List<SampleDistributionTeamVo> sampleDistributionTeamVoList =
								distributionMapper.getDistributionTeamList(sample.getId(), loginUser.getId());
						if (sampleDistributionTeamVoList != null && sampleDistributionTeamVoList.size() > 0) {
							sampleVoList.add(sampleVo);
						}
					}
				}

				if (sampleVoList != null) {
					entrustVo.setSampleNum(sampleVoList.size());
				} else {
					entrustVo.setSampleNum(0);
				}

				entrustVo.setSampleNames(sampleNames);
			}
		}
		pages.setRecords(list);
		return BaseResponse.okData(pages);
	}

	/**
	 * 检测通知单详情（派发的和校核退回）
	 *
	 * @param id 委托表id
	 * @return
	 */
	@Override
	public BaseResponse<Map<String, Object>> getSampleDistributionList(Integer id) {
		LoginUser loginUser = userService.getLoginUser();
		if (loginUser == null) {
			return BaseResponse.noLogin("请登录账号");
		}
		Map<String, Object> map = new HashMap<>();
		Entrust entrust = entrustMapper.selectById(id);
		if (entrust == null) {
			return BaseResponse.errorMsg("信息错误");
		}
		List<Sample> sampleList = sampleMapper.getSampleCheckList(id);
		List<SampleVo> sampleVoList = new ArrayList<>();
		LocalDateTime pfTime = null;
		if (sampleList != null && sampleList.size() > 0) {
			for (Sample sample : sampleList) {
				SampleVo sampleVo = new SampleVo();
				BeanUtils.copyProperties(sample, sampleVo);
				List<SampleDistributionTeamVo> sampleDistributionTeamVoList =
						distributionMapper.getDistributionTeamList2(sample.getId(), loginUser.getId());
				Map<String, String> checkTeamMap = new HashMap<>();
				if (sampleDistributionTeamVoList != null && sampleDistributionTeamVoList.size() > 0) {
					//已经派发过的列表
					List<SampleTeamGroupVo> sampleTeamGroupVoListed = new ArrayList<>();
					List<Integer> teamGroupIdList = new ArrayList<>();
					for (SampleDistributionTeamVo sampleDistributionTeamVo : sampleDistributionTeamVoList) {
						SampleDistribution sampleDistribution = distributionMapper.selectById(sampleDistributionTeamVo.getDistributionId());
						pfTime = sampleDistribution.getCreateTime();
						TeamGroup teamGroup = teamGroupMapper.selectById(sampleDistribution.getTeamGroupId());
						//按照检测组：检测项1，检测项2 展示
						TeamVo teamVo = teamMapper.getDetail(sampleDistribution.getTeamId());
						if (checkTeamMap.containsKey(teamGroup.getName())) {
							String teamNames = checkTeamMap.get(teamGroup.getName());
							if (teamVo != null && teamVo.getIsPf() != 0) {
								teamNames = teamNames.equals("") ? teamVo.getName() : (teamNames + "、" + teamVo.getName());
								checkTeamMap.put(teamGroup.getName(), teamNames);
							}
						} else {
							checkTeamMap.put(teamGroup.getName(), teamVo.getName());
						}

						//检测项关联的检测设备信息
						List<SampleDistributionEquipmentVo> sampleDistributionEquipmentList =
								sampleDistributionEquipmentMapper.getEqupmentList(sampleDistributionTeamVo.getDistributionId(), loginUser.getId());
						if (sampleDistributionEquipmentList != null && sampleDistributionEquipmentList.size() > 0) {
							sampleDistributionTeamVo.setSampleDistributionEquipmentList(sampleDistributionEquipmentList);
						}

						if (!teamGroupIdList.contains(sampleDistribution.getTeamGroupId())) {
							teamGroupIdList.add(sampleDistribution.getTeamGroupId());
							SampleTeamGroupVo sampleTeamGroupVo = new SampleTeamGroupVo();
							sampleTeamGroupVo.setTeamGroupId(sampleDistribution.getTeamGroupId());
							sampleTeamGroupVo.setTeamGroupName(teamGroup.getName());
							List<SampleDistributionTeamVo> sdtvList = new ArrayList<>();
							sdtvList.add(sampleDistributionTeamVo);
							sampleTeamGroupVo.setSampleDistributionTeamVoList(sdtvList);
							sampleTeamGroupVoListed.add(sampleTeamGroupVo);
						} else {
							SampleTeamGroupVo sampleTeamGroupVo2 = null;
							for (SampleTeamGroupVo vo : sampleTeamGroupVoListed) {
								if (vo.getTeamGroupId() == sampleDistribution.getTeamGroupId()) {
									sampleTeamGroupVo2 = vo;
								}
							}
							List<SampleDistributionTeamVo> sdtvList = sampleTeamGroupVo2.getSampleDistributionTeamVoList();
							sdtvList.add(sampleDistributionTeamVo);
							sampleTeamGroupVo2.setSampleDistributionTeamVoList(sdtvList);
						}

					}

					sampleVo.setSampleTeamGroupVoList(sampleTeamGroupVoListed);
					//判断是否有校核退回状态的 如果有需要重新检测
					for (SampleTeamGroupVo stgVo : sampleTeamGroupVoListed) {
						List<SampleDistributionTeamVo> sdtVoList = stgVo.getSampleDistributionTeamVoList();
						if (sdtVoList != null && sdtVoList.size() > 0) {
							SampleDistributionTeamVo sdtVo = sdtVoList.get(0);
							stgVo.setStatus(sdtVo.getStatus());
							stgVo.setStatusValue(sdtVo.getStatusValue());
						}

						//如果时校核退回状态 查询校核输入表是否有校核被退回原因，如果有展示内容
						if (stgVo.getStatus() == 5) {
							QueryWrapper<SampleDistributionCheckinput> checkinputQueryWrapper = new QueryWrapper<>();
							checkinputQueryWrapper.eq("entrust_id", entrust.getId());
							checkinputQueryWrapper.eq("sample_id", sampleVo.getId());
							checkinputQueryWrapper.eq("team_group_id", stgVo.getTeamGroupId());
							checkinputQueryWrapper.eq("user_id", loginUser.getId());
							List<SampleDistributionCheckinput> checkinputList =
									sampleDistributionCheckinputMapper.selectList(checkinputQueryWrapper);
							if (checkinputList != null && checkinputList.size() > 0) {
								stgVo.setReturnReason(checkinputList.get(0).getRemark());
							}
						} else {
							stgVo.setReturnReason(null);
						}
					}
					String checkkteamStr = map2String(checkTeamMap);
					sampleVo.setTeamName(checkkteamStr);
					//一个平行样样品 主样和副样不能为同一个用户检测
					sampleVoList.add(sampleVo);
				}
			}
		}
		map.put("entrust", entrust);
		map.put("sampleVoList", sampleVoList);
		DateTimeFormatter df = DateTimeFormatter.ofPattern("yyyy-MM-dd");
		map.put("pfTime", df.format(pfTime));
		return BaseResponse.okData(map);
	}

	//Map转String
	private String map2String(Map<String, String> map) {
		List<String> list = new ArrayList<>();
		for (Map.Entry<String, String> entry : map.entrySet()) {
			String mapValue = entry.getKey() + ":" + entry.getValue();
			list.add(mapValue);
		}
		return list.toString().replaceAll("\\[", "").replaceAll("\\]", "");
	}

	/**
	 * 接受检测项目任务
	 *
	 * @param ids 样品派发表id
	 * @return
	 */
	@Transactional
	@Override
	public BaseResponse<String> acceptDistribution(Integer[] ids) {
		LoginUser loginUser = userService.getLoginUser();
		if (loginUser == null) {
			return BaseResponse.noLogin("请登录账号");
		}
		if (ids == null || ids.length <= 0) {
			return BaseResponse.errorMsg("参数错误");
		}

		List<SampleDistribution> sampleHandleList = new ArrayList<>();
		for (Integer id : ids) {
			SampleDistribution distribution = distributionMapper.selectById(id);
			if (distribution == null) {
				return BaseResponse.errorMsg("信息错误");
			}
			if (distribution.getStatus() != 0) {
				return BaseResponse.errorMsg("状态错误");
			}
			distribution.setStatus(1).setAcceptTime(LocalDateTime.now());
			sampleHandleList.add(distribution);
		}
		distributionService.updateBatchById(sampleHandleList);

		Sample sample = sampleService.getById(sampleHandleList.get(0).getSampleId());
		if (sample == null) {
			return BaseResponse.errorMsg("样品信息错误");
		}
		logsService.saveLog(SysLogs.ObjType.ENTRUST_LOG, sample.getEntrustId(), "接受检测项目任务", null);
		//处理消息
		userMessageService.dealMessage(loginUser.getId(), sample.getEntrustId(), SysUserMessage.MessageType.ENTRUST);
		return BaseResponse.okMsg("已接受检测项目任务");
	}

	/**
	 * 完成检测项目任务  重新检测也是用这个接口
	 *
	 * @param query 检测组完成请求
	 * @return
	 */
	@Transactional
	@Override
	public BaseResponse<String> finishDistribution(FinishCheckGroupQuery query) {
		LoginUser loginUser = userService.getLoginUser();
		if (loginUser == null) {
			return BaseResponse.noLogin("请登录账号");
		}
		if (query == null || query.getFinishCheckQueryList().size() <= 0) {
			return BaseResponse.errorMsg("参数错误");
		}
		if (query.getSampleId() == null || query.getTeamGroupId() == null) {
			return BaseResponse.errorMsg("请传入样品和检测组信息");
		}

		Sample sample = sampleMapper.selectById(query.getSampleId());
		if (sample == null) {
			return BaseResponse.errorMsg("样品信息错误");
		}
		Entrust entrust = entrustMapper.selectById(sample.getEntrustId());
		if (entrust == null) {
			return BaseResponse.errorMsg("委托信息错误");
		}
		TeamGroup teamGroup = teamGroupMapper.selectById(query.getTeamGroupId());
		if (entrust == null) {
			return BaseResponse.errorMsg("检测组信息错误");
		}
		List<FinishCheckQuery> finishCheckQueryList = query.getFinishCheckQueryList();
		if (finishCheckQueryList != null && finishCheckQueryList.size() > 0) {
			for (FinishCheckQuery finishCheckQuery : finishCheckQueryList) {
				SampleDistribution distribution = distributionMapper.selectById(finishCheckQuery.getDistributionId());
				if (distribution == null) {
					return BaseResponse.errorMsg("信息错误");
				}
				distribution.setStatus(2).setFinishTime(LocalDateTime.now());
				distributionMapper.updateById(distribution);

				//删除之前的设备记录
				QueryWrapper<SampleDistributionEquipment> sdbeWrapper = new QueryWrapper<>();
				sdbeWrapper.eq("distribution_id", distribution.getId());
				List<SampleDistributionEquipment> sdbeList = sampleDistributionEquipmentService.list(sdbeWrapper);
				if (sdbeList != null && sdbeList.size() > 0) {
					sampleDistributionEquipmentService.remove(sdbeWrapper);
				}
				//保存选择设备信息
				if (finishCheckQuery.getEquipmentIdList() != null && finishCheckQuery.getEquipmentIdList().size() > 0) {
					List<SampleDistributionEquipment> sdeList = new ArrayList<>();
					for (int i = 0; i < finishCheckQuery.getEquipmentIdList().size(); i++) {
						Equipment equipment = equipmentMapper.selectById(finishCheckQuery.getEquipmentIdList().get(i));
						if (equipment == null) {
							return BaseResponse.errorMsg("设备信息错误");
						}
						SampleDistributionEquipment sdEquipment = new SampleDistributionEquipment();
						sdEquipment.setDistributionId(distribution.getId())
								.setEquipmentId(equipment.getId())
								.setEquipmentName(equipment.getName())
								.setUserId(loginUser.getId())
								.setCreateTime(LocalDateTime.now());
						sdeList.add(sdEquipment);
						//添加设备使用记录  设备使用记录表 equipment_use
						EquipmentUse equipmentUse2 = new EquipmentUse();
						equipmentUse2.setEquipmentId(equipment.getId())
								.setUserId(loginUser.getId())
								.setUserName(loginUser.getName())
								.setUseDate(LocalDate.now())
								.setProjectId(entrust.getProjectId())
								.setProjectName(entrust.getProjectName())
								.setProjectCode(entrust.getProjectCode())
								.setTestItem(teamGroup.getName())
								.setCreateTime(LocalDateTime.now())
								.setRemark("委托派发-数据校核");
						equipmentUseMapper.insert(equipmentUse2);
					}
					sampleDistributionEquipmentService.saveBatch(sdeList);
				}
				//改变校核表状态为待校核状态
				SampleDistributionCheckinput sampleDistributionCheckinput =
						sampleDistributionCheckinputMapper.selectById(distribution.getCheckInputId());
				if (sampleDistributionCheckinput != null) {
					sampleDistributionCheckinput.setStatus(2);//待校核
					sampleDistributionCheckinputMapper.updateById(sampleDistributionCheckinput);
				}
			}
		}

		//判断该委托单下的所有样品的检测 及 检测项是否全部完成
		QueryWrapper<SampleDistribution> sampleDistributionWrapper = new QueryWrapper<>();
		sampleDistributionWrapper.eq("sample_id", sample.getId());
		sampleDistributionWrapper.lt("status", 2);
		List<SampleDistribution> sampleDistributionIngList = distributionMapper.selectList(sampleDistributionWrapper);
		if (sampleDistributionIngList == null || sampleDistributionIngList.size() == 0) {
			sample.setIsDistribution(1);//样品检测全部完成
		}
		sampleMapper.updateById(sample);

		if (entrust != null) {
			QueryWrapper<SampleDistribution> sdWrapper = new QueryWrapper<>();
			sdWrapper.eq("entrust_id", entrust.getId());
			sdWrapper.lt("status", 2);
			List<SampleDistribution> sdIngList = distributionMapper.selectList(sdWrapper);
			if (sdIngList == null || sdIngList.size() == 0) {
				entrust.setStatus(6);
				;//委托单改成检测完成状态
			}
           /* List<Sample> sampleList = sampleMapper.getNoDistributionList(sample.getEntrustId());
            if (sampleList == null || sampleList.size() == 0) {
                entrust.setStatus(6);
            }*/
			entrustMapper.updateById(entrust);
			//计算产值
			List<SampleTeamGroupUserVo> sampleDistributionTeamGroupVoList = distributionMapper.getDistributionTeamGroupList(sample.getId(), loginUser.getId());
			if (sampleDistributionTeamGroupVoList != null && sampleDistributionTeamGroupVoList.size() > 0) {

				List<Integer> teamGroupIdList = new ArrayList<>();

				for (SampleTeamGroupUserVo sdVo : sampleDistributionTeamGroupVoList) {
					try {
						//先判断该检测是按照检测组计算产值 还是按照检测项计算产值
						//如果时按照检测项，直接计算，如果是按照检测组，则需要判断检测组是否计算过，去除重复的检测组计算。
						//然后判断是否已经参与过产值计算，如果参与过，不用再进行计算

						//判断检测组是否配置
						QueryWrapper<NormProduction> qw = new QueryWrapper<>();
						qw.eq("type", 1);
						qw.eq("assess_id", sdVo.getTeamGroupId());
						List<NormProduction> npList = normProductionMapper.selectList(qw);
						if (npList != null && npList.size() > 0) {
							//判断该样品是否已经计算产值，如果参与过计算不进行再次的产值计算
							QueryWrapper<Preciprice> ppWrapper = new QueryWrapper<>();
							ppWrapper.eq("type", 1);
							ppWrapper.eq("entrust_id", entrust.getId());
							ppWrapper.eq("sample_id", sample.getId());
							ppWrapper.eq("target_id", sdVo.getTeamGroupId());
							ppWrapper.eq("user_id", sdVo.getUserId());
							List<Preciprice> ppList = iPrecipriceService.list(ppWrapper);
							if(ppList == null || ppList.size()==0){
								iPrecipriceService.createPreciprice(sdVo.getUserId(), entrust.getId(), sdVo.getTeamGroupId(), 1, sample.getId(), false);
							}
						}else{
							QueryWrapper<NormProduction> qw2 = new QueryWrapper<>();
							qw2.eq("type", 2);
							qw2.eq("assess_id", sdVo.getTeamId());
							List<NormProduction> npList2 = normProductionMapper.selectList(qw2);
							if (npList2 != null && npList2.size() > 0) {
								//判断该样品是否已经计算产值，如果参与过计算不进行再次的产值计算
								QueryWrapper<Preciprice> ppWrapper = new QueryWrapper<>();
								ppWrapper.eq("type", 2);
								ppWrapper.eq("entrust_id", entrust.getId());
								ppWrapper.eq("sample_id", sample.getId());
								ppWrapper.eq("target_id", sdVo.getTeamId());
								ppWrapper.eq("user_id", sdVo.getUserId());
								List<Preciprice> ppList = iPrecipriceService.list(ppWrapper);
								if(ppList == null || ppList.size()==0){
									iPrecipriceService.createPreciprice(sdVo.getUserId(), entrust.getId(), sdVo.getTeamId(), 2, sample.getId(), false);
								}
							}
						}


					} catch (Exception e) {
						return BaseResponse.errorMsg(e.getMessage());
					}
				}
			}
		}

		logsService.saveLog(SysLogs.ObjType.ENTRUST_LOG, entrust.getId(), "完成检测项目任务", null);
		return BaseResponse.okMsg("已完成检测项目任务");
	}

	/**
	 * 样品检测退回
	 *
	 * @param query
	 * @return
	 */
	@Transactional
	@Override
	public BaseResponse<String> returnDistribution(ReturnDistributionQuery query) {
		LoginUser loginUser = userService.getLoginUser();
		if (loginUser == null) {
			return BaseResponse.noLogin("请登录账号");
		}
		if (query == null || query.getSampleId() == null || query.getTeamGroupId() == null ||
				query.getDistributionid().size() <= 0) {
			return BaseResponse.errorMsg("参数错误");
		}
		if (query.getReturnReason().length() > 100) {
			return BaseResponse.errorMsg("退回原因内容过长");
		}

		//判断该委托单下的所有样品的处理 及 处理项是否全部完成
		Sample sample = sampleMapper.selectById(query.getSampleId());
		if (sample == null) {
			return BaseResponse.errorMsg("样品信息错误");
		}
		Entrust entrust = entrustMapper.selectById(sample.getEntrustId());
		if (entrust == null) {
			return BaseResponse.errorMsg("委托信息错误");
		}
		if (query.getDistributionid() != null && query.getDistributionid().size() > 0) {
			for (Integer distributionId : query.getDistributionid()) {
				SampleDistribution sampleDistribution = distributionMapper.selectById(distributionId);
				if (sampleDistribution == null) {
					return BaseResponse.errorMsg("信息错误");
				}
				if (sampleDistribution.getStatus() != 1 && sampleDistribution.getStatus() != 2) {
					return BaseResponse.errorMsg("状态错误");
				}
				if (loginUser.getId() != sampleDistribution.getUserId()) {
					return BaseResponse.errorMsg("请登录样品处理人账号操作");
				}
				sampleDistribution.setStatus(3).setRemark(query.getReturnReason());
				distributionMapper.updateById(sampleDistribution);

				//样品检测退回记录
				SampleDistributionReturn distributionReturn = new SampleDistributionReturn();
				distributionReturn.setSampleDistributionId(sampleDistribution.getId());
				distributionReturn.setUserId(sampleDistribution.getUserId());
				distributionReturn.setCreateTime(LocalDateTime.now());
				distributionReturn.setReturnReason(sampleDistribution.getRemark());
				distributionReturn.setRemark(null);
				sampleDistributionReturnMapper.insert(distributionReturn);
			}
		}

		//检测人退回样品检测任务，提醒评审人员查看
		List<Integer> approvalIdList = sysApprovalMapper.getApprovalId("委托评审");
		if (approvalIdList == null) {
			return BaseResponse.errorMsg("委托评审信息错误");
		}
		String userIds = "";
		for (Integer approvalId : approvalIdList) {
			SysUser approvalUser = userService.getById(approvalId);
			if (approvalUser == null) {
				return BaseResponse.errorMsg("委托评审信息错误");
			}
			BaseResponse wrapper = userMessageService.sendMessage(approvalId, "您有一条被退回的检测任务", entrust.getId(), SysUserMessage.MessageType.ENTRUST);
			if (wrapper.getCode() != 200) {
				return wrapper;
			}
			if (StringUtils.isNotBlank(approvalUser.getWxId())) {
				userIds = userIds.equals("") ? approvalUser.getWxId() : userIds + "|" + approvalUser.getWxId();
			}
		}
		//发送企业微信消息内容
		String systemUrL = weiXinService.createMessage(entrust.getId(), "/blanche/blancherecive");
		String content = "你好！您有样品检测任务被退回，" + "委托单号" + entrust.getEntrustCode() + "请前往\n" + systemUrL + "进行查看。";
		weiXinService.sendTextMessage(userIds, content);

		//记录委托单日志
		logsService.saveLog(SysLogs.ObjType.ENTRUST_LOG, entrust.getId(), "退回样品检测任务", null);
		return BaseResponse.okMsg("已退回样品检测任务");
	}

	/**
	 * 样品检测退回任务分页
	 *
	 * @param pageQuery
	 * @param projectCode
	 * @return
	 */
	@Override
	public BaseResponse<IPage<SampleReturnHandleVo>> getSampleReturnDistributioPage(PageQuery pageQuery, String entrustCode, Integer userId,
	                                                                                String clientName, String projectName, String projectCode, Integer isUrgent) {
		LoginUser loginUser = userService.getLoginUser();
		if (loginUser == null) {
			return BaseResponse.noLogin("请登录账号");
		}
		Map<String, Object> params = new HashMap<>();
		params.put("entrustCode", entrustCode);
		params.put("clientName", clientName);
		params.put("projectName", projectName);
		params.put("projectCode", projectCode);
		params.put("isUrgent", isUrgent);
		params.put("userId", userId);

		Page<SampleReturnHandleVo> page = new Page<>(pageQuery.getPageNo(), pageQuery.getPageSize());
		IPage<SampleReturnHandleVo> pages = entrustMapper.getSampleReturnDistributioPage(page, params);
		return BaseResponse.okData(pages);
	}

	/**
	 * 重新派发样品检测任务
	 *
	 * @param query
	 * @return
	 */
	public BaseResponse<String> distributioAgain(HandleAgainQuery query) {
		LoginUser loginUser = userService.getLoginUser();
		if (loginUser == null) {
			return BaseResponse.noLogin("请登录账号");
		}
		List<Integer> approvalIdList = sysApprovalMapper.getApprovalId("委托评审");
		if (approvalIdList == null) {
			return BaseResponse.errorMsg("委托评审信息错误");
		}
		if (!approvalIdList.contains(loginUser.getId())) {
			return BaseResponse.errorMsg("没有重新派发权限");
		}

		if (query == null || query.getId() == null) {
			return BaseResponse.errorMsg("参数错误");
		}
		if (query.getUserId() == null) {
			return BaseResponse.errorMsg("请选择接收人");
		}
		SampleDistribution sampleDistribution = distributionMapper.selectById(query.getId());
		if (sampleDistribution == null) {
			return BaseResponse.errorMsg("信息错误");
		}
		if (sampleDistribution.getStatus() != 3) {
			return BaseResponse.errorMsg("状态错误");
		}
		sampleDistribution.setStatus(1)
				.setUserId(query.getUserId())
				.setRemark(null);
		distributionMapper.updateById(sampleDistribution);

		//判断该委托单下的所有样品的处理 及 处理项是否全部完成
		Sample sample = sampleMapper.selectById(sampleDistribution.getSampleId());
		if (sample == null) {
			return BaseResponse.errorMsg("样品信息错误");
		}
		Entrust entrust = entrustMapper.selectById(sample.getEntrustId());
		if (entrust == null) {
			return BaseResponse.errorMsg("委托信息错误");
		}
		//记录委托单日志
		logsService.saveLog(SysLogs.ObjType.ENTRUST_LOG, entrust.getId(), "重新派发样品检测任务", null);

		//消息推送
		BaseResponse wrapper = userMessageService.sendMessage(sampleDistribution.getUserId(), "您有一条样品检测信息等待处理", entrust.getId(), SysUserMessage.MessageType.ENTRUST);
		if (wrapper.getCode() != 200) {
			return wrapper;
		}

		//重新派发样品检测 提醒检测新的人员查看
		// 发送企业微信消息内容
		String systemUrL = weiXinService.createMessage(entrust.getId(), "/blanche/recivedetail");
		String content = "你好！您有新的样品检测任务，" + "委托单号" + entrust.getEntrustCode() + "请前往\n" + systemUrL + "进行查看。";
		SysUser handleUser = userService.getById(sampleDistribution.getUserId());
		if (handleUser == null || StringUtils.isEmpty(handleUser.getWxId())) {
			return BaseResponse.errorMsg("样品检测人信息错误");
		}
		weiXinService.sendTextMessage(handleUser.getWxId(), content);

		return BaseResponse.okMsg("已重新派发检测任务");
	}

	/**
	 * 数据校核分页
	 *
	 * @param pageQuery
	 * @param startDate
	 * @param endDate
	 * @param status
	 * @param clientId
	 * @param projectName
	 * @param projectCode
	 * @return
	 */
	@Override
	public BaseResponse<IPage<EntrustVo>> getCheckPage(PageQuery pageQuery,
	                                                   String entrustCode, String projectName, String projectCode,
	                                                   Integer clientId, String clientName, Integer status,
	                                                   String startDate, String endDate, Integer isUrgent) {
		LoginUser loginUser = userService.getLoginUser();
		if (loginUser == null) {
			return BaseResponse.noLogin("请登录账号");
		}
		Map<String, Object> params = new HashMap<>();

		params.put("entrustCode", entrustCode);
		params.put("projectName", projectName);
		params.put("projectCode", projectCode);
		params.put("clientId", clientId);
		params.put("clientName", clientName);
		params.put("status", status);
		params.put("startDate", startDate);
		params.put("endDate", endDate);
		params.put("isUrgent", isUrgent);

		List<Integer> approvalIdList = sysApprovalMapper.getApprovalId("数据校核");
		if (approvalIdList == null) {
			return BaseResponse.errorMsg("数据校核信息错误");
		}
		if (!approvalIdList.contains(loginUser.getId())) {
			return BaseResponse.errorMsg("当前登录账号没有数据校核权限");
		}
		Page<EntrustVo> page = new Page<>(pageQuery.getPageNo(), pageQuery.getPageSize());
		IPage<EntrustVo> pages = entrustMapper.getCheckPage(page, params);
		List<EntrustVo> list = pages.getRecords();
		if (list != null && list.size() > 0) {
			for (EntrustVo entrustVo : list) {
				QueryWrapper<SampleTmp> queryWrapper = new QueryWrapper<>();
				queryWrapper.eq("entrust_id", entrustVo.getId());
				List<SampleTmp> sampleTmpList = sampleTmpService.list(queryWrapper);
				//检测项目名称
				String sampleNames = "";
				if (sampleTmpList != null && sampleTmpList.size() > 0) {
					//样品名称 列表中的拼接
					for (SampleTmp sampleTmp : sampleTmpList) {
						sampleNames = sampleNames.equals("") ? sampleTmp.getName() : (sampleNames + "、" + sampleTmp.getName());
					}
					entrustVo.setSampleNames(sampleNames);
					//检测项 和检测依据 列表里只显示第一个样品的
					SampleTmp sampleTmp = sampleTmpList.get(0);
					String teamIds = sampleTmp.getTeamIds();
					String checkTeam = "";
					if (teamIds != null) {
						String[] teamIdS = teamIds.split("、");
						for (String teamId : teamIdS) {
							Team team = teamMapper.selectById(Integer.valueOf(teamId));
							if (team != null) {
								checkTeam = checkTeam.equals("") ? team.getName() : (checkTeam + "、" + team.getName());
							}
						}
					}
					entrustVo.setCheckTeam(checkTeam);
					entrustVo.setCheckMethodNumber(sampleTmp.getMethodNumbers());
				}
			}
		}
		pages.setRecords(list);
		return BaseResponse.okData(pages);
	}

	/**
	 * 数据校核查看列表
	 *
	 * @param id 委托表id
	 * @return
	 */
	@Override
	public BaseResponse<CheckVo> getCheckList(Integer id) {
		LoginUser loginUser = userService.getLoginUser();
		if (loginUser == null) {
			return BaseResponse.noLogin("请登录账号");
		}
		Entrust entrust = entrustMapper.selectById(id);
		if (entrust == null) {
			return BaseResponse.errorMsg("信息错误");
		}
		EntrustVo entrustVo = entrustMapper.getDetail(id);
		CheckVo checkVo = new CheckVo();
		BeanUtils.copyProperties(entrustVo, checkVo);
		checkVo.setEntrustId(entrustVo.getId());
		QueryWrapper<SampleDistribution> queryWrapper = new QueryWrapper<>();
		queryWrapper.eq("entrust_id", entrust.getId());
		List<SampleDistribution> distributionList = distributionMapper.selectList(queryWrapper);

		if (distributionList != null && distributionList.size() > 0) {
			List<CheckUserVo> checkUserVoList = new ArrayList<>();

			//先循环遍历 按照检测人分组
			Map<Integer, List<SampleDistribution>> mapUser = new HashMap<>();
			for (SampleDistribution sampleDistribution : distributionList) {
				List<SampleDistribution> list2 = mapUser.get(sampleDistribution.getUserId());
				if (list2 == null) {
					list2 = new ArrayList<>();
					list2.add(sampleDistribution);
				} else {
					list2.add(sampleDistribution);
				}
				mapUser.put(sampleDistribution.getUserId(), list2);
			}

			//再次将人下面的检测项信息进行按照样品分组
			if (mapUser != null && mapUser.size() > 0) {
				for (Map.Entry entryUser : mapUser.entrySet()) {
					CheckUserVo checkUserVo = new CheckUserVo();
					SysUser sysUser = userService.getById((Integer) entryUser.getKey());
					checkUserVo.setUserId(sysUser.getId());
					checkUserVo.setUserName(sysUser.getName());

					List<SampleDistribution> list3 = (List<SampleDistribution>) entryUser.getValue();
					if (list3 != null && list3.size() > 0) {
						List<CheckSampleVo> checkSampleVoList = new ArrayList<>();
						Map<Integer, List<SampleDistribution>> mapSample = new HashMap<>();
						for (SampleDistribution sampleDistribution : list3) {
							List<SampleDistribution> list2 = mapSample.get(sampleDistribution.getSampleId());
							if (list2 == null) {
								list2 = new ArrayList<>();
								list2.add(sampleDistribution);
							} else {
								list2.add(sampleDistribution);
							}
							mapSample.put(sampleDistribution.getSampleId(), list2);
						}

						if (mapSample != null && mapSample.size() > 0) {
							for (Map.Entry entrySample : mapSample.entrySet()) {
								CheckSampleVo checkSampleVo = new CheckSampleVo();
								Sample sample = sampleMapper.selectById((Integer) entrySample.getKey());
								BeanUtils.copyProperties(sample, checkSampleVo);
								checkSampleVo.setSampleId(sample.getId());

								List<SampleDistribution> list4 = (List<SampleDistribution>) entrySample.getValue();
								if (list4 != null && list4.size() > 0) {
									List<CheckTeamGroupVo> checkTeamGroupVoList = new ArrayList<>();
									Map<Integer, List<SampleDistribution>> mapTeamGroup = new HashMap<>();
									for (SampleDistribution sampleDistribution : list4) {
										List<SampleDistribution> list2 = mapTeamGroup.get(sampleDistribution.getTeamGroupId());
										if (list2 == null) {
											list2 = new ArrayList<>();
											list2.add(sampleDistribution);
										} else {
											list2.add(sampleDistribution);
										}
										mapTeamGroup.put(sampleDistribution.getTeamGroupId(), list2);
									}
									if (mapTeamGroup != null && mapTeamGroup.size() > 0) {
										for (Map.Entry entryTeamGroup : mapTeamGroup.entrySet()) {
											CheckTeamGroupVo checkTeamGroupVo = new CheckTeamGroupVo();
											TeamGroup teamGroup = teamGroupMapper.selectById((Integer) entryTeamGroup.getKey());
											checkTeamGroupVo.setTeamGroupId(teamGroup.getId());
											checkTeamGroupVo.setTeamGroupName(teamGroup.getName());
											List<SampleDistribution> list5 = (List<SampleDistribution>) entryTeamGroup.getValue();
											if (list5 != null && list5.size() > 0) {
												List<CheckTeamVo> checkTeamVoList = new ArrayList<>(); //=========================
												for (SampleDistribution sampleDistribution : list5) {
													CheckTeamVo checkTeamVo = new CheckTeamVo();
													checkTeamVo.setDistributionId(sampleDistribution.getId());
													checkTeamVo.setTeamId(sampleDistribution.getTeamId());
													Team team = teamMapper.selectById(sampleDistribution.getTeamId());
													checkTeamVo.setTeamName(team.getName());
													checkTeamVo.setStatus(sampleDistribution.getStatus());
													String statusValue = sampleDistribution.getStatus() == 0 ? "未接受"
															: sampleDistribution.getStatus() == 1 ? "接受"
															: sampleDistribution.getStatus() == 2 ? "待校核"
															: sampleDistribution.getStatus() == 3 ? "退回"
															: sampleDistribution.getStatus() == 4 ? "校核通过"
															: sampleDistribution.getStatus() == 5 ? "校核退回"
															: "";
													checkTeamVo.setStatusValue(statusValue);
													checkTeamVo.setSortNo(team.getSortNo());
													checkTeamVoList.add(checkTeamVo);
												}
												checkTeamGroupVo.setCheckTeamVoList(checkTeamVoList);
											}
											checkTeamGroupVoList.add(checkTeamGroupVo);
										}
									}
									checkSampleVo.setCheckTeamGroupVoList(checkTeamGroupVoList);

								}
								checkSampleVoList.add(checkSampleVo);
							}
							checkUserVo.setCheckSampleVoList(checkSampleVoList.stream().sorted(Comparator.comparing(CheckSampleVo::getParallelCode)).collect(Collectors.toList()));
						}
					}
					checkUserVoList.add(checkUserVo);
				}
			}
			checkVo.setCheckUserVoList(checkUserVoList);
		}

		return BaseResponse.okData(checkVo);
	}

	/**
	 * 数据校核结果查看
	 *
	 * @param entrustId
	 * @param userId
	 * @return
	 */
	@Override
	public BaseResponse<CheckUserResultVo> getCheckResult(Integer entrustId, Integer userId) {
		LoginUser loginUser = userService.getLoginUser();
		if (loginUser == null) {
			return BaseResponse.noLogin("请登录账号");
		}
		Entrust entrust = entrustMapper.selectById(entrustId);
		if (entrust == null) {
			return BaseResponse.errorMsg("信息错误");
		}
		EntrustVo entrustVo = entrustMapper.getDetail(entrustId);
		CheckUserResultVo checkUserResultVo = new CheckUserResultVo();
		checkUserResultVo.setEntrustId(entrust.getId());
		checkUserResultVo.setEntrustCode(entrust.getEntrustCode());
		SysUser sysUser = userService.getById(userId);
		checkUserResultVo.setUserId(sysUser.getId());
		checkUserResultVo.setUserName(sysUser.getName());
		checkUserResultVo.setProjectName(entrustVo.getProjectName());
		checkUserResultVo.setProjectCode(entrustVo.getProjectCode());
		checkUserResultVo.setProjectType(entrustVo.getProjectType());
		checkUserResultVo.setEntrustUserName(entrustVo.getUserName());
		checkUserResultVo.setClientName(entrustVo.getClientName());

		QueryWrapper<SampleDistribution> queryWrapper = new QueryWrapper<>();
		queryWrapper.eq("entrust_id", entrust.getId());
		queryWrapper.eq("user_id", userId);
		List<SampleDistribution> distributionList = distributionMapper.selectList(queryWrapper);

		if (distributionList != null && distributionList.size() > 0) {
			List<CheckTeamGroupResultVo> checkTeamGroupResultVoList = new ArrayList<>();
			//先循环遍历 按照检测组分组
			Map<Integer, List<SampleDistribution>> mapTeamGroup = new HashMap<>();
			for (SampleDistribution sampleDistribution : distributionList) {
				List<SampleDistribution> list2 = mapTeamGroup.get(sampleDistribution.getTeamGroupId());
				if (list2 == null) {
					list2 = new ArrayList<>();
					list2.add(sampleDistribution);
				} else {
					list2.add(sampleDistribution);
				}
				mapTeamGroup.put(sampleDistribution.getTeamGroupId(), list2);
			}

			//再次将检测组下面的按照样品去分组
			if (mapTeamGroup != null && mapTeamGroup.size() > 0) {

				for (Map.Entry entryTeamGroup : mapTeamGroup.entrySet()) {
					CheckTeamGroupResultVo checkTeamGroupResultVo = new CheckTeamGroupResultVo();
					TeamGroup teamGroup = teamGroupMapper.selectById((Integer) entryTeamGroup.getKey());
					checkTeamGroupResultVo.setTeamGroupId(teamGroup.getId());
					checkTeamGroupResultVo.setTeamGroupName(teamGroup.getName());

					List<SampleDistribution> list3 = (List<SampleDistribution>) entryTeamGroup.getValue();
					if (list3 != null && list3.size() > 0) {
						List<CheckSampleResultVo> checkSampleResultVoList = new ArrayList<>();
						Map<Integer, List<SampleDistribution>> mapSample = new HashMap<>();
						for (SampleDistribution sampleDistribution : list3) {
							List<SampleDistribution> list2 = mapSample.get(sampleDistribution.getSampleId());
							if (list2 == null) {
								list2 = new ArrayList<>();
								list2.add(sampleDistribution);
							} else {
								list2.add(sampleDistribution);
							}
							mapSample.put(sampleDistribution.getSampleId(), list2);
						}

						if (mapSample != null && mapSample.size() > 0) {
							for (Map.Entry entrySample : mapSample.entrySet()) {
								CheckSampleResultVo checkSampleResultVo = new CheckSampleResultVo();  //================
								Sample sample = sampleMapper.selectById((Integer) entrySample.getKey());
								BeanUtils.copyProperties(sample, checkSampleResultVo);
								BeanUtils.copyProperties(entrustVo, checkSampleResultVo);
								checkSampleResultVo.setSampleId(sample.getId());

								//根据委托id，样品id 检测组id，检测人id判断校核输入表是否有数据，如果有显示校核输入表数据，如果没有显示检测项表数据。
								String inputResult = null;

								//通过检测组和检测人查询上次输入的检测组校核元素 用于展示
								QueryWrapper<Titration> titrationQueryWrapper = new QueryWrapper<>();
								titrationQueryWrapper.eq("user_id", userId);
								titrationQueryWrapper.eq("team_group_id", teamGroup.getId());
								List<Titration> titrationList = titrationMapper.selectList(titrationQueryWrapper);
								if (titrationList != null && titrationList.size() > 0) {
									inputResult = titrationList.get(0).getCheckElement();
								} else {
									inputResult = teamGroup.getCheckElement();
								}

								Integer status = 2;
								QueryWrapper<SampleDistributionCheckinput> checkinputQueryWrapper = new QueryWrapper<>();
								checkinputQueryWrapper.eq("entrust_id", entrust.getId());
								checkinputQueryWrapper.eq("sample_id", sample.getId());
								checkinputQueryWrapper.eq("team_group_id", teamGroup.getId());
								checkinputQueryWrapper.eq("user_id", userId);
								List<SampleDistributionCheckinput> checkinputList =
										sampleDistributionCheckinputMapper.selectList(checkinputQueryWrapper);
								if (checkinputList != null && checkinputList.size() > 0) {
									SampleDistributionCheckinput checkinput = checkinputList.get(0);
									if (checkinput != null) {
										inputResult = checkinput.getInputResult();
										status = checkinput.getStatus();
									}
								}
								checkSampleResultVo.setInputResult(inputResult);
								checkSampleResultVo.setStatus(status);

								List<SampleDistribution> list4 = (List<SampleDistribution>) entrySample.getValue();
								if (list4 != null && list4.size() > 0) {
									List<CheckTeamResultVo> checkTeamResultVoList = new ArrayList<>();  //===========
									for (SampleDistribution sampleDistribution : list4) {
										CheckTeamResultVo checkTeamResultVo = new CheckTeamResultVo();
										checkTeamResultVo.setDistributionId(sampleDistribution.getId());
										checkTeamResultVo.setTeamId(sampleDistribution.getTeamId());
										Team team = teamMapper.selectById(sampleDistribution.getTeamId());
										checkTeamResultVo.setTeamName(team.getName());
										checkTeamResultVo.setRetain(team.getRetain());
										checkTeamResultVo.setStatus(sampleDistribution.getStatus());
										String statusValue = sampleDistribution.getStatus() == 0 ? "未接受"
												: sampleDistribution.getStatus() == 1 ? "接受"
												: sampleDistribution.getStatus() == 2 ? "待校核"
												: sampleDistribution.getStatus() == 3 ? "退回"
												: sampleDistribution.getStatus() == 4 ? "校核通过"
												: sampleDistribution.getStatus() == 5 ? "校核退回"
												: "";
										checkTeamResultVo.setStatusValue(statusValue);
										checkTeamResultVo.setMethodId(team.getMethodId());
										checkTeamResultVo.setMethodName(team.getMethodName());
										checkTeamResultVo.setNumber(team.getNumber());
										checkTeamResultVo.setSortNo(team.getSortNo());
										checkTeamResultVo.setUnits(team.getUnits());
										//当前检测人检测结果
										checkTeamResultVo.setDistributionResult(sampleDistribution.getDistributionResult());

										//检测项关联的检测设备信息
										List<SampleDistributionEquipmentVo> sampleDistributionEquipmentList =
												sampleDistributionEquipmentMapper.getEqupmentList(sampleDistribution.getId(), userId);
										if (sampleDistributionEquipmentList != null && sampleDistributionEquipmentList.size() > 0) {
											checkTeamResultVo.setSampleDistributionEquipmentList(sampleDistributionEquipmentList);
										}

										//获取其他平行样的结果值
										List<String> ortherResultList = new ArrayList<>();
										List<Sample> otherSampleList = sampleMapper.getOtherSampleList(entrust.getId(), sample.getCementCode(), sample.getId());
										if (otherSampleList != null && otherSampleList.size() > 0) {
											for (Sample s : otherSampleList) {

												QueryWrapper<SampleDistribution> ortherWrapper = new QueryWrapper<>();
												ortherWrapper.eq("entrust_id", entrust.getId());
												ortherWrapper.eq("sample_id", s.getId());
												ortherWrapper.eq("team_group_id", checkTeamGroupResultVo.getTeamGroupId());
												ortherWrapper.eq("team_id", checkTeamResultVo.getTeamId());
												List<SampleDistribution> ortherDdistributionList = distributionMapper.selectList(ortherWrapper);
												if (ortherDdistributionList.size() > 1) {
													return BaseResponse.errorMsg("检测派发表检测项信息重复");
												}
												if (ortherDdistributionList != null && ortherDdistributionList.size() > 0) {
													SampleDistribution other = ortherDdistributionList.get(0);
													if (StringUtils.isNotBlank(other.getDistributionResult())) {
														ortherResultList.add(other.getDistributionResult());
													}
												}

											}
										}
										checkTeamResultVo.setOrtherUserResult(ortherResultList);

										//该检测项的误差结果值和最终结果值
										if (sampleDistribution.getCheckId() != null) {
											SampleDistributionCheck sampleDistributionCheck = sampleDistributionCheckMapper.selectById(sampleDistribution.getCheckId());
											if (sampleDistributionCheck != null) {
												checkTeamResultVo.setDistributionError(sampleDistributionCheck.getDistributionError());
												checkTeamResultVo.setLastResult(sampleDistributionCheck.getLastResult());
											}
										}
										checkTeamResultVoList.add(checkTeamResultVo);
									}

									//********如果检测项不需要派发检测，并且关联当前检测人的 直接显示**********
                                   /* List<Integer> teamIDs = new ArrayList<>(); //检测项
                                    String teamIds = sample.getTeamIds();
                                    if (teamIds != null) {
                                        String[] teamIdS = teamIds.split("、");
                                        for (String teamId : teamIdS) {
                                            TeamVo teamVo = teamMapper.getDetail(Integer.valueOf(teamId));
                                            //必须为同一个检测组下面的检测项，而且为不需要派发的检测项。
                                            if (teamVo != null && teamVo.getIsPf()==0 && teamVo.getGroupId() == checkTeamGroupResultVo.getTeamGroupId()){
                                                String handleIds = teamVo.getHandleId();
                                                if (handleIds != null) {
                                                    String[] handleIdS = handleIds.split("、");
                                                    for (String handleId : handleIdS) {
                                                        //样品id 处理人id  关联的处理项表信息进行展示
                                                        QueryWrapper<SampleHandle> handleWrapper = new QueryWrapper<>();
                                                        handleWrapper.eq("sample_id", sample.getId());
                                                        handleWrapper.eq("user_id", userId);
                                                        handleWrapper.eq("handle_id", handleId);
                                                        List<SampleHandle> handleList = sampleHandleMapper.selectList(handleWrapper);
                                                        if(handleList != null && handleList.size()>0){
                                                            if(!teamIDs.contains(teamVo.getId())){
                                                                SampleHandle sampleHandle = handleList.get(0);
                                                                teamIDs.add(teamVo.getId());
                                                                CheckTeamResultVo checkTeamResultVo = new CheckTeamResultVo();
                                                                checkTeamResultVo.setDistributionId(sampleHandle.getId());
                                                                checkTeamResultVo.setTeamId(teamVo.getId());
                                                                Team team = teamMapper.selectById(teamVo.getId());
                                                                checkTeamResultVo.setTeamName(team.getName());
                                                                checkTeamResultVo.setStatus(sampleHandle.getStatus());
                                                                String statusValue = sampleHandle.getStatus()==0?"未接受"
                                                                        :sampleHandle.getStatus()==1?"接受"
                                                                        :sampleHandle.getStatus()==2?"待校核"
                                                                        :sampleHandle.getStatus()==3?"退回"
                                                                        :sampleHandle.getStatus()==4?"校核通过"
                                                                        :sampleHandle.getStatus()==5?"校核退回"
                                                                        :"";
                                                                checkTeamResultVo.setStatusValue(statusValue);
                                                                checkTeamResultVo.setMethodId(team.getMethodId());
                                                                checkTeamResultVo.setMethodName(team.getMethodName());
                                                                checkTeamResultVo.setNumber(team.getNumber());
                                                                checkTeamResultVoList.add(checkTeamResultVo);

                                                            }

                                                        }

                                                    }
                                                }
                                            }
                                        }
                                    }*/

									//********如果检测项不需要派发检测，并且关联当前检测人的 直接显示**********

									checkSampleResultVo.setCheckTeamResultVoList(checkTeamResultVoList);
								}
								checkSampleResultVoList.add(checkSampleResultVo);
							}

						}
						checkTeamGroupResultVo.setCheckSampleResultVoList(checkSampleResultVoList.stream().sorted(Comparator.comparing(CheckSampleResultVo::getParallelCode)).collect(Collectors.toList()));
					}
					checkTeamGroupResultVoList.add(checkTeamGroupResultVo);
				}
			}
			checkUserResultVo.setCheckTeamGroupResultVoList(checkTeamGroupResultVoList);
		}

		return BaseResponse.okData(checkUserResultVo);
	}

	/**
	 * 校核计算 立马返回数据
	 *
	 * @param query
	 * @return
	 */
	@Transactional
	@Override
	public BaseResponse<Map<String, Object>> checkCountFast(CheckCountFastQuery query) {
		if (query == null) {
			return BaseResponse.errorMsg("参数错误");
		}
		LoginUser loginUser = userService.getLoginUser();
		if (loginUser == null) {
			return BaseResponse.noLogin("请登录用户");
		}
		if (query.getUserId() == null) {
			return BaseResponse.errorMsg("检测人信息错误");
		}

		//通过检测组获取检测项信息
		List<String> checkResutlList = teamMapper.getByGroup(query.getTeamGroupId());
		//该检测项的每个样品的检测计算结果集合
		if (query.getIntputResult() == null) {
			return BaseResponse.errorMsg("请输入检测数据");
		}
		JSONArray inputResult_jsonArray = commonService.mapToJSONArray2(query.getUserId(), query.getTeamGroupId(), query.getIntputResult());

		//计算检测结果
		Map<String, String> resultMap = commonService.checkCount(checkResutlList, query.getIntputResult());

		//如果存在工业分析检测组的检测项，需要把 Mad1 Mad2 Aad1 Aad2 Vad1 Vad2 的值展示出来
		if (inputResult_jsonArray != null) {
			for (int i = 0; i < inputResult_jsonArray.size(); i++) {
				JSONObject jsonObjectInput = inputResult_jsonArray.getJSONObject(i);
				if (jsonObjectInput.get("name").equals("Mad1")) {
					jsonObjectInput.put("value", resultMap.get("Mad1"));
				}
				if (jsonObjectInput.get("name").equals("Mad2")) {
					jsonObjectInput.put("value", resultMap.get("Mad2"));
				}
				if (jsonObjectInput.get("name").equals("Aad1")) {
					jsonObjectInput.put("value", resultMap.get("Aad1"));
				}
				if (jsonObjectInput.get("name").equals("Aad2")) {
					jsonObjectInput.put("value", resultMap.get("Aad2"));
				}
				if (jsonObjectInput.get("name").equals("Vad1")) {
					jsonObjectInput.put("value", resultMap.get("Vad1"));
				}
				if (jsonObjectInput.get("name").equals("Vad2")) {
					jsonObjectInput.put("value", resultMap.get("Vad2"));
				}
			}

		}
		String inputResult_String = JSON.toJSON(inputResult_jsonArray).toString();
		System.out.println(inputResult_String);
		Map<String, Object> map = new HashMap<>();
		map.put("inputResult", inputResult_String);
		map.put("resultMap", resultMap);

		return BaseResponse.okData(map);
	}

	/**
	 * 按照检测组校核计算 （新的）
	 *
	 * @param query
	 * @return
	 */
	@Override
	@Transactional
	public BaseResponse<String> checkCountNew(CheckCountNewQuery query) {
		LoginUser loginUser = userService.getLoginUser();
		if (loginUser == null) {
			return BaseResponse.noLogin("请登录用户");
		}
		if (query == null || query.getEntrustId() == null || query.getUserId() == null ||
				query.getSmapleId() == null) {
			return BaseResponse.errorMsg("参数错误");
		}
		if (query.getTeamGroupId() == null) {
			return BaseResponse.errorMsg("请选择检测组");
		}
		if (query.getIsAgree() == null ||
				(query.getIsAgree() != 0 && query.getIsAgree() != 1 && query.getIsAgree() != 2)) {
			return BaseResponse.errorMsg("请选择正确按钮");
		}
		if (query.getIsAgree() == 2) {
			if (StringUtils.isEmpty(query.getAuditReason())) {
				return BaseResponse.errorMsg("请填写退回原因");
			}
			if (query.getAuditReason().length() > 100) {
				return BaseResponse.errorMsg("退回原因内容过长");
			}
		}

		Entrust entrust = entrustMapper.selectById(query.getEntrustId());
		if (entrust == null) {
			return BaseResponse.errorMsg("信息错误");
		}
		Sample sample_this = sampleMapper.selectById(query.getSmapleId());
		if (sample_this == null) {
			return BaseResponse.errorMsg("样品信息错误");
		}

		entrust.setStatus(7); //校核中状态
		//保存校核输入表信息
		QueryWrapper<SampleDistributionCheckinput> checkinputQueryWrapper = new QueryWrapper<>();
		checkinputQueryWrapper.eq("entrust_id", query.getEntrustId());
		checkinputQueryWrapper.eq("sample_id", query.getSmapleId());
		checkinputQueryWrapper.eq("team_group_id", query.getTeamGroupId());
		checkinputQueryWrapper.eq("user_id", query.getUserId());
		List<SampleDistributionCheckinput> checkinputList =
				sampleDistributionCheckinputMapper.selectList(checkinputQueryWrapper);
		SampleDistributionCheckinput checkinput = null;
		JSONArray inputResult_jsonArray = commonService.mapToJSONArray2(query.getUserId(), query.getTeamGroupId(), query.getIntputResult());

		String inputResult_String = JSON.toJSON(inputResult_jsonArray).toString();
		System.out.println(inputResult_String);

		if (checkinputList != null && checkinputList.size() > 0) {
			checkinput = checkinputList.get(0);
			checkinput.setCheckUserId(loginUser.getId())
					.setInputResult(inputResult_String)
					.setStatus(query.getIsAgree() == 1 ? 4 : query.getIsAgree() == 0 ? 2 : 5)
					.setUpdateTime(LocalDateTime.now())
					.setRemark(query.getAuditReason());
			sampleDistributionCheckinputMapper.updateById(checkinput);
		} else {
			checkinput = new SampleDistributionCheckinput();
			checkinput.setEntrustId(query.getEntrustId())
					.setSampleId(query.getSmapleId())
					.setTeamGroupId(query.getTeamGroupId())
					.setUserId(query.getUserId())
					.setCheckUserId(loginUser.getId())
					.setInputResult(inputResult_String)
					.setStatus(query.getIsAgree() == 1 ? 4 : query.getIsAgree() == 0 ? 2 : 5)
					.setCreateTime(LocalDateTime.now())
					.setUpdateTime(LocalDateTime.now())
					.setRemark(query.getAuditReason());
			sampleDistributionCheckinputMapper.insert(checkinput);
		}

		//根据按钮判断
		if (query.getIsAgree() == 0) {  //保存
			//保存校核结果信息
			List<CheckCountDistributionNewQuery> queryList = query.getCheckCountDistributionNewQueryList();
			if (queryList != null && queryList.size() > 0) {
				for (CheckCountDistributionNewQuery disQuery : queryList) {
					SampleDistribution dis = distributionMapper.selectById(disQuery.getDistributionId());
					dis.setStatus(2); //待校核
					dis.setCheckInputId(checkinput.getId());
					dis.setDistributionResult(disQuery.getDistributionResult());
					if (dis.getCheckId() != null) {
						SampleDistributionCheck disCheck = sampleDistributionCheckMapper.selectById(dis.getCheckId());
						disCheck.setDistributionError(disQuery.getDistributionError())
								.setLastResult(disQuery.getLastResult())
								.setUpdateTime(LocalDateTime.now());
						sampleDistributionCheckMapper.updateById(disCheck);
						dis.setCheckId(disCheck.getId());
					} else {

						//判断是否有平行样
						List<Sample> otherSampleList = sampleMapper.getOtherSampleList(entrust.getId(), sample_this.getCementCode(), sample_this.getId());
						if (otherSampleList != null && otherSampleList.size() > 0) {
							for (Sample s : otherSampleList) {
								QueryWrapper<SampleDistribution> ortherWrapper = new QueryWrapper<>();
								ortherWrapper.eq("entrust_id", entrust.getId());
								ortherWrapper.eq("sample_id", s.getId());
								ortherWrapper.eq("team_group_id", dis.getTeamGroupId());
								ortherWrapper.eq("team_id", dis.getTeamId());
								List<SampleDistribution> ortherDdistributionList = distributionMapper.selectList(ortherWrapper);
								if (ortherDdistributionList.size() > 1) {
									return BaseResponse.errorMsg("检测派发表检测项信息重复");
								}
								if (ortherDdistributionList != null && ortherDdistributionList.size() > 0) {
									SampleDistribution other = ortherDdistributionList.get(0);
									if (other.getCheckId() != null) {
										dis.setCheckId(other.getCheckId());
									}
								}

							}
							//如果其他的平行样也都没有进行校核，则新增校核内容
							if (dis.getCheckId() != null) {
								SampleDistributionCheck disCheck = sampleDistributionCheckMapper.selectById(dis.getCheckId());
								disCheck.setDistributionError(disQuery.getDistributionError())
										.setLastResult(disQuery.getLastResult())
										.setUpdateTime(LocalDateTime.now());
								sampleDistributionCheckMapper.updateById(disCheck);
							} else {
								SampleDistributionCheck disCheck = new SampleDistributionCheck();
								disCheck.setDistributionError(disQuery.getDistributionError())
										.setLastResult(disQuery.getLastResult())
										.setCreateTime(LocalDateTime.now())
										.setUpdateTime(LocalDateTime.now());
								sampleDistributionCheckMapper.insert(disCheck);
								dis.setCheckId(disCheck.getId());
							}
						} else {
							SampleDistributionCheck disCheck = new SampleDistributionCheck();
							disCheck.setDistributionError(disQuery.getDistributionError())
									.setLastResult(disQuery.getLastResult())
									.setCreateTime(LocalDateTime.now())
									.setUpdateTime(LocalDateTime.now());
							sampleDistributionCheckMapper.insert(disCheck);
							dis.setCheckId(disCheck.getId());
						}
					}
					distributionMapper.updateById(dis);
				}

			}
		} else if (query.getIsAgree() == 1) {  //通过
			//保存校核结果信息
			List<CheckCountDistributionNewQuery> queryList = query.getCheckCountDistributionNewQueryList();
			if (queryList != null && queryList.size() > 0) {
				for (CheckCountDistributionNewQuery disQuery : queryList) {
					SampleDistribution dis = distributionMapper.selectById(disQuery.getDistributionId());
					dis.setStatus(4); //校核通过
					dis.setCheckInputId(checkinput.getId());
					dis.setDistributionResult(disQuery.getDistributionResult());
					if (dis.getCheckId() != null) {
						SampleDistributionCheck disCheck = sampleDistributionCheckMapper.selectById(dis.getCheckId());
						disCheck.setDistributionError(disQuery.getDistributionError())
								.setLastResult(disQuery.getLastResult())
								.setUpdateTime(LocalDateTime.now());
						sampleDistributionCheckMapper.updateById(disCheck);
						dis.setCheckId(disCheck.getId());
					} else {

						//判断是否有平行样
						List<Sample> otherSampleList = sampleMapper.getOtherSampleList(entrust.getId(), sample_this.getCementCode(), sample_this.getId());
						if (otherSampleList != null && otherSampleList.size() > 0) {
							for (Sample s : otherSampleList) {
								QueryWrapper<SampleDistribution> ortherWrapper = new QueryWrapper<>();
								ortherWrapper.eq("entrust_id", entrust.getId());
								ortherWrapper.eq("sample_id", s.getId());
								ortherWrapper.eq("team_group_id", dis.getTeamGroupId());
								ortherWrapper.eq("team_id", dis.getTeamId());
								List<SampleDistribution> ortherDdistributionList = distributionMapper.selectList(ortherWrapper);
								if (ortherDdistributionList.size() > 1) {
									return BaseResponse.errorMsg("检测派发表检测项信息重复");
								}
								if (ortherDdistributionList != null && ortherDdistributionList.size() > 0) {
									SampleDistribution other = ortherDdistributionList.get(0);
									if (other.getCheckId() != null) {
										dis.setCheckId(other.getCheckId());
									}
								}

							}
							//如果其他的平行样也都没有进行校核，则新增校核内容
							if (dis.getCheckId() != null) {
								SampleDistributionCheck disCheck = sampleDistributionCheckMapper.selectById(dis.getCheckId());
								disCheck.setDistributionError(disQuery.getDistributionError())
										.setLastResult(disQuery.getLastResult())
										.setUpdateTime(LocalDateTime.now());
								sampleDistributionCheckMapper.updateById(disCheck);
							} else {
								SampleDistributionCheck disCheck = new SampleDistributionCheck();
								disCheck.setDistributionError(disQuery.getDistributionError())
										.setLastResult(disQuery.getLastResult())
										.setCreateTime(LocalDateTime.now())
										.setUpdateTime(LocalDateTime.now());
								sampleDistributionCheckMapper.insert(disCheck);
								dis.setCheckId(disCheck.getId());
							}
						} else {
							SampleDistributionCheck disCheck = new SampleDistributionCheck();
							disCheck.setDistributionError(disQuery.getDistributionError())
									.setLastResult(disQuery.getLastResult())
									.setCreateTime(LocalDateTime.now())
									.setUpdateTime(LocalDateTime.now());
							sampleDistributionCheckMapper.insert(disCheck);
							dis.setCheckId(disCheck.getId());
						}

					}
					distributionMapper.updateById(dis);
				}

			}

			Boolean ret = true;   //全部校核计算完成
			QueryWrapper<SampleDistribution> disWrapper = new QueryWrapper<>();
			disWrapper.eq("entrust_id", entrust.getId());
			disWrapper.ne("status", 4);
			List<SampleDistribution> disList = distributionMapper.selectList(disWrapper);
			if (disList != null && disList.size() > 0) {
				ret = false; //有一个检测组没有校核，证明校核没有完成
			}

			//如果检测组的校核全部完成
			if (ret) {
				//更新样品表状态为已经校核状态 is_check 0未处理 1处理完成
				QueryWrapper<Sample> sampleQW = new QueryWrapper<>();
				sampleQW.eq("entrust_id", entrust.getId());
				List<Sample> sampleList = sampleMapper.selectList(sampleQW);
				if (sampleList != null && sampleList.size() > 0) {
					for (Sample sample : sampleList) {
						sample.setIsCheck(1);
						sampleMapper.updateById(sample);
					}
				}

				entrust.setStatus(8);
				//校核完成之后 ，提醒评审人员，和委托人员查看
				List<Integer> approvalIdList = sysApprovalMapper.getApprovalId("委托评审");
				if (approvalIdList == null) {
					return BaseResponse.errorMsg("委托评审信息错误");
				}
				String userIds = "";
				SysUser createUser = userService.getById(entrust.getUserId());
				if (createUser != null && createUser.getWxId() != null) {
					userIds = createUser.getWxId();
				}
				for (Integer approvalId : approvalIdList) {
					SysUser approvalUser = userService.getById(approvalId);
					if (approvalUser == null) {
						return BaseResponse.errorMsg("委托评审信息错误");
					}
					BaseResponse wrapper = userMessageService.sendMessage(approvalId, "您有一条委托单校核完成", entrust.getId(), SysUserMessage.MessageType.ENTRUST);
					if (wrapper.getCode() != 200) {
						return wrapper;
					}
					if (StringUtils.isNotBlank(approvalUser.getWxId())) {
						userIds = userIds.equals("") ? approvalUser.getWxId() : userIds + "|" + approvalUser.getWxId();
					}
				}
				//发送企业微信消息内容
				String systemUrL = weiXinService.createMessage(entrust.getId(), "/blanche/recivedetail");
				String content = "你好！" + "委托单号" + entrust.getEntrustCode() + "已校核完成，请前往\n" +
						systemUrL + "进行查看。";
				weiXinService.sendTextMessage(userIds, content);

			}

		} else if (query.getIsAgree() == 2) {   //退回
			//保存校核结果信息
			List<CheckCountDistributionNewQuery> queryList = query.getCheckCountDistributionNewQueryList();
			if (queryList != null && queryList.size() > 0) {
				for (CheckCountDistributionNewQuery disQuery : queryList) {
					SampleDistribution dis = distributionMapper.selectById(disQuery.getDistributionId());
					dis.setStatus(5);//校核退回
					dis.setDistributionResult(null);
					dis.setRemark(query.getAuditReason());
					dis.setCheckInputId(checkinput.getId());
					if (dis.getCheckId() != null) {
						SampleDistributionCheck disCheck = sampleDistributionCheckMapper.selectById(dis.getCheckId());
						disCheck.setDistributionError(null)
								.setLastResult(null)
								.setUpdateTime(LocalDateTime.now());
						sampleDistributionCheckMapper.updateById(disCheck);
						dis.setCheckId(disCheck.getId());
					} else {
						SampleDistributionCheck disCheck = new SampleDistributionCheck();
						disCheck.setDistributionError(null)
								.setLastResult(null)
								.setCreateTime(LocalDateTime.now())
								.setUpdateTime(LocalDateTime.now());
						sampleDistributionCheckMapper.insert(disCheck);
						dis.setCheckId(disCheck.getId());
					}

					distributionMapper.updateById(dis);
				}

				//校核退回 提醒检测人员查看
				String userIds = "";
				SysUser checkUser = userService.getById(checkinput.getUserId());
				if (checkUser != null && checkUser.getWxId() != null) {
					userIds = checkUser.getWxId();
				}
				//发送企业微信消息内容
				String systemUrL = weiXinService.createMessage(entrust.getId(), "/blanche/recivedetail");
				String content = "你好！" + "委托单号" + entrust.getEntrustCode() + "校核退回，请前往\n" +
						systemUrL + "进行查看。";
				weiXinService.sendTextMessage(userIds, content);

			}

		}
		entrustMapper.updateById(entrust);
		if (query.getIsAgree() != 0) {
			logsService.saveLog(SysLogs.ObjType.ENTRUST_LOG, entrust.getId(), "数据校核", null);
			iPrecipriceService.updateStatusByEntrustId(entrust.getId());
		}
		return BaseResponse.okData("校核完成");

	}

	/**
	 * 按照检测组重新校核计算 -不改变当前状态  只改变结果（新的）
	 *
	 * @param query
	 * @return
	 */
	@Override
	@Transactional
	public BaseResponse<String> checkCountNewAgain(CheckCountNewQuery query) {
		LoginUser loginUser = userService.getLoginUser();
		if (loginUser == null) {
			return BaseResponse.noLogin("请登录用户");
		}
		if (query == null || query.getEntrustId() == null || query.getUserId() == null ||
				query.getSmapleId() == null) {
			return BaseResponse.errorMsg("参数错误");
		}
		if (query.getTeamGroupId() == null) {
			return BaseResponse.errorMsg("请选择检测组");
		}
		Entrust entrust = entrustMapper.selectById(query.getEntrustId());
		if (entrust == null) {
			return BaseResponse.errorMsg("信息错误");
		}
		Sample sample_this = sampleMapper.selectById(query.getSmapleId());
		if (sample_this == null) {
			return BaseResponse.errorMsg("样品信息错误");
		}
		//保存校核输入表信息
		QueryWrapper<SampleDistributionCheckinput> checkinputQueryWrapper = new QueryWrapper<>();
		checkinputQueryWrapper.eq("entrust_id", query.getEntrustId());
		checkinputQueryWrapper.eq("sample_id", query.getSmapleId());
		checkinputQueryWrapper.eq("team_group_id", query.getTeamGroupId());
		checkinputQueryWrapper.eq("user_id", query.getUserId());
		List<SampleDistributionCheckinput> checkinputList =
				sampleDistributionCheckinputMapper.selectList(checkinputQueryWrapper);
		SampleDistributionCheckinput checkinput = null;
		JSONArray inputResult_jsonArray = commonService.mapToJSONArray2(query.getUserId(), query.getTeamGroupId(), query.getIntputResult());
		String inputResult_String = JSON.toJSON(inputResult_jsonArray).toString();
		System.out.println(inputResult_String);
		if (checkinputList != null && checkinputList.size() > 0) {
			checkinput = checkinputList.get(0);
			if (checkinput.getStatus() != 4) {
				return BaseResponse.errorMsg("校核状态错误");
			}
			checkinput.setInputResult(inputResult_String)
					.setUpdateTime(LocalDateTime.now());
			sampleDistributionCheckinputMapper.updateById(checkinput);
		} else {
			return BaseResponse.errorMsg("校核输入表信息错误");
		}

		//保存校核结果信息
		List<CheckCountDistributionNewQuery> queryList = query.getCheckCountDistributionNewQueryList();
		if (queryList != null && queryList.size() > 0) {
			for (CheckCountDistributionNewQuery disQuery : queryList) {
				SampleDistribution dis = distributionMapper.selectById(disQuery.getDistributionId());
				dis.setCheckInputId(checkinput.getId());
				dis.setDistributionResult(disQuery.getDistributionResult());
				if (dis.getCheckId() != null) {
					SampleDistributionCheck disCheck = sampleDistributionCheckMapper.selectById(dis.getCheckId());
					disCheck.setDistributionError(disQuery.getDistributionError())
							.setLastResult(disQuery.getLastResult())
							.setUpdateTime(LocalDateTime.now());
					sampleDistributionCheckMapper.updateById(disCheck);
					dis.setCheckId(disCheck.getId());
				} else {
					return BaseResponse.errorMsg("校核结果表信息错误");
				}
				distributionMapper.updateById(dis);
			}
		}

		return BaseResponse.okData("校核完成");

	}

	/**
	 * 数据校核退回任务分页
	 *
	 * @param pageQuery
	 * @param projectCode
	 * @return
	 */
	@Override
	public BaseResponse<IPage<EntrustVo>> getSampleReturnCheckPage(PageQuery pageQuery, String entrustCode, Integer status,
	                                                               String clientName, String projectName, String projectCode,
	                                                               Integer isUrgent, Integer distributionStatus) {
		LoginUser loginUser = userService.getLoginUser();
		if (loginUser == null) {
			return BaseResponse.noLogin("请登录账号");
		}
		Map<String, Object> params = new HashMap<>();
		params.put("entrustCode", entrustCode);
		params.put("status", status);
		params.put("clientName", clientName);
		params.put("projectName", projectName);
		params.put("projectCode", projectCode);
		params.put("isUrgent", isUrgent);
		params.put("userId", loginUser.getId());
		params.put("distributionStatus", distributionStatus);
		Page<EntrustVo> page = new Page<>(pageQuery.getPageNo(), pageQuery.getPageSize());
		IPage<EntrustVo> pages = entrustMapper.getSampleReturnCheckPage(page, params);
		List<EntrustVo> list = pages.getRecords();
		if (list != null && list.size() > 0) {
			for (EntrustVo entrustVo : list) {
				List<Sample> sampleList = sampleMapper.getSampleList(entrustVo.getId());
				//检测项目名称
				String sampleNames = "";
				if (sampleList != null && sampleList.size() > 0) {
					//样品名称 列表中的拼接
					for (Sample sample : sampleList) {
						sampleNames = sampleNames.equals("") ? sample.getName() : (sampleNames + "、" + sample.getName());
					}
					//检测项 和检测依据 列表里只显示第一个样品的
					Sample sample = sampleList.get(0);
					String teamIds = sample.getTeamIds();
					String checkTeam = "";
					if (teamIds != null) {
						String[] teamIdS = teamIds.split("、");
						for (String teamId : teamIdS) {
							Team team = teamMapper.selectById(Integer.valueOf(teamId));
							if (team != null) {
								checkTeam = checkTeam.equals("") ? team.getName() : (checkTeam + "、" + team.getName());
							}
						}
					}
					entrustVo.setCheckTeam(checkTeam);
					entrustVo.setCheckMethodNumber(sample.getMethodNumbers());
				}
				//列表中的样品数量只展示关联到检测人的样品数量
				List<Sample> sampleCheckList = sampleMapper.getSampleCheckList(entrustVo.getId());
				List<SampleVo> sampleVoList = new ArrayList<>();
				if (sampleCheckList != null && sampleCheckList.size() > 0) {
					for (Sample sample : sampleCheckList) {
						SampleVo sampleVo = new SampleVo();
						BeanUtils.copyProperties(sample, sampleVo);
						List<SampleDistributionTeamVo> sampleDistributionTeamVoList =
								distributionMapper.getDistributionTeamList(sample.getId(), loginUser.getId());
						if (sampleDistributionTeamVoList != null && sampleDistributionTeamVoList.size() > 0) {
							sampleVoList.add(sampleVo);
						}
					}
				}

				if (sampleVoList != null) {
					entrustVo.setSampleNum(sampleVoList.size());
				} else {
					entrustVo.setSampleNum(0);
				}

				entrustVo.setSampleNames(sampleNames);
			}
		}
		pages.setRecords(list);
		return BaseResponse.okData(pages);
	}

	/**
	 * 样品检测项校核详情  （之前的方式）
	 *
	 * @param cementCode 本所编号
	 * @return
	 */
	public BaseResponse<List<SampleCheckVo>> getCheckDetail(String cementCode) {
		LoginUser loginUser = userService.getLoginUser();
		if (loginUser == null) {
			return BaseResponse.noLogin("请登录账号");
		}
		if (StringUtils.isEmpty(cementCode)) {
			return BaseResponse.errorMsg("参数错误");
		}

		QueryWrapper<Sample> sampleQW = new QueryWrapper<>();
		sampleQW.eq("cement_code", cementCode);
		List<Sample> sampleList = sampleMapper.selectList(sampleQW);

		List<SampleCheckVo> sampleCheckVoList = new ArrayList<>();
		if (sampleList != null && sampleList.size() > 0) {
			Sample sampleOne = sampleList.get(0);
			if (sampleOne == null) {
				return BaseResponse.errorMsg("样品信息错误");
			}
			Entrust entrust = entrustMapper.selectById(sampleOne.getEntrustId());
			if (entrust == null) {
				return BaseResponse.errorMsg("委托单信息错误");
			}
			//根据样品 和派发任务表 找到都有哪些检测组
			List<SampleCheckGroupVo> sampleCheckGroupVoList = distributionMapper.getSampleCheckGroupList(sampleOne.getId());
			if (sampleCheckGroupVoList != null && sampleCheckGroupVoList.size() > 0) {
				for (SampleCheckGroupVo sampleCheckGroupVo : sampleCheckGroupVoList) {
					QueryWrapper<SampleCheck> sampleCheckQW = new QueryWrapper<>();
					sampleCheckQW.eq("entrust_id", entrust.getId());
					sampleCheckQW.eq("cement_code", cementCode);
					sampleCheckQW.eq("team_group_id", sampleCheckGroupVo.getTeamGroupId());
					List<SampleCheck> sampleCheckList = sampleCheckMapper.selectList(sampleCheckQW);

					if (sampleCheckList != null && sampleCheckList.size() > 0) {   //参与校核计算的
						SampleCheck sampleCheck = sampleCheckList.get(0);
						SampleCheckVo sampleCheckVo = new SampleCheckVo();
						sampleCheckVo.setId(sampleCheck.getId());
						sampleCheckVo.setEntrustId(entrust.getId());
						sampleCheckVo.setSampleName(sampleOne.getName());
						sampleCheckVo.setCementCode(cementCode);
						sampleCheckVo.setTeamGroupId(sampleCheck.getTeamGroupId());
						sampleCheckVo.setTeamGroupName(sampleCheck.getTeamGroupName());
						sampleCheckVo.setCountResults(sampleCheck.getCountResults());
						sampleCheckVo.setCountError(sampleCheck.getCountError());
						sampleCheckVo.setIsParallel(sampleOne.getIsParallel());
						QueryWrapper<SampleCheckTeam> sampleCheckTeamQW = new QueryWrapper<>();
						sampleCheckTeamQW.eq("check_id", sampleCheck.getId());
						List<SampleCheckTeam> sampleCheckTeamList = sampleCheckTeamMapper.selectList(sampleCheckTeamQW);
						List<SampleCheckTeamVo> sctVoList = new ArrayList<>();
						if (sampleCheckTeamList != null && sampleCheckTeamList.size() > 0) {
							for (SampleCheckTeam sampleCheckTeam : sampleCheckTeamList) {
								SampleCheckTeamVo sampleCheckTeamVo = new SampleCheckTeamVo();
								sampleCheckTeamVo.setId(sampleCheckTeam.getId());
								sampleCheckTeamVo.setCheckId(sampleCheck.getId());
								sampleCheckTeamVo.setSampleId(sampleCheckTeam.getSampleId());
								sampleCheckTeamVo.setSampleId(sampleCheckTeam.getSampleId());
								//本所编号 展示的是平行样编号
								sampleCheckTeamVo.setParallelCode(sampleCheckTeam.getParallelCode());
								sampleCheckTeamVo.setUserId(sampleCheckTeam.getUserId());
								SysUser sysUser = userService.getById(sampleCheckTeam.getUserId());
								sampleCheckTeamVo.setUserName(sysUser.getName());
								sampleCheckTeamVo.setEquipmentId(sampleCheckTeam.getEquipmentId());
								sampleCheckTeamVo.setEquipmentName(sampleCheckTeam.getEquipmentName());
								sampleCheckTeamVo.setInputResult(sampleCheckTeam.getInputResult());
								sampleCheckTeamVo.setCountResult(sampleCheckTeam.getCountResult());
								sctVoList.add(sampleCheckTeamVo);
							}
						}
						sampleCheckVo.setSampleCheckGroupVoList(sctVoList);
						sampleCheckVoList.add(sampleCheckVo);
					} else {         //未参与校核计算的
						SampleCheckVo sampleCheckVo = new SampleCheckVo();
						sampleCheckVo.setEntrustId(entrust.getId());
						sampleCheckVo.setSampleName(sampleOne.getName());
						sampleCheckVo.setCementCode(cementCode);
						sampleCheckVo.setTeamGroupId(sampleCheckGroupVo.getTeamGroupId());
						sampleCheckVo.setTeamGroupName(sampleCheckGroupVo.getTeamGroupName());
						TeamGroup teamGroup = teamGroupMapper.selectById(sampleCheckGroupVo.getTeamGroupId());
						if (teamGroup == null) {
							return BaseResponse.errorMsg("检测组信息错误");
						}
						//根据检测组id返回检测组对应的所有检测项信息
//                        List<String> checkResutlList = teamMapper.getByGroup(teamGroup.getId());
						List<Map<String, String>> checkResutlList2 = teamMapper.getListByGroup(teamGroup.getId());
						JSONArray jsonArray = new JSONArray();
						for (Map<String, String> cr : checkResutlList2) {
							jsonArray.add(cr);
						}
						String checkResut_String = JSON.toJSON(jsonArray).toString();
						System.out.println(checkResut_String);
						sampleCheckVo.setCountResults(checkResut_String);
						sampleCheckVo.setIsParallel(sampleOne.getIsParallel());
						List<SampleCheckTeamVo> sctVoList = new ArrayList<>();
						for (Sample sample : sampleList) {
							SampleCheckTeamVo sampleCheckTeamVo = new SampleCheckTeamVo();
							sampleCheckTeamVo.setCheckId(null);
							sampleCheckTeamVo.setSampleId(sample.getId());
							//本所编号 展示的是平行样编号
							sampleCheckTeamVo.setParallelCode(sample.getParallelCode());
							QueryWrapper<SampleDistribution> sdWrapper = new QueryWrapper<>();
							sdWrapper.eq("sample_id", sample.getId());
							sdWrapper.eq("team_group_id", sampleCheckGroupVo.getTeamGroupId());
							List<SampleDistribution> sdList = distributionMapper.selectList(sdWrapper);
							SampleDistribution sampleDistribution = sdList.get(0);
							SysUser sysUser = userService.getById(sampleDistribution.getUserId());
							sampleCheckTeamVo.setUserId(sampleDistribution.getUserId());
							sampleCheckTeamVo.setUserName(sysUser.getName());
							sampleCheckTeamVo.setInputResult(teamGroup.getCheckElement());
							sctVoList.add(sampleCheckTeamVo);
						}
						sampleCheckVo.setSampleCheckGroupVoList(sctVoList);
						sampleCheckVoList.add(sampleCheckVo);
					}
				}
			}
		}
		return BaseResponse.okData(sampleCheckVoList);
	}

	//附件****************************************************************

	/**
	 * 样品处理附件列表
	 *
	 * @param sampleHandleId 样品处理表id
	 * @return
	 */
	public BaseResponse<List<SampleHandleEnclosureVo>> getSampleHandleEnclosureList(Integer sampleHandleId) {
		LoginUser loginUser = userService.getLoginUser();
		if (loginUser == null) {
			return BaseResponse.noLogin("请登录账号");
		}
		if (sampleHandleId == null) {
			return BaseResponse.errorMsg("参数错误");
		}
		Map<String, Object> params = new HashMap<>();
		params.put("sampleHandleId", sampleHandleId);
		List<SampleHandleEnclosureVo> list = sampleHandleEnclosureMapper.getList(params);
		return BaseResponse.okData(list);
	}

	/**
	 * 样品检测附件列表
	 *
	 * @param sampleDistributionId 样品检测表id
	 * @return
	 */
	@Override
	public BaseResponse<List<SampleDistributionEnclosureVo>> getSampleDistributionEnclosureList(Integer sampleDistributionId) {
		LoginUser loginUser = userService.getLoginUser();
		if (loginUser == null) {
			return BaseResponse.noLogin("请登录账号");
		}
		Map<String, Object> params = new HashMap<>();
		params.put("sampleDistributionId", sampleDistributionId);
		List<SampleDistributionEnclosureVo> list = sampleDistributionEnclosureMapper.getList(params);
		return BaseResponse.okData(list);
	}

	/**
	 * 检测结果查看附件 （平行样的样品附件 也都展示在一个检测项下面里面）
	 *
	 * @param sampleDistributionId
	 * @return
	 */
	@Override
	public BaseResponse<List<SampleDistributionEnclosureVo>> getClientEnclosureList(Integer sampleDistributionId) {
		LoginUser loginUser = userService.getLoginUser();
		if (loginUser == null) {
			return BaseResponse.noLogin("请登录账号");
		}
		if (sampleDistributionId == null) {
			return BaseResponse.errorMsg("参数错误");
		}
		Map<String, Object> params = new HashMap<>();
		params.put("sampleDistributionId", sampleDistributionId);
		params.put("isPublic", 1);
		List<SampleDistributionEnclosureVo> list = sampleDistributionEnclosureMapper.getList(params);

		//如果有平行样
		SampleDistribution sampleDistribution = distributionMapper.selectById(sampleDistributionId);
		Sample sample = sampleMapper.selectById(sampleDistribution.getSampleId());
		List<Sample> otherSampleList = sampleMapper.getOtherSampleList(sampleDistribution.getEntrustId(), sample.getCementCode(), sample.getId());
		if (otherSampleList != null && otherSampleList.size() > 0) {
			for (Sample s : otherSampleList) {
				QueryWrapper<SampleDistribution> sdWrapper = new QueryWrapper<>();
				sdWrapper.eq("entrust_id", sampleDistribution.getEntrustId());
				sdWrapper.eq("sample_id", s.getId());
				sdWrapper.eq("team_group_id", sampleDistribution.getTeamGroupId());
				sdWrapper.eq("team_id", sampleDistribution.getTeamId());
				List<SampleDistribution> sdList = distributionMapper.selectList(sdWrapper);
				if (sdList != null && sdList.size() > 0) {
					for (SampleDistribution sd : sdList) {
						Map<String, Object> params2 = new HashMap<>();
						params2.put("sampleDistributionId", sd.getId());
						params.put("isPublic", 1);
						List<SampleDistributionEnclosureVo> sdevList = sampleDistributionEnclosureMapper.getList(params2);
						if (sdevList != null && sdevList.size() > 0) {
							for (SampleDistributionEnclosureVo sdev : sdevList) {
								list.add(sdev);
							}
						}
					}
				}
			}
		}
		return BaseResponse.okData(list);
	}

	/**
	 * 样品处理上传附件
	 *
	 * @param query
	 * @return
	 */
	@Transactional
	@Override
	public BaseResponse<SampleHandleEnclosure> uploadEnclosureCL(SampleHandleEnclosureQuery query) {
		LoginUser loginUser = userService.getLoginUser();
		if (loginUser == null) {
			return BaseResponse.noLogin("请登录账号");
		}
		if (query == null || query.getId() == null) {
			return BaseResponse.errorMsg("参数错误");
		}
		SampleHandle sampleHandle = sampleHandleMapper.selectById(query.getId());
		SampleHandleEnclosure sampleHandleEnclosure = new SampleHandleEnclosure();
		sampleHandleEnclosure.setSampleHandleId(sampleHandle.getId())
				.setHandleId(sampleHandle.getHandleId())
				.setUserId(sampleHandle.getUserId())
				.setCreateTime(LocalDateTime.now())
				.setStatus(1)
				.setAlias(query.getAlias())
				.setEnclosureUrl(query.getEnclosureUrl())
				.setExtName(query.getExtName())
				.setUpdateTime(LocalDateTime.now())
				.setPdfUrl(query.getPdfUrl())
				.setIsPublic(0); ////默认不允许公开
		if (StringUtils.isEmpty(query.getEnclosureUrl())) {
			sampleHandleEnclosure.setStatus(2); //上传失败
		}
		sampleHandleEnclosureMapper.insert(sampleHandleEnclosure);
		return BaseResponse.okData(sampleHandleEnclosure);
	}

	/**
	 * 样品检测上传附件
	 *
	 * @param query
	 * @return
	 */
	@Transactional
	@Override
	public BaseResponse<SampleDistributionEnclosure> uploadEnclosurePF(SampleHandleEnclosureQuery query) {
		LoginUser loginUser = userService.getLoginUser();
		if (loginUser == null) {
			return BaseResponse.noLogin("请登录账号");
		}
		if (query == null || query.getId() == null) {
			return BaseResponse.errorMsg("参数错误");
		}
		SampleDistribution distribution = distributionMapper.selectById(query.getId());
		if (distribution == null) {
			return BaseResponse.errorMsg("信息错误");
		}

		SampleDistributionEnclosure sampleDistributionEnclosure = new SampleDistributionEnclosure();
		sampleDistributionEnclosure.setSampleDistributionId(distribution.getId())
				.setTeamId(distribution.getTeamId())
				.setUserId(distribution.getUserId())
				.setCreateTime(LocalDateTime.now())
				.setStatus(1)
				.setAlias(query.getAlias())
				.setEnclosureUrl(query.getEnclosureUrl())
				.setExtName(query.getExtName())
				.setUpdateTime(LocalDateTime.now())
				.setPdfUrl(query.getPdfUrl())
				.setIsPublic(0); //默认不允许公开
		if (StringUtils.isEmpty(query.getEnclosureUrl())) {
			sampleDistributionEnclosure.setStatus(2); //上传失败
		}
		sampleDistributionEnclosureMapper.insert(sampleDistributionEnclosure);
		return BaseResponse.okData(sampleDistributionEnclosure);
	}

	/**
	 * 删除附件
	 *
	 * @param id   附件表id
	 * @param type 0处理项，1检测项
	 * @return
	 */
	@Transactional
	@Override
	public BaseResponse<String> deleteEnclosure(Integer id, Integer type) {
		if (type == 0) {
			SampleHandleEnclosure sampleHandleEnclosure =
					sampleHandleEnclosureMapper.selectById(id);
			if (sampleHandleEnclosure == null) {
				return BaseResponse.errorMsg("信息错误");
			}
			sampleHandleEnclosureMapper.deleteById(id);
			return BaseResponse.okData("附件信息删除成功");
		} else if (type == 1) {
			SampleDistributionEnclosure sampleDistributionEnclosure =
					sampleDistributionEnclosureMapper.selectById(id);

			if (sampleDistributionEnclosure == null) {
				return BaseResponse.errorMsg("信息错误");
			}
			sampleDistributionEnclosureMapper.deleteById(id);
			return BaseResponse.okData("附件信息删除成功");
		} else {
			return BaseResponse.errorMsg("请选择类型");
		}
	}

	/**
	 * 设置附件是否公开
	 *
	 * @param isPublic
	 * @param id
	 * @param type     0处理项，1检测项
	 * @return
	 */
	@Override
	@Transactional
	public BaseResponse<String> makePublicEnclosure(Integer isPublic, Integer id, Integer type) {
		if (isPublic == null || id == null) {
			return BaseResponse.errorMsg("参数错误!");
		}
		if (type == 0) {
			SampleHandleEnclosure sampleHandleEnclosure =
					sampleHandleEnclosureMapper.selectById(id);
			if (sampleHandleEnclosure == null) {
				return BaseResponse.errorMsg("信息错误");
			}
			sampleHandleEnclosure.setIsPublic(isPublic);
			sampleHandleEnclosureMapper.updateById(sampleHandleEnclosure);
			String result = "";
			if (sampleHandleEnclosure.getIsPublic() == 1) {
				result = "已公开";
			} else {
				result = "已禁止";
			}
			return BaseResponse.okData(result);
		} else if (type == 1) {
			SampleDistributionEnclosure sampleDistributionEnclosure =
					sampleDistributionEnclosureMapper.selectById(id);
			if (sampleDistributionEnclosure == null) {
				return BaseResponse.errorMsg("信息错误");
			}
			sampleDistributionEnclosure.setIsPublic(isPublic);
			sampleDistributionEnclosureMapper.updateById(sampleDistributionEnclosure);
			String result = "";
			if (sampleDistributionEnclosure.getIsPublic() == 1) {
				result = "已公开";
			} else {
				result = "已禁止";
			}
			return BaseResponse.okData(result);
		} else {
			return BaseResponse.errorMsg("请选择类型");
		}
	}

	/**
	 * 校核计算 （之前的方式）
	 *
	 * @param query
	 * @return
	 */
	@Transactional
	public BaseResponse<Map<String, Object>> checkCount(CheckCountQuery query) {
		if (query == null) {
			return BaseResponse.errorMsg("参数错误");
		}
		Entrust entrust = entrustMapper.selectById(query.getEntrustId());
		if (entrust == null) {
			return BaseResponse.errorMsg("信息错误");
		}
		if (entrust.getStatus() < 6) {
			return BaseResponse.errorMsg("当前状态不允许校核");
		}
		entrust.setStatus(7); //校核中状态
		LoginUser loginUser = userService.getLoginUser();
		if (loginUser == null) {
			return BaseResponse.noLogin("请登录用户");
		}

		if (query.getCementCode() == null) {
			return BaseResponse.errorMsg("本所编号不能为空");
		}
		if (query.getTeamGroupId() == null || StringUtils.isEmpty(query.getTeamGroupName())) {
			return BaseResponse.errorMsg("请选择检测组");
		}
		SampleCheck check = null;
		//保存校核表信息
		if (query.getId() == null) {
			check = new SampleCheck();
			check.setEntrustId(entrust.getId())
					.setCementCode(query.getCementCode())
					.setTeamGroupId(query.getTeamGroupId())
					.setTeamGroupName(query.getTeamGroupName())
					.setCountResults(null)
					.setCountError(null)
					.setIsParallel(query.getIsParallel())
					.setCreateTime(LocalDateTime.now());
			sampleCheckMapper.insert(check);
		} else {
			check = sampleCheckMapper.selectById(query.getId());
		}
		//返回信息：样品的计算结果集合
		Map<String, Object> countResultMap = new HashMap<>();
		if (query.getSampleList() != null && query.getSampleList().size() > 0) {
			//通过检测组获取检测项信息
			List<String> checkResutlList = teamMapper.getByGroup(check.getTeamGroupId());
			//该检测项的每个样品的检测计算结果集合
			List<Map<String, String>> resultMapList = new ArrayList<>();
			for (CheckCountSampleQuery sampleQuery : query.getSampleList()) {
				if (sampleQuery.getSmapleId() == null) {
					return BaseResponse.errorMsg("样品信息错误");
				}
				if (StringUtils.isEmpty(sampleQuery.getParallelCode())) {
					return BaseResponse.errorMsg("请传入平行样编号");
				}
				if (sampleQuery.getUserId() == null) {
					return BaseResponse.errorMsg("请传入检测人id");
				}
				if (sampleQuery.getEquipmentId() == null || StringUtils.isEmpty(sampleQuery.getEquipmentName())) {
					return BaseResponse.errorMsg("请传选择检测设备");
				}
				if (sampleQuery.getIntputResult() == null) {
					return BaseResponse.errorMsg("请输入检测数据");
				}
				JSONArray inputResult_jsonArray = commonService.mapToJSONArray2(sampleQuery.getUserId(), check.getTeamGroupId(), sampleQuery.getIntputResult());

				String inputResult_String = JSON.toJSON(inputResult_jsonArray).toString();
				System.out.println(inputResult_String);
				//计算检测结果
				Map<String, String> resultMap = commonService.checkCount(checkResutlList, sampleQuery.getIntputResult());
				resultMapList.add(resultMap);
				JSONArray resultMap_jsonArray = commonService.mapToJSONArray(resultMap);
				String resultMap_String = JSON.toJSON(resultMap_jsonArray).toString();
				System.out.println(resultMap_String);
				//保存校核检测组检测项信息
				SampleCheckTeam sampleCheckTeam = null;
				if (sampleQuery.getId() == null) {
					sampleCheckTeam = new SampleCheckTeam();
					sampleCheckTeam.setCheckId(check.getId())
							.setSampleId(sampleQuery.getSmapleId())
							.setParallelCode(sampleQuery.getParallelCode())
							.setUserId(sampleQuery.getUserId())
							.setEquipmentId(sampleQuery.getEquipmentId())
							.setEquipmentName(sampleQuery.getEquipmentName())
							.setInputResult(inputResult_String)
							.setCountResult(resultMap_String)
							.setCreateTime(LocalDateTime.now());
					sampleCheckTeamMapper.insert(sampleCheckTeam);
					//添加设备使用记录  设备使用记录表 equipment_use
					EquipmentUse equipmentUse2 = new EquipmentUse();
					SysUser sysUser = userService.getById(sampleCheckTeam.getUserId());
					equipmentUse2.setEquipmentId(sampleCheckTeam.getEquipmentId())
							.setUserId(sampleCheckTeam.getUserId())
							.setUserName(sysUser.getName())
							.setUseDate(LocalDate.now())
							.setProjectId(entrust.getProjectId())
							.setProjectName(entrust.getProjectName())
							.setProjectCode(entrust.getProjectCode())
							.setTestItem(check.getTeamGroupName())
							.setCreateTime(LocalDateTime.now())
							.setRemark("委托派发-数据校核");
					equipmentUseMapper.insert(equipmentUse2);
				} else {
					sampleCheckTeam = sampleCheckTeamMapper.selectById(sampleQuery.getId());
					sampleCheckTeam.setCheckId(check.getId())
							.setInputResult(inputResult_String)
							.setCountResult(resultMap_String);
					sampleCheckTeamMapper.updateById(sampleCheckTeam);
				}
			}
			countResultMap.put("resultMapList", resultMapList);
			//计算最终计算结果  和 结果误差值
			Map<String, String> groupMap = new HashMap<>();   //"L.O.I":"1,2,3,4..."
			if (resultMapList != null && resultMapList.size() > 0) {
				for (String name : checkResutlList) {
					if (name.equals("L.O.I")) {
						for (Map<String, String> map : resultMapList) {
							groupMap.put(name, (groupMap == null || groupMap.size() == 0 ||
									groupMap.get(name) == null || groupMap.get(name).equals(""))
									? map.get(name) : groupMap.get(name) + "," + map.get(name));
						}
					} else if (name.equals("SiO<sub>2</sub>")) {
						for (Map<String, String> map : resultMapList) {
							groupMap.put(name, (groupMap == null || groupMap.size() == 0 ||
									groupMap.get(name) == null || groupMap.get(name).equals("")) ?
									map.get(name) : groupMap.get(name) + "," + map.get(name));
						}
					} else if (name.equals("Al<sub>2</sub>O<sub>3</sub>+TiO<sub>2</sub>")) {
						for (Map<String, String> map : resultMapList) {
							groupMap.put(name, (groupMap == null || groupMap.size() == 0 ||
									groupMap.get(name) == null || groupMap.get(name).equals(""))
									? map.get(name) : groupMap.get(name) + "," + map.get(name));
						}
					} else if (name.equals("Al<sub>2</sub>O<sub>3</sub>")) {
						for (Map<String, String> map : resultMapList) {
							groupMap.put(name, (groupMap == null || groupMap.size() == 0 ||
									groupMap.get(name) == null || groupMap.get(name).equals(""))
									? map.get(name) : groupMap.get(name) + "," + map.get(name));
						}
					} else if (name.equals("Fe<sub>2</sub>O<sub>3</sub>")) {
						for (Map<String, String> map : resultMapList) {
							groupMap.put(name, (groupMap == null || groupMap.size() == 0 ||
									groupMap.get(name) == null || groupMap.get(name).equals(""))
									? map.get(name) : groupMap.get(name) + "," + map.get(name));
						}
					} else if (name.equals("CaO")) {
						for (Map<String, String> map : resultMapList) {
							groupMap.put(name, (groupMap == null || groupMap.size() == 0 ||
									groupMap.get(name) == null || groupMap.get(name).equals(""))
									? map.get(name) : groupMap.get(name) + "," + map.get(name));
						}
					} else if (name.equals("MgO")) {
						for (Map<String, String> map : resultMapList) {
							groupMap.put(name, (groupMap == null || groupMap.size() == 0 ||
									groupMap.get(name) == null || groupMap.get(name).equals(""))
									? map.get(name) : groupMap.get(name) + "," + map.get(name));
						}
					} else if (name.equals("TiO<sub>2</sub>")) {
						for (Map<String, String> map : resultMapList) {
							groupMap.put(name, (groupMap == null || groupMap.size() == 0 ||
									groupMap.get(name) == null || groupMap.get(name).equals(""))
									? map.get(name) : groupMap.get(name) + "," + map.get(name));
						}
					} else if (name.equals("K<sub>2</sub>O")) {
						for (Map<String, String> map : resultMapList) {
							groupMap.put(name, (groupMap == null || groupMap.size() == 0 ||
									groupMap.get(name) == null || groupMap.get(name).equals(""))
									? map.get(name) : groupMap.get(name) + "," + map.get(name));
						}
					} else if (name.equals("Na<sub>2</sub>O")) {
						for (Map<String, String> map : resultMapList) {
							groupMap.put(name, (groupMap == null || groupMap.size() == 0 ||
									groupMap.get(name) == null || groupMap.get(name).equals(""))
									? map.get(name) : groupMap.get(name) + "," + map.get(name));
						}
					} else if (name.equals("SO<sub>3</sub>")) {
						for (Map<String, String> map : resultMapList) {
							groupMap.put(name, (groupMap == null || groupMap.size() == 0 ||
									groupMap.get(name) == null || groupMap.get(name).equals(""))
									? map.get(name) : groupMap.get(name) + "," + map.get(name));
						}
					} else if (name.equals("Cl")) {
						for (Map<String, String> map : resultMapList) {
							groupMap.put(name, (groupMap == null || groupMap.size() == 0 ||
									groupMap.get(name) == null || groupMap.get(name).equals(""))
									? map.get(name) : groupMap.get(name) + "," + map.get(name));
						}
					} else if (name.equals("合量")) {
						for (Map<String, String> map : resultMapList) {
							groupMap.put(name, (groupMap == null || groupMap.size() == 0 ||
									groupMap.get(name) == null || groupMap.get(name).equals(""))
									? map.get(name) : groupMap.get(name) + "," + map.get(name));
						}
					} else if (name.equals("KH")) {
						for (Map<String, String> map : resultMapList) {
							groupMap.put(name, (groupMap == null || groupMap.size() == 0 ||
									groupMap.get(name) == null || groupMap.get(name).equals(""))
									? map.get(name) : groupMap.get(name) + "," + map.get(name));
						}
					} else if (name.equals("SM")) {
						for (Map<String, String> map : resultMapList) {
							groupMap.put(name, (groupMap == null || groupMap.size() == 0 ||
									groupMap.get(name) == null || groupMap.get(name).equals(""))
									? map.get(name) : groupMap.get(name) + "," + map.get(name));
						}
					} else if (name.equals("AM")) {
						for (Map<String, String> map : resultMapList) {
							groupMap.put(name, (groupMap == null || groupMap.size() == 0 ||
									groupMap.get(name) == null || groupMap.get(name).equals(""))
									? map.get(name) : groupMap.get(name) + "," + map.get(name));
						}
					} else {
						groupMap.put(name, "0");
					}
				}
			}
			//计算平行样的平均值  最大误差值
			Map<String, Map<String, String>> mapResult = CheckCountUtil.countParallelAvgError(groupMap);
			if (mapResult != null) {
				Map<String, String> countResults = mapResult.get("countResults");
				JSONArray countResults_jsonArray = commonService.mapToJSONArray(countResults);
				String countResults_String = JSON.toJSON(countResults_jsonArray).toString();
				System.out.println(countResults_String);
				Map<String, String> countError = mapResult.get("countError");
				JSONArray countError_jsonArray = commonService.mapToJSONArray(countError);
				String countError_String = JSON.toJSON(countError_jsonArray).toString();
				System.out.println(countError_String);
				check.setCountResults(countResults_String);
				check.setCountError(countError_String);
				sampleCheckMapper.updateById(check);
				countResultMap.put("countResults", countResults_String);
				countResultMap.put("countError", countError_String);
			}
		}
		Boolean ret = true; //全部校核计算完成
		List<SampleCheckGroupVo> sampleCheckGroupVoList = distributionMapper.getSampleCheckGroupList(query.getSampleList().get(0).getSmapleId());
		for (SampleCheckGroupVo groupVo : sampleCheckGroupVoList) {
			QueryWrapper<SampleCheck> sampleCheckQW = new QueryWrapper<>();
			sampleCheckQW.eq("entrust_id", entrust.getId());
			sampleCheckQW.eq("cement_code", query.getCementCode());
			sampleCheckQW.eq("team_group_id", groupVo.getTeamGroupId());
			List<SampleCheck> sampleCheckList = sampleCheckMapper.selectList(sampleCheckQW);
			if (sampleCheckList == null || sampleCheckList.size() == 0) {
				ret = false; //有一个检测组没有校核，证明校核没有完成
			}
		}
		//如果检测组的校核全部完成
		if (ret) {
			//更新样品表状态为已经校核状态 is_check 0未处理 1处理完成
			QueryWrapper<Sample> sampleQW = new QueryWrapper<>();
			sampleQW.eq("cement_code", query.getCementCode());
			List<Sample> sampleList = sampleMapper.selectList(sampleQW);
			if (sampleList != null && sampleList.size() > 0) {
				for (Sample sample : sampleList) {
					sample.setIsCheck(1);
					sampleMapper.updateById(sample);
				}
			}
		}

		//如果是保存并完成按钮，检测委托单中其他样品是否都是保存冰完成状态，如果是，委托单为检测完成状态
		List<Sample> sampleNoCheckList = sampleMapper.getNoCheckList(entrust.getId());
		if (sampleNoCheckList == null || sampleNoCheckList.size() == 0) {
			entrust.setStatus(8);
			//校核完成之后 ，提醒评审人员查看
			List<Integer> approvalIdList = sysApprovalMapper.getApprovalId("委托评审");
			if (approvalIdList == null) {
				return BaseResponse.errorMsg("委托评审信息错误");
			}

			String userIds = "";
			SysUser createUser = userService.getById(entrust.getUserId());
			if (createUser != null) {
				userIds = createUser.getWxId();
			}
			for (Integer approvalId : approvalIdList) {
				SysUser approvalUser = userService.getById(approvalId);
				if (approvalUser == null) {
					return BaseResponse.errorMsg("委托评审信息错误");
				}
				BaseResponse wrapper = userMessageService.sendMessage(approvalId, "您有一条委托单校核完成", entrust.getId(), SysUserMessage.MessageType.ENTRUST);
				if (wrapper.getCode() != 200) {
					return wrapper;
				}
				if (StringUtils.isNotBlank(approvalUser.getWxId())) {
					userIds = userIds.equals("") ? approvalUser.getWxId() : userIds + "|" + approvalUser.getWxId();
				}
			}
			//发送企业微信消息内容
			String content = "你好！" + "委托单号" + entrust.getEntrustCode() + "已校核完成，请前往\n" +
					systemUrl + "进行查看。";
			weiXinService.sendTextMessage(userIds, content);

		}
		entrustMapper.updateById(entrust);
		logsService.saveLog(SysLogs.ObjType.ENTRUST_LOG, entrust.getId(), "数据校核", null);

		iPrecipriceService.updateStatusByEntrustId(entrust.getId());
		return BaseResponse.okData(countResultMap);
	}

	//******************************报告************************************************************

	/**
	 * 检测报告分页列表
	 * @param pageQuery
	 * @param startDate
	 * @param endDate
	 * @param clientId
	 * @param projectName
	 * @param projectCode
	 * @return
	 */
	@Override
	public BaseResponse<IPage<EntrustVo>> getReportPage(PageQuery pageQuery, String startDate, String endDate, Integer clientId, String projectName, String projectCode) {

		Map<String, Object> params = new HashMap<>();
		params.put("startDate", startDate);
		params.put("endDate", endDate);
//		params.put("reportStatus", 4);
		params.put("status", 8);
		params.put("clientId", clientId);
		params.put("projectName", projectName);
		params.put("projectCode", projectCode);
		params.put("projectType", "常规项目");
		List<Integer> approvalIdList = sysApprovalMapper.getApprovalId("委托评审");
		if (approvalIdList == null) {
			return BaseResponse.errorMsg("委托评审信息错误");
		}
		Page<EntrustVo> page = new Page<>(pageQuery.getPageNo(), pageQuery.getPageSize());
		IPage<EntrustVo> pages = entrustMapper.getPage(page, params);
		List<EntrustVo> list = pages.getRecords();

		if (list != null && list.size() > 0) {
			for (EntrustVo entrustVo : list) {
				QueryWrapper<SampleTmp> queryWrapper = new QueryWrapper<>();
				queryWrapper.eq("entrust_id", entrustVo.getId());
				List<SampleTmp> sampleTmpList = sampleTmpService.list(queryWrapper);
				//检测项目名称
				String sampleNames = "";
				if (sampleTmpList != null && sampleTmpList.size() > 0) {
					//样品名称 列表中的拼接
					for (SampleTmp sampleTmp : sampleTmpList) {
						sampleNames = "".equals(sampleNames) ? sampleTmp.getName() : (sampleNames + "、" + sampleTmp.getName());
					}
					entrustVo.setSampleNames(sampleNames);
					//检测项 和检测依据 列表里只显示第一个样品的
					SampleTmp sampleTmp = sampleTmpList.get(0);
					String teamIds = sampleTmp.getTeamIds();
					String checkTeam = "";
					if (teamIds != null) {
						String[] teamIdS = teamIds.split("、");
						for (String teamId : teamIdS) {
							Team team = teamMapper.selectById(Integer.valueOf(teamId));
							if (team != null) {
								checkTeam = "".equals(checkTeam) ? team.getName() : (checkTeam + "、" + team.getName());
							}
						}
					}
					entrustVo.setCheckTeam(checkTeam);
					entrustVo.setCheckMethodNumber(sampleTmp.getMethodNumbers());
				}

				//查询报告,如果有上传的委托报告，用户可已进行下载
				QueryWrapper<EntityEnclosure> wrapper = new QueryWrapper<>();
				wrapper.eq("entity_type", EntityEnclosure.EntityType.ENTRUST_REPORT);
				wrapper.eq("entity_id", entrustVo.getId());
				wrapper.eq("is_deleted", 1);
				List<EntityEnclosure> entityEnclosureList = entityEnclosureMapper.selectList(wrapper);
				if (entityEnclosureList != null && entityEnclosureList.size() > 0) {
					EntityEnclosure entityEnclosure = entityEnclosureList.get(0);
					entrustVo.setAlias(entityEnclosure.getAlias());
					entrustVo.setEnclosureUrl(entityEnclosure.getEnclosureUrl());
					entrustVo.setExtName(entityEnclosure.getExtName());
				}
			}
		}
		pages.setRecords(list);
		return BaseResponse.okData(pages);
	}

	/**
	 * 导出报告封面和首页
	 * @param entrustId
	 * @return
	 */
	@Override
	public ReportDetailVo getReportDetail(Integer entrustId) {

		List<EntrustReport> list = entrustMapper.getReportDetail(entrustId);

		String teamKey = "CACHE:TEAM";
		String methodKey = "CACHE:METHOD";

		//缓存 60s
		if (!redisUtil.existsKey(teamKey)) {
			List<Team> teams = teamMapper.selectList(new QueryWrapper<>());
			String value = JSON.toJSONString(teams);
			redisUtil.setString(teamKey, value, 60);
		}
		if (!redisUtil.existsKey(methodKey)) {
			List<Method> methods = methodMapper.selectList(new QueryWrapper<>());
			String value = JSON.toJSONString(methods);
			redisUtil.setString(methodKey, value, 60);
		}
		List<Team> teams = JSON.parseArray(redisUtil.getString(teamKey) + "", Team.class);
		List<Method> methods = JSON.parseArray(redisUtil.getString(methodKey) + "", Method.class);
		list.forEach(arg -> {
			String teamIds = arg.getTeamIds();
			if(StringUtils.isNoneBlank(teamIds)){
				String[] teamSplit = teamIds.split("、");
				for (String idStr : teamSplit) {
					int id = Integer.parseInt(idStr);
					teams.forEach(opt -> {
						if (opt.getId() == id && 1 == opt.getQualifications()) {
							String teamName = replaceSub(opt.getName());
							if (StrUtil.isBlank(arg.getTeamName())) {
								arg.setTeamName(teamName);
							} else {
								arg.setTeamName(arg.getTeamName() + "、" + teamName);
							}
						}
					});
				}
			}

			String methodNumbers = arg.getMethodNumbers();
			if(StringUtils.isNoneBlank(methodNumbers)){
				String[] methodSplit = methodNumbers.split("、");
				for (String idStr : methodSplit) {
//					String id = Integer.parseInt(idStr) + "";
					methods.forEach(opt -> {
						if (idStr.equals(opt.getNumber())) {
							if (StrUtil.isBlank(arg.getMethodName())) {
								arg.setMethodName(opt.getName() + "  " + opt.getStandard());
							} else {
								arg.setMethodName(arg.getMethodName() + "、" + opt.getName() + "  " + opt.getStandard());
							}
						}
					});
				}
			}
		});

		if (list.size() != 0) {
			//过滤 id==null 和 projectType != '常规项目'
			List<EntrustReport> normalList = list.stream()
					.filter(arg -> arg.getId() != null && "常规项目".equals(arg.getProjectType()))
					.collect(Collectors.toList());
			return initReportDetailVo(normalList);
		}

		return null;
	}

	/**
	 * 获取十元素1的检测信息
	 * @param entrustId 委托id
	 * @return
	 */
	@Override
	public List<TenElementReport> getTen1List(Integer entrustId){
		List<TenElementReport> newList = null;
		List<ElementReport> list = getSampleTenElementCheck(entrustId);
		if(list != null && list.size() > 0){
			newList = new ArrayList<>();
		}
		Map<Integer, List<ElementReport>> groupBySample = list.stream().collect(Collectors.groupingBy(ElementReport::getId, Collectors.toList()));
		for(Map.Entry<Integer,List<ElementReport>> entry : groupBySample.entrySet()){
			ElementReport first = entry.getValue().get(0);
			TenElementReport report = new TenElementReport();
			BeanUtils.copyProperties(first, report);
			Map<String,ElementReport> reportMap = entry.getValue().stream().collect(Collectors.toMap(ElementReport::getTeamName, item -> item));
			report.setLoi(reportMap.get(ElementKey.LOI.getKey())==null || org.springframework.util.StringUtils.isEmpty(reportMap.get(ElementKey.LOI.getKey()).getLastResult()) ?"—":reportMap.get(ElementKey.LOI.getKey()).getLastResult());
			report.setSio2(reportMap.get(ElementKey.SiO2.getKey())==null || org.springframework.util.StringUtils.isEmpty(reportMap.get(ElementKey.SiO2.getKey()).getLastResult()) ?"—":reportMap.get(ElementKey.SiO2.getKey()).getLastResult());
			report.setAl2o3AndTio2(reportMap.get(ElementKey.Al2O3AndTiO2.getKey())==null || org.springframework.util.StringUtils.isEmpty(reportMap.get(ElementKey.Al2O3AndTiO2.getKey()).getLastResult()) ?"—":reportMap.get(ElementKey.Al2O3AndTiO2.getKey()).getLastResult());
			report.setFe2o3(reportMap.get(ElementKey.Fe2O3.getKey())==null || org.springframework.util.StringUtils.isEmpty(reportMap.get(ElementKey.Fe2O3.getKey()).getLastResult()) ?"—":reportMap.get(ElementKey.Fe2O3.getKey()).getLastResult());
			report.setCao(reportMap.get(ElementKey.CaO.getKey())==null || org.springframework.util.StringUtils.isEmpty(reportMap.get(ElementKey.CaO.getKey()).getLastResult()) ?"—":reportMap.get(ElementKey.CaO.getKey()).getLastResult());
			report.setMgo(reportMap.get(ElementKey.MgO.getKey())==null || org.springframework.util.StringUtils.isEmpty(reportMap.get(ElementKey.MgO.getKey()).getLastResult()) ?"—":reportMap.get(ElementKey.MgO.getKey()).getLastResult());
			report.setK2o(reportMap.get(ElementKey.K2O.getKey())==null || org.springframework.util.StringUtils.isEmpty(reportMap.get(ElementKey.K2O.getKey()).getLastResult()) ?"—":reportMap.get(ElementKey.K2O.getKey()).getLastResult());
			report.setNa2o(reportMap.get(ElementKey.Na2O.getKey())==null || org.springframework.util.StringUtils.isEmpty(reportMap.get(ElementKey.Na2O.getKey()).getLastResult()) ?"—":reportMap.get(ElementKey.Na2O.getKey()).getLastResult());
			report.setSo3(reportMap.get(ElementKey.SO3.getKey())==null || org.springframework.util.StringUtils.isEmpty(reportMap.get(ElementKey.SO3.getKey()).getLastResult()) ?"—":reportMap.get(ElementKey.SO3.getKey()).getLastResult());
			report.setCl(reportMap.get(ElementKey.Cl.getKey())==null || org.springframework.util.StringUtils.isEmpty(reportMap.get(ElementKey.Cl.getKey()).getLastResult()) ?"—":reportMap.get(ElementKey.Cl.getKey()).getLastResult());
			report.setFsio2(reportMap.get(ElementKey.FSiO2.getKey())==null || org.springframework.util.StringUtils.isEmpty(reportMap.get(ElementKey.FSiO2.getKey()).getLastResult()) ?"—":reportMap.get(ElementKey.FSiO2.getKey()).getLastResult());
			newList.add(report);
		}
		//将样品重量换算成kg
		if(newList != null){
			newList.stream().forEach(arg ->{

				arg.setWeight(Sample.weight2Kg(arg.getWeight(), arg.getWeightType()));
			});
		}
		return newList;
	}

	//获取十元素1的检测项检测信息
	private List<ElementReport> getSampleTenElementCheck(Integer entrustId) {
		Set<String> list = new HashSet<>(11);
		list.add("L.O.I");
		list.add("SiO<sub>2</sub>");
		list.add("Al<sub>2</sub>O<sub>3</sub>+TiO<sub>2</sub>");
		list.add("Fe<sub>2</sub>O<sub>3</sub>");
		list.add("CaO");
		list.add("MgO");
		list.add("K<sub>2</sub>O");
		list.add("Na<sub>2</sub>O");
		list.add("SO<sub>3</sub>");
		list.add("Cl<sup>-</sup>");
		list.add("f-SiO<sub>2</sub>");
		return sampleCheckMapper.getSampleElementCheck(entrustId, list);
	}


	/**
	 * 获取十元素2的检测信息
	 * @param entrustId 委托id
	 * @return
	 */
	@Override
	public List<TenElementReport> getTen2List(Integer entrustId){
		List<TenElementReport> newList = null;
		List<ElementReport> list = getSampleTenElementCheck(entrustId);
		if(list != null && list.size() > 0){
			newList = new ArrayList<>();
		}
		Map<Integer, List<ElementReport>> groupBySample = list.stream().collect(Collectors.groupingBy(ElementReport::getId, Collectors.toList()));
		for(Map.Entry<Integer,List<ElementReport>> entry : groupBySample.entrySet()){
			ElementReport first = entry.getValue().get(0);
			TenElementReport report = new TenElementReport();
			BeanUtils.copyProperties(first, report);
			Map<String,ElementReport> reportMap = entry.getValue().stream().collect(Collectors.toMap(ElementReport::getTeamName, item -> item));
			report.setLoi(reportMap.get(ElementKey.LOI.getKey())==null || org.springframework.util.StringUtils.isEmpty(reportMap.get(ElementKey.LOI.getKey()).getLastResult()) ?"—":reportMap.get(ElementKey.LOI.getKey()).getLastResult());
			report.setSio2(reportMap.get(ElementKey.SiO2.getKey())==null || org.springframework.util.StringUtils.isEmpty(reportMap.get(ElementKey.SiO2.getKey()).getLastResult()) ?"—":reportMap.get(ElementKey.SiO2.getKey()).getLastResult());
			report.setAl2o3(reportMap.get(ElementKey.Al2O3.getKey())==null || org.springframework.util.StringUtils.isEmpty(reportMap.get(ElementKey.Al2O3.getKey()).getLastResult()) ?"—":reportMap.get(ElementKey.Al2O3.getKey()).getLastResult());
			report.setFe2o3(reportMap.get(ElementKey.Fe2O3.getKey())==null || org.springframework.util.StringUtils.isEmpty(reportMap.get(ElementKey.Fe2O3.getKey()).getLastResult()) ?"—":reportMap.get(ElementKey.Fe2O3.getKey()).getLastResult());
			report.setCao(reportMap.get(ElementKey.CaO.getKey())==null || org.springframework.util.StringUtils.isEmpty(reportMap.get(ElementKey.CaO.getKey()).getLastResult()) ?"—":reportMap.get(ElementKey.CaO.getKey()).getLastResult());
			report.setMgo(reportMap.get(ElementKey.MgO.getKey())==null || org.springframework.util.StringUtils.isEmpty(reportMap.get(ElementKey.MgO.getKey()).getLastResult()) ?"—":reportMap.get(ElementKey.MgO.getKey()).getLastResult());
			report.setTio2(reportMap.get(ElementKey.TiO2.getKey())==null || org.springframework.util.StringUtils.isEmpty(reportMap.get(ElementKey.TiO2.getKey()).getLastResult()) ?"—":reportMap.get(ElementKey.TiO2.getKey()).getLastResult());
			report.setK2o(reportMap.get(ElementKey.K2O.getKey())==null || org.springframework.util.StringUtils.isEmpty(reportMap.get(ElementKey.K2O.getKey()).getLastResult()) ?"—":reportMap.get(ElementKey.K2O.getKey()).getLastResult());
			report.setNa2o(reportMap.get(ElementKey.Na2O.getKey())==null || org.springframework.util.StringUtils.isEmpty(reportMap.get(ElementKey.Na2O.getKey()).getLastResult()) ?"—":reportMap.get(ElementKey.Na2O.getKey()).getLastResult());
			report.setSo3(reportMap.get(ElementKey.SO3.getKey())==null || org.springframework.util.StringUtils.isEmpty(reportMap.get(ElementKey.SO3.getKey()).getLastResult()) ?"—":reportMap.get(ElementKey.SO3.getKey()).getLastResult());
			report.setCl(reportMap.get(ElementKey.Cl.getKey())==null || org.springframework.util.StringUtils.isEmpty(reportMap.get(ElementKey.Cl.getKey()).getLastResult()) ?"—":reportMap.get(ElementKey.Cl.getKey()).getLastResult());
			report.setMno(reportMap.get(ElementKey.MnO.getKey())==null || org.springframework.util.StringUtils.isEmpty(reportMap.get(ElementKey.MnO.getKey()).getLastResult()) ?"—":reportMap.get(ElementKey.MnO.getKey()).getLastResult());
			newList.add(report);
		}
		//将样品重量换算成kg
		if(newList != null){
			newList.stream().forEach(arg ->{
				arg.setWeight(Sample.weight2Kg(arg.getWeight(), arg.getWeightType()));
			});
		}
		return newList;
	}

	//获取十元素2的检测项检测信息
	private List<ElementReport> getSampleTenElementCheck2(Integer entrustId) {
		Set<String> list = new HashSet<>(12);
		list.add("L.O.I");
		list.add("SiO<sub>2</sub>");
		list.add("Al<sub>2</sub>O<sub>3</sub>");
		list.add("Fe<sub>2</sub>O<sub>3</sub>");
		list.add("CaO");
		list.add("MgO");
		list.add("TiO<sub>2</sub>");
		list.add("K<sub>2</sub>O");
		list.add("Na<sub>2</sub>O");
		list.add("SO<sub>3</sub>");
		list.add("Cl<sup>-</sup>");
		list.add("MnO");
		return sampleCheckMapper.getSampleElementCheck(entrustId, list);
	}


	/**
	 * 获取煤的工业分析的检测信息
	 * @param entrustId 委托id
	 * @return
	 */
	@Override
	public List<IndustrialReport> getIndustrialList(Integer entrustId) {
		List<IndustrialReport> newList = null;
		List<ElementReport> list = getSampleIndustrialElementCheck(entrustId);
		if(list != null && list.size() > 0){
			newList = new ArrayList<>();
		}
		Map<Integer, List<ElementReport>> groupBySample = list.stream().collect(Collectors.groupingBy(ElementReport::getId, Collectors.toList()));
		for(Map.Entry<Integer,List<ElementReport>> entry : groupBySample.entrySet()){
			ElementReport first = entry.getValue().get(0);
			IndustrialReport report = new IndustrialReport();
			BeanUtils.copyProperties(first, report);
			Map<String,ElementReport> reportMap = entry.getValue().stream().collect(Collectors.toMap(ElementReport::getTeamName, item -> item));
			//工业分析结果拆分
			String gyfxLastResult = reportMap.get(ElementKey.Gyfx.getKey())==null || org.springframework.util.StringUtils.isEmpty(reportMap.get(ElementKey.Gyfx.getKey()).getLastResult())?"":reportMap.get(ElementKey.Gyfx.getKey()).getLastResult();
			if(StringUtils.isNotBlank(gyfxLastResult) && gyfxLastResult.contains("=")){
				Map<String,String> gyfxMap = splitLastResult(gyfxLastResult);
				report.setMad(gyfxMap==null || org.springframework.util.StringUtils.isEmpty(gyfxMap.get("Mad")) ?"—":gyfxMap.get("Mad"));
				report.setAad(gyfxMap==null || org.springframework.util.StringUtils.isEmpty(gyfxMap.get("Aad")) ?"—":gyfxMap.get("Aad"));
				report.setVad(gyfxMap==null || org.springframework.util.StringUtils.isEmpty(gyfxMap.get("Vad")) ?"—":gyfxMap.get("Vad"));
				report.setQnetadmjkg(gyfxMap==null || org.springframework.util.StringUtils.isEmpty(gyfxMap.get("Qnet,ad MJ/kg")) ?"—":gyfxMap.get("Qnet,ad MJ/kg"));
				report.setJztx(gyfxMap==null || org.springframework.util.StringUtils.isEmpty(gyfxMap.get("焦渣")) ?"—":gyfxMap.get("焦渣"));
			}else{
				report.setMad("—");
				report.setAad("—");
				report.setVad("—");
				report.setQnetadmjkg("—");
				report.setJztx("—");
			}
			report.setStad(reportMap.get(ElementKey.Stad.getKey())==null || org.springframework.util.StringUtils.isEmpty(reportMap.get(ElementKey.Stad.getKey()).getLastResult()) ?"—":reportMap.get(ElementKey.Stad.getKey()).getLastResult());
			report.setClad("—");

			//可磨性结果拆分
			String kmLastResult = reportMap.get(ElementKey.Km.getKey())==null || org.springframework.util.StringUtils.isEmpty(reportMap.get(ElementKey.Km.getKey()).getLastResult())?"":reportMap.get(ElementKey.Km.getKey()).getLastResult();
			if(StringUtils.isNotBlank(kmLastResult) && kmLastResult.contains("=")){
				Map<String,String> kmMap = splitLastResult(kmLastResult);
				report.setHgi(kmMap==null || org.springframework.util.StringUtils.isEmpty(kmMap.get("HGI")) ?"—":kmMap.get("HGI"));
				report.setCyl(kmMap==null || org.springframework.util.StringUtils.isEmpty(kmMap.get("制样率")) ?"—":kmMap.get("制样率"));
			}else{
				report.setHgi("—");
				report.setCyl("—");
			}

			newList.add(report);
		}
		//将样品重量换算成kg
		if(newList != null){
			newList.stream().forEach(arg ->{
				arg.setWeight(Sample.weight2Kg(arg.getWeight(), arg.getWeightType()));
			});
		}
		return newList;
	}

	//获取煤的工业分析的检测项检测信息
	private List<ElementReport> getSampleIndustrialElementCheck(Integer entrustId) {
		Set<String> list = new HashSet<>(3);
		list.add("工业分析");
		list.add("St,ad");
		list.add("可磨");
		return sampleCheckMapper.getSampleElementCheck(entrustId, list);
	}



	/**
	 * 获取工艺性能的检测信息
	 * @param entrustId 委托id
	 * @return
	 */
	@Override
	public List<CraftReport> getCraftList(Integer entrustId) {
		List<CraftReport> newList = null;
		List<ElementReport> list = getSampleCraftElementCheck(entrustId);
		if(list != null && list.size() > 0){
			newList = new ArrayList<>();
		}
		Map<Integer, List<ElementReport>> groupBySample = list.stream().collect(Collectors.groupingBy(ElementReport::getId, Collectors.toList()));
		for(Map.Entry<Integer,List<ElementReport>> entry : groupBySample.entrySet()){
			ElementReport first = entry.getValue().get(0);
			CraftReport report = new CraftReport();
			BeanUtils.copyProperties(first, report);
			Map<String,ElementReport> reportMap = entry.getValue().stream().collect(Collectors.toMap(ElementReport::getTeamName, item -> item));

			//易磨性结果拆分
			String ymxLastResult = reportMap.get(ElementKey.Ymx.getKey())==null || org.springframework.util.StringUtils.isEmpty(reportMap.get(ElementKey.Ymx.getKey()).getLastResult())?"":reportMap.get(ElementKey.Ymx.getKey()).getLastResult();
			if(StringUtils.isNotBlank(ymxLastResult) && ymxLastResult.contains("=")){
				Map<String,String> ymxMap = splitLastResult(ymxLastResult);
				report.setMjt(ymxMap==null || org.springframework.util.StringUtils.isEmpty(ymxMap.get("易磨性(MJ/t)")) ?"—":ymxMap.get("易磨性(MJ/t)"));
				report.setKwht(ymxMap==null || org.springframework.util.StringUtils.isEmpty(ymxMap.get("易磨性值")) ?"—":ymxMap.get("易磨性值"));
			}else{
				report.setMjt("—");
				report.setKwht("—");
			}

			//磨蚀性结果拆分
			String fsxLastResult = reportMap.get(ElementKey.Fsx.getKey())==null || org.springframework.util.StringUtils.isEmpty(reportMap.get(ElementKey.Fsx.getKey()).getLastResult())?"":reportMap.get(ElementKey.Fsx.getKey()).getLastResult();
			if(StringUtils.isNotBlank(fsxLastResult) && fsxLastResult.contains("=")){
				Map<String,String> fsxMap = splitLastResult(fsxLastResult);
				report.setAi(fsxMap==null || org.springframework.util.StringUtils.isEmpty(fsxMap.get("Ai(g)")) ?"—":fsxMap.get("Ai(g)"));
			}else{
				report.setAi("—");
			}

			//可磨性结果拆分
			String kmLastResult = reportMap.get(ElementKey.Km.getKey())==null || org.springframework.util.StringUtils.isEmpty(reportMap.get(ElementKey.Km.getKey()).getLastResult())?"":reportMap.get(ElementKey.Km.getKey()).getLastResult();
			if(StringUtils.isNotBlank(kmLastResult) && kmLastResult.contains("=")){
				Map<String,String> kmMap = splitLastResult(kmLastResult);
				report.setHgi(kmMap==null || org.springframework.util.StringUtils.isEmpty(kmMap.get("HGI")) ?"—":kmMap.get("HGI"));
				report.setCyl(kmMap==null || org.springframework.util.StringUtils.isEmpty(kmMap.get("制样率")) ?"—":kmMap.get("制样率"));
			}else{
				report.setHgi("—");
				report.setCyl("—");
			}

			//辊磨结果拆分
			String gmLastResult = reportMap.get(ElementKey.Gm.getKey())==null || org.springframework.util.StringUtils.isEmpty(reportMap.get(ElementKey.Gm.getKey()).getLastResult())?"":reportMap.get(ElementKey.Gm.getKey()).getLastResult();
			if(StringUtils.isNotBlank(gmLastResult) && gmLastResult.contains("=")){
				Map<String,String> gmMap = splitLastResult(gmLastResult);
				report.setTmf(gmMap==null || org.springframework.util.StringUtils.isEmpty(gmMap.get("TMF")) ?"—":gmMap.get("TMF"));
				report.setTwf(gmMap==null || org.springframework.util.StringUtils.isEmpty(gmMap.get("TWF(kg/t)")) ?"—":gmMap.get("TWF(kg/t)"));
			}else{
				report.setTmf("—");
				report.setTwf("—");
			}

			//易烧性结果 不用拆分  直接取游离钙结果
			report.setFcao1350(reportMap.get(ElementKey.FCaO1350.getKey())==null || org.springframework.util.StringUtils.isEmpty(reportMap.get(ElementKey.FCaO1350.getKey()).getLastResult()) ?"—":reportMap.get(ElementKey.FCaO1350.getKey()).getLastResult());
			report.setFcao1400(reportMap.get(ElementKey.FCaO1400.getKey())==null || org.springframework.util.StringUtils.isEmpty(reportMap.get(ElementKey.FCaO1400.getKey()).getLastResult()) ?"—":reportMap.get(ElementKey.FCaO1400.getKey()).getLastResult());
			report.setFcao1450(reportMap.get(ElementKey.FCaO1450.getKey())==null || org.springframework.util.StringUtils.isEmpty(reportMap.get(ElementKey.FCaO1450.getKey()).getLastResult()) ?"—":reportMap.get(ElementKey.FCaO1450.getKey()).getLastResult());
			newList.add(report);
		}
		//将样品重量换算成kg
		if(newList != null){
			newList.stream().forEach(arg ->{
				arg.setWeight(Sample.weight2Kg(arg.getWeight(), arg.getWeightType()));
			});
		}
		return newList;
	}

	//获取工艺性能的检测项检测信息
	private List<ElementReport> getSampleCraftElementCheck(Integer entrustId) {
		Set<String> list = new HashSet<>(7);
		list.add("易磨性");
		list.add("磨蚀性");
		list.add("可磨");
		list.add("辊磨");
		list.add("fCaO 1350");
		list.add("fCaO 1400");
		list.add("fCaO 1450");
		return sampleCheckMapper.getSampleElementCheck(entrustId, list);
	}





	/**
	 * 获取物理性能的检测信息
	 * @param entrustId 委托id
	 * @return
	 */
	@Override
	public List<PhysicsReport> getPhysicsList(Integer entrustId) {
		List<PhysicsReport> newList = null;
		List<ElementReport> list = getSamplePhysicsElementCheck(entrustId);
		if(list != null && list.size() > 0){
			newList = new ArrayList<>();
		}
		Map<Integer, List<ElementReport>> groupBySample = list.stream().collect(Collectors.groupingBy(ElementReport::getId, Collectors.toList()));
		for(Map.Entry<Integer,List<ElementReport>> entry : groupBySample.entrySet()){
			ElementReport first = entry.getValue().get(0);
			PhysicsReport report = new PhysicsReport();
			BeanUtils.copyProperties(first, report);
			Map<String,ElementReport> reportMap = entry.getValue().stream().collect(Collectors.toMap(ElementReport::getTeamName, item -> item));

			report.setXd45um(reportMap.get(ElementKey.Xd45um.getKey())==null || org.springframework.util.StringUtils.isEmpty(reportMap.get(ElementKey.Xd45um.getKey()).getLastResult()) ?"—":reportMap.get(ElementKey.Xd45um.getKey()).getLastResult());
			report.setXd80um(reportMap.get(ElementKey.Xd80um.getKey())==null || org.springframework.util.StringUtils.isEmpty(reportMap.get(ElementKey.Xd80um.getKey()).getLastResult()) ?"—":reportMap.get(ElementKey.Xd80um.getKey()).getLastResult());
			report.setXd200um(reportMap.get(ElementKey.Xd200um.getKey())==null || org.springframework.util.StringUtils.isEmpty(reportMap.get(ElementKey.Xd200um.getKey()).getLastResult()) ?"—":reportMap.get(ElementKey.Xd200um.getKey()).getLastResult());
			report.setBbmj(reportMap.get(ElementKey.Bbmj.getKey())==null || org.springframework.util.StringUtils.isEmpty(reportMap.get(ElementKey.Bbmj.getKey()).getLastResult()) ?"—":reportMap.get(ElementKey.Bbmj.getKey()).getLastResult());
			report.setMd(reportMap.get(ElementKey.Md.getKey())==null || org.springframework.util.StringUtils.isEmpty(reportMap.get(ElementKey.Md.getKey()).getLastResult()) ?"—":reportMap.get(ElementKey.Md.getKey()).getLastResult());

			//塑性指数结果拆分
			String sxzsLastResult = reportMap.get(ElementKey.Sxzs.getKey())==null || org.springframework.util.StringUtils.isEmpty(reportMap.get(ElementKey.Sxzs.getKey()).getLastResult())?"":reportMap.get(ElementKey.Sxzs.getKey()).getLastResult();
			if(StringUtils.isNotBlank(sxzsLastResult) && sxzsLastResult.contains("=")){
				Map<String,String> sxzsMap = splitLastResult(sxzsLastResult);
				report.setSxzs_yx(sxzsMap==null || org.springframework.util.StringUtils.isEmpty(sxzsMap.get("液限")) ?"—":sxzsMap.get("液限"));
				report.setSxzs_sx(sxzsMap==null || org.springframework.util.StringUtils.isEmpty(sxzsMap.get("塑限")) ?"—":sxzsMap.get("塑限"));
				report.setSxzs_sxzs(sxzsMap==null || org.springframework.util.StringUtils.isEmpty(sxzsMap.get("塑性指数")) ?"—":sxzsMap.get("塑性指数"));
			}else{
				report.setSxzs_yx("—");
				report.setSxzs_sx("—");
				report.setSxzs_sxzs("—");
			}
			report.setHsl(reportMap.get(ElementKey.Sf.getKey())==null || org.springframework.util.StringUtils.isEmpty(reportMap.get(ElementKey.Sf.getKey()).getLastResult()) ?"—":reportMap.get(ElementKey.Sf.getKey()).getLastResult());
			newList.add(report);
		}
		//将样品重量换算成kg
		if(newList != null){
			newList.stream().forEach(arg ->{
				arg.setWeight(Sample.weight2Kg(arg.getWeight(), arg.getWeightType()));
			});
		}
		return newList;
	}

	//获取物理性能的检测项检测信息
	private List<ElementReport> getSamplePhysicsElementCheck(Integer entrustId) {
		Set<String> list = new HashSet<>(7);
		list.add("45μm细度");
		list.add("80μm细度");
		list.add("200μm细度");
		list.add("比表面积");
		list.add("密度");
		list.add("塑性指数");
		list.add("水分");
		return sampleCheckMapper.getSampleElementCheck(entrustId, list);
	}



	/**
	 * 获取颗粒分析的检测信息
	 * @param entrustId 委托id
	 * @return
	 */
	@Override
	public List<GrainReport> getGrainList(Integer entrustId) {
		List<GrainReport> newList = null;
		List<ElementReport> list = getSampleGrainElementCheck(entrustId);
		if(list != null && list.size() > 0){
			newList = new ArrayList<>();
		}
		Map<Integer, List<ElementReport>> groupBySample = list.stream().collect(Collectors.groupingBy(ElementReport::getId, Collectors.toList()));
		for(Map.Entry<Integer,List<ElementReport>> entry : groupBySample.entrySet()){
			ElementReport first = entry.getValue().get(0);
			GrainReport report = new GrainReport();
			BeanUtils.copyProperties(first, report);
			Map<String,ElementReport> reportMap = entry.getValue().stream().collect(Collectors.toMap(ElementReport::getTeamName, item -> item));

			report.setJzmc("—");
			report.setJzzsl("—");
			report.setYqzjjzrq("—");
			report.setZgb("—");
			report.setCsbgl("—");

			//激光粒度分析结果拆分
			String jgldrfxLastResult = reportMap.get(ElementKey.Jgldfx.getKey())==null || org.springframework.util.StringUtils.isEmpty(reportMap.get(ElementKey.Jgldfx.getKey()).getLastResult())?"":reportMap.get(ElementKey.Jgldfx.getKey()).getLastResult();
			if(StringUtils.isNotBlank(jgldrfxLastResult) && jgldrfxLastResult.contains("=")){
				Map<String,String> jgldrfxMap = splitLastResult(jgldrfxLastResult);
				report.setD10(jgldrfxMap==null || org.springframework.util.StringUtils.isEmpty(jgldrfxMap.get("D10")) ?"—":jgldrfxMap.get("D10"));
				report.setD50(jgldrfxMap==null || org.springframework.util.StringUtils.isEmpty(jgldrfxMap.get("D50")) ?"—":jgldrfxMap.get("D50"));
				report.setD90(jgldrfxMap==null || org.springframework.util.StringUtils.isEmpty(jgldrfxMap.get("D90")) ?"—":jgldrfxMap.get("D90"));
			}else{
				report.setD10("—");
				report.setD50("—");
				report.setD90("—");
			}
			report.setBmjjyljd("—");
			report.setTzlj("—");
			report.setJyxxsn("—");
			report.setLjfbbt("—");
			newList.add(report);
		}
		//将样品重量换算成kg
		if(newList != null){
			newList.stream().forEach(arg ->{
				arg.setWeight(Sample.weight2Kg(arg.getWeight(), arg.getWeightType()));
			});
		}
		return newList;
	}

	//获取颗粒分析的检测项检测信息
	private List<ElementReport> getSampleGrainElementCheck(Integer entrustId) {
		Set<String> list = new HashSet<>(1);
		list.add("激光粒度分析");
		return sampleCheckMapper.getSampleElementCheck(entrustId, list);
	}



	/**
	 * 特殊处理的检测项最终结果拆分显示
	 * @param lastResult
	 * @return
	 */
	private Map<String,String> splitLastResult(String lastResult){
		Map<String,String> map = new HashMap<>();
		if(StringUtils.isNotBlank(lastResult)){
			String[] str = lastResult.split("\n");
			for(String str2 : str){
				String[] ele = str2.trim().split("=");
				map.put(ele[0], ele[1]);
			}
		}
		return map;
	}

	//***********************************质量控制************************************
	/**
	 * 质量检测分页列表
	 * @param pageQuery
	 * @param startDate
	 * @param endDate
	 * @param clientId
	 * @param projectName
	 * @param projectCode
	 * @return
	 */
	@Override
	public BaseResponse<IPage<EntrustVo>> getQualityPage(PageQuery pageQuery, String startDate, String endDate, Integer clientId, String projectName, String projectCode) {

		Map<String, Object> params = new HashMap<>(9);
		params.put("startDate", startDate);
		params.put("endDate", endDate);
		params.put("status", 8);
		params.put("clientId", clientId);
		params.put("projectName", projectName);
		params.put("projectCode", projectCode);
		params.put("projectType", "质量控制");
		List<Integer> approvalIdList = sysApprovalMapper.getApprovalId("委托评审");
		if (approvalIdList == null) {
			return BaseResponse.errorMsg("委托评审信息错误");
		}

		Page<EntrustVo> page = new Page<>(pageQuery.getPageNo(), pageQuery.getPageSize());
		IPage<EntrustVo> pages = entrustMapper.getPage(page, params);
		List<EntrustVo> list = pages.getRecords();

		if (list != null && list.size() > 0) {
			for (EntrustVo entrustVo : list) {
				QueryWrapper<SampleTmp> queryWrapper = new QueryWrapper<>();
				queryWrapper.eq("entrust_id", entrustVo.getId());
				List<SampleTmp> sampleTmpList = sampleTmpService.list(queryWrapper);
				//检测项目名称
				String sampleNames = "";
				if (sampleTmpList != null && sampleTmpList.size() > 0) {
					//样品名称 列表中的拼接
					for (SampleTmp sampleTmp : sampleTmpList) {
						sampleNames = "".equals(sampleNames) ? sampleTmp.getName() : (sampleNames + "、" + sampleTmp.getName());
					}
					entrustVo.setSampleNames(sampleNames);
					//检测项 和检测依据 列表里只显示第一个样品的
					SampleTmp sampleTmp = sampleTmpList.get(0);
					String teamIds = sampleTmp.getTeamIds();
					String checkTeam = "";
					if (teamIds != null) {
						String[] teamIdS = teamIds.split("、");
						for (String teamId : teamIdS) {
							Team team = teamMapper.selectById(Integer.valueOf(teamId));
							if (team != null) {
								checkTeam = "".equals(checkTeam) ? team.getName() : (checkTeam + "、" + team.getName());
							}
						}
					}
					entrustVo.setCheckTeam(checkTeam);
					entrustVo.setCheckMethodNumber(sampleTmp.getMethodNumbers());
				}
			}
		}
		pages.setRecords(list);
		return BaseResponse.okData(pages);
	}

	/**
	 * 获取质量详情
	 * @param entrustId
	 * @return
	 */
	@Override
	public QualityDetailVo getQualityDetail(Integer entrustId) {

		List<QualityDetail> qualityDetails = entrustMapper.getQualityDetail(entrustId);
		if (qualityDetails.size() == 0) {
			return null;
		}
		QualityDetail firstQualityDetail = qualityDetails.get(0);
		QualityDetailVo qualityDetailVo = new QualityDetailVo();

		BeanUtil.copyProperties(firstQualityDetail, qualityDetailVo);
		//获取clientId和userName
//		String userCacheKey = "CACHE:USER";
//		String clientCacheKey = "CACHE:CLIENT";
	/*	//缓存 60s
		if (!redisUtil.existsKey(userCacheKey)) {
			List<SysUser> users = userService.list();
			String value = JSON.toJSONString(users);
			redisUtil.setString(userCacheKey, value, 60);
		}*/
//		if (!redisUtil.existsKey(clientCacheKey)) {
//			List<Client> clients = clientMapper.selectList(new QueryWrapper<>());
//			String value = JSON.toJSONString(clients);
//			redisUtil.setString(clientCacheKey, value, 60);
//		}
//		List<SysUser> sysUsers = JSON.parseArray(redisUtil.getString(userCacheKey) + "", SysUser.class);
//		List<Client> clients = JSON.parseArray(redisUtil.getString(clientCacheKey) + "", Client.class);
		//关联委托方
//		clients.stream()
//				.filter(opt -> opt.getId().intValue() == firstQualityDetail.getClientId())
//				.findFirst().ifPresent(opt -> qualityDetailVo.setClientName(opt.getName()));
		//关联检测者名字
	/*	qualityDetails.forEach(arg -> sysUsers.stream()
				.filter(opt -> opt.getId().intValue() == arg.getUserId())
				.findFirst().ifPresent(opt -> arg.setUserName(opt.getName())));*/
		//为每个检测组 创建实体
		qualityDetails.stream()
				.map(QualityDetail::getTeamGroupName)
				.collect(Collectors.toSet())
				.forEach(arg -> {
					//遍历每个检测组名字 创建检测组集合
					QualityDetailVo.QualityTeamGroup qualityTeamGroup = new QualityDetailVo.QualityTeamGroup();
					qualityTeamGroup.setTeamGroupName(arg);
					qualityDetailVo.getQualityTeamGroups().add(qualityTeamGroup);
					List<QualityDetail> collect = qualityDetails.stream()
							//找到当前检测组下的是要样品
							.filter(opt -> opt.getTeamGroupName().equals(arg))
							.collect(Collectors.toList());
					collect.forEach(opt -> qualityTeamGroup.setTeamGroupId(opt.getTeamGroupId()));
					collect.stream()
							.map(QualityDetail::getSampleName)
							.collect(Collectors.toSet())
							.forEach(opt -> {
								//拿到了样品名字和检测组名字 开始创建Vo对象
								QualityDetailVo.QualitySample qualitySample =
										new QualityDetailVo.QualitySample();
								qualitySample.setSampleName(opt);
								qualityTeamGroup.getQualitySamples().add(qualitySample);
								qualityDetails.forEach(param -> {
									//找到检测组和样品相同的详情
									if (param.getTeamGroupName().equals(arg) && param.getSampleName().equals(opt)) {
										QualityDetailVo.SampleOriginal sampleOriginal = new QualityDetailVo.SampleOriginal();
										sampleOriginal.setCementCode(param.getEntrustCode());
										sampleOriginal.setTeamValues(param.getDisResult());
										sampleOriginal.setUserName(param.getUserName());
										sampleOriginal.setUserId(param.getUserId());
										qualitySample.getSampleOriginals().add(sampleOriginal);
										qualitySample.setCementCode(param.getCementCode());
										qualitySample.setSampleCode(param.getSampleCode());
										qualitySample.setSampleForm(param.getSampleForm());
										qualitySample.setSampleId(param.getSampleId());
									}
								});
							});
				});

		return qualityDetailVo;
	}


	/**
	 * 替换特殊角标
	 *
	 * @param teamName 待替换元素名
	 * @return 新元素名
	 */
	private String replaceSub(String teamName) {
		String[] subs = {"₀", "₁", "₂", "₃", "₄", "₅", "₆", "₇", "₈", "₉", "₊", "₋"};
		String[] sups = {"⁰", "¹", "²", "³", "⁴", "⁵", "⁶", "⁷", "⁸", "⁹", "⁺", "⁻", "⁼", "⁽", "⁾", "ⁿ"};
		String rts = teamName;
		if (StringUtils.isEmpty(teamName)) {
			return rts;
		}

		if (teamName.contains("<sub>1</sub>")) {
			rts = rts.replace("<sub>1</sub>", "₁");
		}
		if (teamName.contains("<sub>2</sub>")) {
			rts = rts.replace("<sub>2</sub>", "₂");
		}
		if (teamName.contains("<sub>3</sub>")) {
			rts = rts.replace("<sub>3</sub>", "₃");
		}
		if (teamName.contains("<sub>4</sub>")) {
			rts = rts.replace("<sub>4</sub>", "₄");
		}
		if (teamName.contains("<sub>5</sub>")) {
			rts = rts.replace("<sub>5</sub>", "₅");
		}
		if (teamName.contains("<sub>6</sub>")) {
			rts = rts.replace("<sub>6</sub>", "₆");
		}
		if (teamName.contains("<sub>7</sub>")) {
			rts = rts.replace("<sub>7</sub>", "₇");
		}
		if (teamName.contains("<sub>8</sub>")) {
			rts = rts.replace("<sub>8</sub>", "₈");
		}
		if (teamName.contains("<sub>9</sub>")) {
			rts = rts.replace("<sub>9</sub>", "₉");
		}
		if (teamName.contains("<sup>-</sup>")) {
			rts = rts.replace("<sup>-</sup>", "⁻");
		}
		if (teamName.contains("<sup>+</sup>")) {
			rts = rts.replace("<sup>+</sup>", "⁺");
		}

		return rts;
	}

	/**
	 * 根据样品检测列表获取报告管理首页
	 *
	 * @param source 样品检测
	 * @return ReportDetailVo
	 */
	private ReportDetailVo initReportDetailVo(List<EntrustReport> source) {

		if (source.size() == 0) {
			return null;
		}

		ReportDetailVo rts = new ReportDetailVo();
		EntrustReport firstReportDetail = source.get(0);
		//初始化标准信息
		rts.setPrintDate(DateUtil.format(new Date(), "yyyy年MM月dd日"));
		rts.setSender(firstReportDetail.getSendName());
		rts.setSendDate(DateUtil.format(firstReportDetail.getSendTime(), "yyyy年MM月dd日"));
		rts.setProjectName(firstReportDetail.getProjectName());
		rts.setReportNo(firstReportDetail.getEntrustCode());
		rts.setSendName("\\");
		//获取检测项目、检测仪器、检测依据
		rts.setEquipmentNames(new HashSet<>(source.size()));
		rts.setSampleNames(new HashSet<>(source.size()));
		rts.setMethodNames(new HashSet<>(source.size()));
		rts.setTeamNames(new HashSet<>(source.size()));
		Set<Integer> equipmentIds = new HashSet<>(source.size());
		source.forEach(arg -> {
			if(arg.getEquipmentId() != null){
				equipmentIds.add(arg.getEquipmentId());
			}
			//获取使用的设备id view
			//关联检测项目
			if (StrUtil.isNotBlank(arg.getTeamName())) {
				String[] teamSplits = arg.getTeamName().split("、");
				for (String teamName : teamSplits) {
					rts.getTeamNames().add(teamName);
				}
			}
			//关联检测依据
			if (StrUtil.isNotBlank(arg.getMethodName())) {
				String[] methodSplits = arg.getMethodName().split("、");
				for (String methodName : methodSplits) {
					rts.getMethodNames().add(methodName);
				}
			}
			//关联样品名
			if (StrUtil.isNotBlank(arg.getName())) {
				rts.getSampleNames().add(arg.getName());
			}

		});
		rts.setSampleNum(rts.getSampleNames().size());

		//关联仪器名字 去重
		if(equipmentIds != null && equipmentIds.size()>0){
			List<String> equipmentNames = equipmentUseMapper.getEquipmentNamesByEquipmentIds(equipmentIds);
			equipmentNames.forEach(arg -> rts.getEquipmentNames().add(arg));
		}


		//关联送样单位名字
		String clientKey = "CACHE:CLIENT";
		if (!redisUtil.existsKey(clientKey)) {
			List<Client> clients = clientMapper.selectList(null);
			redisUtil.setString(clientKey, JSON.toJSONString(clients), 60);
		}
		List<Client> clients = JSON.parseArray(redisUtil.getString(clientKey) + "", Client.class);
		if (clients.size() == 0) {
			log.error("=================送样单位表中没有任何数据!==================");
		} else {
			clients.stream().filter(arg -> arg.getId().intValue() == firstReportDetail.getClientId())
					.findFirst().ifPresent(arg -> rts.setSendName(arg.getName()));
		}
		return rts;
	}

	//******************************导出****************************************

	/**
	 * 委托列表导出
	 */
	@Override
	public void export(String startDate, String endDate, Integer status,
	                   Integer clientId, String projectName, String projectCode,
	                   String fileName, HttpServletResponse response) {
		Map<String, Object> params = new HashMap<>();
		params.put("startDate", startDate);
		params.put("endDate", endDate);
		params.put("status", status);
		params.put("clientId", clientId);
		params.put("projectName", projectName);
		params.put("projectCode", projectCode);
		List<Map<String, Object>> list = entrustMapper.exportList(params);

		if (!CollectionUtils.isEmpty(list)) {
			Map<String, Object> map = list.get(0);
			String[] headers = new String[map.size() + 1];
			headers[0] = "序号";
			headers[1] = "委托单号";
			headers[2] = "是否加急";
			headers[3] = "项目名称";
			headers[4] = "项目编号";
			headers[5] = "委托人";
			headers[6] = "委托单位";
			headers[7] = "委托日期";
			headers[8] = "样品数量";
			headers[9] = "检测项目";
			headers[10] = "检测依据编号";
			headers[11] = "状态";

			List<Object[]> datas = new ArrayList<>(headers.length);
			for (Map<String, Object> m : list) {
				//获取委托单的检测项目  检测依据编号
				//根据委托编号id 获取所有样品的检测项目 和 检测依据编号
				Integer entrustId = Integer.valueOf(m.get("entrustId").toString());
				System.out.println(entrustId);
				QueryWrapper<SampleTmp> queryWrapper = new QueryWrapper<>();
				queryWrapper.eq("entrust_id", entrustId);
				List<SampleTmp> sampleTmpList = sampleTmpService.list(queryWrapper);

//                String sampleNames = "";
				/**
				 * 检测项目名称
				 * 格式：
				 * 样品1：Mad、Vad、Cao
				 * 样品2：Mad、Vad、Cao
				 * 样品3：Mad、Vad、Cao
				 */
				String checkTeamStr = "";
				/**
				 * 检测依据编号
				 * 格式：
				 * 样品1：0012、0013
				 * 样品2：0014、0015
				 * 样品3：0016、0017
				 */
				String checkMethodStr = "";
				if (sampleTmpList != null && sampleTmpList.size() > 0) {
					//样品名称 列表中的拼接
					for (SampleTmp sampleTmp : sampleTmpList) {
//                        sampleNames = sampleNames.equals("") ? sampleTmp.getName() : (sampleNames + "、" + sampleTmp.getName());
						String teamIds = sampleTmp.getTeamIds();
						String checkTeam = "";
						String checkMethod = "";
						if (teamIds != null) {
							String[] teamIdS = teamIds.split("、");
							for (String teamId : teamIdS) {
								Team team = teamMapper.selectById(Integer.valueOf(teamId));
								if (team != null && team.getQualifications() == 1) {
									String teamName = team.getName();
                                    /*if(team.getName().contains("<sub>") || team.getName().contains("</sub>") ){
                                        teamName = team.getName().replaceAll("<sub>","")
                                                .replaceAll("</sub>","");
                                    }*/
									checkTeam = checkTeam.equals("") ? teamName : (checkTeam + "、" + teamName);
									Method method = methodMapper.selectById(team.getMethodId());
									if (method != null) {
										checkMethod = checkMethod.equals("") ? method.getNumber() : (checkMethod + "、" + method.getNumber());
									}
								}
							}
						}
						String spampe_checkTeam = sampleTmp.getName() + "：" + checkTeam;
						checkTeamStr = checkTeamStr.equals("") ? spampe_checkTeam : (checkTeamStr + "\n" + spampe_checkTeam);
						String spampe_checkMethod = sampleTmp.getName() + "：" + checkMethod;
						checkMethodStr = checkMethodStr.equals("") ? spampe_checkMethod : (checkMethodStr + "\n" + spampe_checkMethod);
					}
				}
				Object[] objects = new Object[headers.length];
				for (int j = 0; j < headers.length; j++) {
					String obj = "";
					if (j == 9) {
						obj = checkTeamStr;
					} else if (j == 10) {
						obj = checkMethodStr;
					} else {
						obj = m.get(headers[j]) != null ? m.get(headers[j]).toString() : "";
						//如果序号带小数点 去除.0,保留整数
						if (j == 0) {
							obj = obj.split("\\.")[0];
						}
					}
					objects[j] = obj;
				}
				datas.add(objects);
			}
			ExcelUtil.excelExport(
					fileName == null || fileName.trim().length() <= 0 ? "委托列表" : fileName, headers,
					datas, response);
		}
	}

	/**
	 * 委托单进展导出（东哥写的例子）
	 * @param cycle
	 * @param startDate
	 * @param endDate
	 * @param clientId
	 * @param clientName
	 * @param fileName
	 * @param response
	 */
	public void exportConsignation(Integer cycle, String startDate,
	                               String endDate, Integer clientId, String clientName,
	                               String fileName, HttpServletResponse response) {

		//1.首先查询所有可见的检测项
		QueryWrapper<Team> teamQueryWrapper = new QueryWrapper<>();
		teamQueryWrapper.eq("is_display", 1);
		teamQueryWrapper.eq("status", 1);
		List<Team> teamList = teamService.list(teamQueryWrapper);

		//转化成id为key的map
//        Map<Integer, Team> teamMap = teamList.stream().collect(Collectors.toMap(Team::getId, a -> a, (k1, k2) -> k1));
		//2.条件查询委托单项
		Map<String, Object> params = new HashMap<>();
		params.put("cycle", cycle);
		params.put("startDate", startDate);
		params.put("endDate", endDate);
		params.put("clientId", clientId);
		params.put("clientName", clientName);
		List<Map<String, Object>> entrustList = entrustMapper.exportList(params);
		//获取所有的委托单号
		List<Integer> queryList = entrustList.stream().map(x -> Integer.parseInt(x.get("entrustId").toString())).collect(Collectors.toList());
		//3.根据委托单号获取样品
		QueryWrapper<Sample> queryWrapper = new QueryWrapper<>();
		queryWrapper.in("entrust_id", queryList);
		List<Sample> sampleList = sampleService.list(queryWrapper);
		//4.查询所有的派发检测项
		QueryWrapper<SampleDistribution> sdqueryWrapper = new QueryWrapper<>();
		sdqueryWrapper.in("entrust_id", queryList);
		sdqueryWrapper.in("sample_id", sampleList.stream().filter(x -> x.getCementCode().equals(x.getParallelCode())).map(Sample::getId).collect(Collectors.toList()));
		List<SampleDistribution> sampleDistributions = sampleDistributionService.list(sdqueryWrapper);
		//5.分组并统计
		Map<Integer, List<SampleDistribution>> groupByEntrustId = sampleDistributions.stream().collect(Collectors.groupingBy(SampleDistribution::getEntrustId, Collectors.toList()));
		//6.开始填充数据。首先填充表头
		XSSFWorkbook wb = new XSSFWorkbook();
		XSSFSheet sheet = wb.createSheet();
		XSSFCellStyle style = wb.createCellStyle();
		XSSFFont font = wb.createFont();
		//水平居中
		style.setAlignment(HorizontalAlignment.CENTER);
		//垂直居中
		style.setVerticalAlignment(VerticalAlignment.CENTER);
		//生成表头
		String[] in_fo = {
				"委托单号",
				"委托单位",
				"项目编号",
				"项目名称",
				"委托人",
				"样品数量",
				"委托日期",
				"是否加急",
				"实际产值",
		};
		List<String> infoHeader = Arrays.asList(in_fo);
		generatoronsignationWorkbook(font, style, sheet, infoHeader, teamList.stream().map(Team::getName).collect(Collectors.toList()));
		//填充数据,从第二行开始
		int rownum = 2;
		for (int j = 0; j < entrustList.size(); j++) {

			Map<String, Object> entrust = entrustList.get(j);
			int entrustId = Integer.parseInt(entrust.get("entrustId").toString());
			if (groupByEntrustId.get(entrustId) != null) {
				XSSFRow row0 = sheet.createRow(rownum++);
				Map<Integer, List<SampleDistribution>> sampleDistribuGroup = groupByEntrustId.get(entrustId).stream().collect(Collectors.groupingBy(SampleDistribution::getTeamId, Collectors.toList()));
				//处理委托人信息表头
				for (int i = 0; i < infoHeader.size(); i++) {
					XSSFCell tempCell = row0.createCell(i);
					if (infoHeader.get(i).equals("实际产值")) {
						Double outPutValue = sampleList
								.stream()
								.filter(x -> x.getCementCode().equals(x.getParallelCode()) && x.getEntrustId() == entrustId)
								.mapToDouble(item -> item.getOutputValue().doubleValue()).sum();
						tempCell.setCellValue(outPutValue == null ? "" : outPutValue.toString());
						tempCell.setCellStyle(style);
					} else {
						tempCell.setCellValue(entrust.get(infoHeader.get(i)) == null ? "" : entrust.get(infoHeader.get(i)).toString());
						tempCell.setCellStyle(style);
					}
				}

				//处理检测项总产值表头
				for (int i = infoHeader.size(); i < teamList.size() + infoHeader.size(); i++) {
					XSSFCell tempCell = row0.createCell(i);
					Team team = teamList.get(i - infoHeader.size());
					tempCell.setCellValue(sampleDistribuGroup.get(team.getId()) == null ? "" : team.getCharge().multiply(new BigDecimal(sampleDistribuGroup.get(team.getId()).size())).toString());
					tempCell.setCellStyle(style);
				}

				//增加1空列
				{
					XSSFCell tempCell = row0.createCell(teamList.size() + infoHeader.size());
					tempCell.setCellValue("");
					tempCell.setCellStyle(style);
				}
				//处理检测项信息表头
				for (int i = teamList.size() + infoHeader.size() + 1; i < infoHeader.size() + 1 + teamList.size() * 3; i++) {
					Team team = teamList.get((i - teamList.size() - infoHeader.size() - 1) / 2);
					XSSFCell tempCell1 = row0.createCell(i);
					tempCell1.setCellStyle(style);
					tempCell1.setCellValue(sampleDistribuGroup.get(team.getId()) == null ? "" : team.getCharge().toString());
					XSSFCell tempCell2 = row0.createCell(++i);
					tempCell2.setCellValue(sampleDistribuGroup.get(team.getId()) == null ? "" : sampleDistribuGroup.get(team.getId()).size() + "");
					tempCell2.setCellStyle(style);
				}
			}
		}
		ExcelUtil.excelExportNew(fileName == null || fileName.trim().length() <= 0 ? "委托单进展" : fileName, wb, response);

	}

	private static void generatoronsignationWorkbook(XSSFFont font, XSSFCellStyle style, XSSFSheet sheet, List<String> infoHeader, List<String> teamsHeader) {
		//委托项的表头

		int rownum = 0;
		//创建第一行
		XSSFRow row0 = sheet.createRow(rownum++);
		//处理委托人信息表头
		for (int i = 0; i < infoHeader.size(); i++) {
			XSSFCell tempCell = row0.createCell(i);
			tempCell.setCellValue(infoHeader.get(i));
			tempCell.setCellStyle(style);
		}
		//处理检测项总产值表头
		for (int i = infoHeader.size(); i < teamsHeader.size() + infoHeader.size(); i++) {
			XSSFCell tempCell = row0.createCell(i);
			tempCell.setCellValue(ExcelUtil.applyFontsub(teamsHeader.get(i - infoHeader.size()), font));
			tempCell.setCellStyle(style);
		}

		//增加1空列
		{
			XSSFCell tempCell = row0.createCell(teamsHeader.size() + infoHeader.size());
			tempCell.setCellValue("");
			tempCell.setCellStyle(style);
		}
		//处理检测项信息表头
		for (int i = teamsHeader.size() + infoHeader.size() + 1; i < infoHeader.size() + 1 + teamsHeader.size() * 3; i++) {
			String header = teamsHeader.get((i - teamsHeader.size() - infoHeader.size() - 1) / 2);
			XSSFCell tempCell1 = row0.createCell(i);
			tempCell1.setCellStyle(style);
			tempCell1.setCellValue(ExcelUtil.applyFontsub(header, font));
			XSSFCell tempCell2 = row0.createCell(++i);
			tempCell2.setCellValue(ExcelUtil.applyFontsub(header, font));
			tempCell2.setCellStyle(style);
			//合并表头
			sheet.addMergedRegion(new CellRangeAddress(0, 0, i - 1, i));
		}
		//创建第二行
		XSSFRow row1 = sheet.createRow(rownum++);
		//处理委托人信息表头
		for (int i = 0; i < infoHeader.size(); i++) {
			XSSFCell tempCell = row1.createCell(i);
			tempCell.setCellValue("");
			tempCell.setCellStyle(style);
			//合并表头
			sheet.addMergedRegion(new CellRangeAddress(0, 1, i, i));
		}
		//处理检测项总产值表头
		for (int i = infoHeader.size(); i < teamsHeader.size() + infoHeader.size(); i++) {
			XSSFCell tempCell = row1.createCell(i);
			tempCell.setCellValue("计算产值");
			tempCell.setCellStyle(style);
		}
		//增加1空列
		{
			XSSFCell tempCell = row0.createCell(teamsHeader.size() + infoHeader.size());
			tempCell.setCellValue("");
			tempCell.setCellStyle(style);
		}
		//处理检测项信息表头
		for (int i = teamsHeader.size() + 1 + infoHeader.size(); i < infoHeader.size() + 1 + teamsHeader.size() * 3; i++) {
			XSSFCell tempCell1 = row1.createCell(i);
			tempCell1.setCellValue("单价");
			tempCell1.setCellStyle(style);
			XSSFCell tempCell2 = row1.createCell(++i);
			tempCell2.setCellValue("数量");
			tempCell2.setCellStyle(style);
		}
	}

	/**
	 * 查看样品流转信息
	 *
	 * @param entrustId
	 * @return
	 */
	@Override
	public BaseResponse<Map<String, Object>> getSampleFlowList(Integer entrustId) {
		if (entrustId == null) {
			return BaseResponse.errorMsg("参数错误");
		}
		Map<String, Object> map = new HashMap<>();
		Entrust entrust = entrustMapper.selectById(entrustId);
		if (entrust == null) {
			return BaseResponse.errorMsg("信息错误");
		}
		List<SampleFlowVo> list = distributionMapper.getSampleFlowList(entrustId);
		map.put("list", list);
		map.put("entrust", entrust);
		return BaseResponse.okData(map);
	}


	/**
	 * 导出样品标签
	 *
	 * @param entrustId
	 * @return
	 */
	public void exportSampleLabel(Integer entrustId,
	                              String fileName, HttpServletResponse response) {

		EntrustVo entrustVo = entrustMapper.getDetail(entrustId);
		List<Sample> list = sampleMapper.getSampleCheckList(entrustId);
		//将map集合的key值进行排序
		Map<String, List<Sample>> groupByCementCode = list.stream().collect(Collectors.groupingBy(Sample::getCementCode, Collectors.toList()))
				.entrySet().stream().sorted(Map.Entry.comparingByKey()).collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue, (oldVal, newVal) -> oldVal, LinkedHashMap::new));

		//开始填充数据。首先填充表头
		XSSFWorkbook wb = new XSSFWorkbook();
		XSSFSheet sheet = wb.createSheet();

		sheet.setDefaultColumnWidth(20);
		XSSFCellStyle style = wb.createCellStyle();
		XSSFFont font = wb.createFont();
		//水平居中
		style.setAlignment(HorizontalAlignment.CENTER);

		//垂直居中
		style.setVerticalAlignment(VerticalAlignment.CENTER);

		//字体加粗
		font.setFontHeightInPoints((short) 13);
		font.setBold(true);
		style.setFont(font);

		//自动换行
		style.setWrapText(true);

		int coloum = 0;
		for (Map.Entry<String, List<Sample>> entry : groupByCementCode.entrySet()) {
			List<Sample> sampleList = entry.getValue();
			int rownum = 0;  //记录当前插入数据的行数
			if (coloum == 0) {
				for (int i = 0; i < sampleList.size(); i++) {
					rownum = i * 4;
					XSSFRow row1 = sheet.createRow(rownum);
					row1.setHeightInPoints(46.5f);
					XSSFCell tempCell_1 = row1.createCell(coloum);
					tempCell_1.setCellValue(entrustVo.getProjectName());
					tempCell_1.setCellStyle(style);

					XSSFRow row2 = sheet.createRow(rownum + 1);
					row2.setHeightInPoints(19.5f);
					XSSFCell tempCell_2 = row2.createCell(coloum);
					tempCell_2.setCellValue(entrustVo.getProjectCode());
					tempCell_2.setCellStyle(style);

					XSSFRow row3 = sheet.createRow(rownum + 2);
					row3.setHeightInPoints(19.5f);
					XSSFCell tempCell_3 = row3.createCell(coloum);
					tempCell_3.setCellValue(sampleList.get(i).getName());
					tempCell_3.setCellStyle(style);

					XSSFRow row4 = sheet.createRow(rownum + 3);
					row4.setHeightInPoints(19.5f);
					XSSFCell tempCell_4 = row4.createCell(coloum);
					tempCell_4.setCellValue(sampleList.get(i).getParallelCode());
					tempCell_4.setCellStyle(style);
				}
			} else {
				for (int i = 0; i < sampleList.size(); i++) {
					rownum = i * 4;
					XSSFRow row1 = sheet.getRow(rownum) == null ? sheet.createRow(rownum) : sheet.getRow(rownum);
					row1.setHeightInPoints(46.5f);
					XSSFCell tempCell_1 = row1.createCell(coloum);
					tempCell_1.setCellValue(entrustVo.getProjectName());
					tempCell_1.setCellStyle(style);

					XSSFRow row2 = sheet.getRow(rownum + 1) == null ? sheet.createRow(rownum + 1) : sheet.getRow(rownum + 1);
					row2.setHeightInPoints(19.5f);
					XSSFCell tempCell_2 = row2.createCell(coloum);
					tempCell_2.setCellValue(entrustVo.getProjectCode());
					tempCell_2.setCellStyle(style);

					XSSFRow row3 = sheet.getRow(rownum + 2) == null ? sheet.createRow(rownum + 2) : sheet.getRow(rownum + 2);
					row3.setHeightInPoints(19.5f);
					XSSFCell tempCell_3 = row3.createCell(coloum);
					tempCell_3.setCellValue(sampleList.get(i).getName());
					tempCell_3.setCellStyle(style);

					XSSFRow row4 = sheet.getRow(rownum + 3) == null ? sheet.createRow(rownum + 3) : sheet.getRow(rownum + 3);
					row4.setHeightInPoints(19.5f);
					XSSFCell tempCell_4 = row4.createCell(coloum);
					tempCell_4.setCellValue(sampleList.get(i).getParallelCode());
					tempCell_4.setCellStyle(style);
				}

			}
			coloum = coloum + 1;
		}

		ExcelUtil.excelExportNew(fileName == null || fileName.trim().length() <= 0 ? "样品标签" : fileName, wb, response);

	}

	/**
	 * 导出-委托单校核详情
	 *
	 * @param entrustId
	 * @param fileName
	 * @param response
	 */
	@Override
	public void exportCheckDetail(Integer entrustId,
	                              String fileName, HttpServletResponse response) {

		EntrustVo entrustVo = entrustMapper.getDetail(entrustId);
		if (entrustVo == null) {
			return;
		}
		//1.首先查询所有可见的检测项
		QueryWrapper<Team> teamQueryWrapper = new QueryWrapper<>();
		teamQueryWrapper.eq("is_display", 1);
		teamQueryWrapper.eq("status", 1);
		teamQueryWrapper.orderByAsc("group_id", "sort_no");
		List<Team> teamList = teamService.list(teamQueryWrapper);

		//2只看主样  对应的最终结果值
		List<Sample> sampleList = sampleMapper.getCheckSampleList(entrustId);

		//4.查询所有的派发检测项
		QueryWrapper<SampleDistribution> sdqueryWrapper = new QueryWrapper<>();
		sdqueryWrapper.eq("entrust_id", entrustId);
		sdqueryWrapper.in("sample_id", sampleList.stream().map(Sample::getId).collect(Collectors.toList()));
		List<SampleDistribution> sampleDistributions = sampleDistributionService.list(sdqueryWrapper);

		//5.分组并统计 检测项id   list
		Map<Integer, List<SampleDistribution>> groupByTeamId = sampleDistributions.stream().collect(Collectors.groupingBy(SampleDistribution::getTeamId, Collectors.toList()));

		//6.开始填充数据。首先填充表头
		XSSFWorkbook wb = new XSSFWorkbook();
		XSSFSheet sheet = wb.createSheet();
		XSSFCellStyle style = wb.createCellStyle();
		XSSFFont font = wb.createFont();
		//水平居中
		style.setAlignment(HorizontalAlignment.CENTER);
		//垂直居中
		style.setVerticalAlignment(VerticalAlignment.CENTER);
		//生成表头
		String[] in_fo = {
				"委托单号",
				"委托单位",
				"项目编号",
				"项目名称",
				"委托人",
				"样品名称",
				"本所编号",
				"是否加急",
		};
		List<String> infoHeader = Arrays.asList(in_fo);
		generatoronsignationWorkbook2(font, style, sheet, infoHeader, teamList.stream().map(Team::getName).collect(Collectors.toList()));
		//填充数据,从第二行开始
		int rownum = 1;

		if (sampleList != null && sampleList.size() > 0) {
			for (Sample sample : sampleList) {
				XSSFRow row0 = sheet.createRow(rownum++);
//                Map<Integer, List<SampleDistribution>> sampleDistribuGroup = groupByEntrustId.get(sample.getId()).stream().collect(Collectors.groupingBy(SampleDistribution::getTeamId, Collectors.toList()));
				//处理委托人信息表头
				for (int i = 0; i < infoHeader.size(); i++) {
					XSSFCell tempCell = row0.createCell(i);
					if (infoHeader.get(i).equals("委托单号")) {
						tempCell.setCellValue(entrustVo.getEntrustCode() == null ? "" : entrustVo.getEntrustCode());
						tempCell.setCellStyle(style);
					} else if (infoHeader.get(i).equals("委托单位")) {
						tempCell.setCellValue(entrustVo.getClientName() == null ? "" : entrustVo.getClientName());
						tempCell.setCellStyle(style);
					} else if (infoHeader.get(i).equals("项目编号")) {
						tempCell.setCellValue(entrustVo.getProjectCode() == null ? "" : entrustVo.getProjectCode());
						tempCell.setCellStyle(style);
					} else if (infoHeader.get(i).equals("项目名称")) {
						tempCell.setCellValue(entrustVo.getProjectName() == null ? "" : entrustVo.getProjectName());
						tempCell.setCellStyle(style);
					} else if (infoHeader.get(i).equals("委托人")) {
						tempCell.setCellValue(entrustVo.getUserName() == null ? "" : entrustVo.getUserName());
						tempCell.setCellStyle(style);
					} else if (infoHeader.get(i).equals("样品名称")) {
						tempCell.setCellValue(sample.getName() == null ? "" : sample.getName());
						tempCell.setCellStyle(style);
					} else if (infoHeader.get(i).equals("本所编号")) {
						tempCell.setCellValue(sample.getParallelCode() == null ? "" : sample.getParallelCode());
						tempCell.setCellStyle(style);
					} else if (infoHeader.get(i).equals("是否加急")) {
						tempCell.setCellValue(entrustVo.getIsUrgent() == 1 ? "加急" : "不加急");
						tempCell.setCellStyle(style);
					}
				}

				//处理检测项总产值表头
				for (int i = infoHeader.size(); i < teamList.size() + infoHeader.size(); i++) {
					XSSFCell tempCell = row0.createCell(i);
					Team team = teamList.get(i - infoHeader.size());

					List<SampleDistribution> sdList = groupByTeamId.get(team.getId());

					List<SampleDistribution> sdList2 = new ArrayList<>();
					String lastResult = "";
					if (sdList != null && sdList.size() > 0) {
						sdList2 = sdList.stream().filter(arg -> (arg.getSampleId().equals(sample.getId()))).collect(Collectors.toList());
						if(sdList2 != null && sdList2.size()>0){
							if (sdList2.get(0).getCheckId() != null) {
								SampleDistributionCheck disCheck = sampleDistributionCheckMapper.selectById(sdList2.get(0).getCheckId());
								lastResult = disCheck.getLastResult();
							} else {
								lastResult = sdList2.get(0).getDistributionResult();
							}
						}
					}
					tempCell.setCellValue(sdList2 == null ? "" : lastResult);
					tempCell.setCellStyle(style);
				}

			}
		}

		ExcelUtil.excelExportNew(fileName == null || fileName.trim().length() <= 0 ? "委托单校核" : fileName, wb, response);

	}

	private static void generatoronsignationWorkbook2(XSSFFont font, XSSFCellStyle style, XSSFSheet sheet, List<String> infoHeader, List<String> teamsHeader) {
		//委托项的表头
		int rownum = 0;
		//创建第一行
		XSSFRow row0 = sheet.createRow(rownum++);
		//处理委托人信息表头
		for (int i = 0; i < infoHeader.size(); i++) {
			XSSFCell tempCell = row0.createCell(i);
			tempCell.setCellValue(infoHeader.get(i));
			tempCell.setCellStyle(style);
		}
		//处理检测项总产值表头
		for (int i = infoHeader.size(); i < teamsHeader.size() + infoHeader.size(); i++) {
			XSSFCell tempCell = row0.createCell(i);
			tempCell.setCellValue(ExcelUtil.applyFontsub(teamsHeader.get(i - infoHeader.size()), font));
			tempCell.setCellStyle(style);
		}
	}

	/**
	 * 测试xml转word
	 * @param file
	 * @param response
	 */
	@Override
	public void testXml2Word(MultipartFile file, HttpServletResponse response){
		try{
			InputStream in = file.getInputStream();
			ExcelUtil.readData((FileInputStream)in);

		} catch (Exception e) {
			e.printStackTrace();
		}


	}







}
